@@ -59,7 +59,7 @@ private:
     QLineEdit * M_ball_pos_x;
     QLineEdit * M_ball_pos_y;
 
-    QLineEdit * M_synmetry_unum[11];
+    QLineEdit * M_symmetry_unum[11];
     QLineEdit * M_role_name[11];
 
     QLineEdit * M_pos_x[11];
