@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -38,8 +38,9 @@
 
 #include <iostream>
 #include <cstring>
-#include <cmath>
 #include <cctype>
+#include <climits>
+#include <cmath>
 #include <cstdlib>
 #include <cstdio>
 
@@ -109,6 +110,7 @@ DebugClientParser::parse( const char * msg,
                 << std::endl;
             break;
         case 2:
+        case 3:
             msg += n_read;
             result = parseV2( msg, ptr );
             break;
@@ -285,6 +287,24 @@ DebugClientParser::parseV2( const char * msg,
                 return false;
             }
         }
+        else if ( ! std::strcmp( id, "say" ) )
+        {
+            if ( ( n_read = parseSayMessage( msg, debug_view ) ) == 0 )
+            {
+                std::cerr << "DebugClientParser::parse(). "
+                          << "Error parseSayMessage" << std::endl;
+                return false;
+            }
+        }
+        else if ( ! std::strcmp( id, "hear" ) )
+        {
+            if ( ( n_read = parseHearMessage( msg, debug_view ) ) == 0 )
+            {
+                std::cerr << "DebugClientParser::parse(). "
+                          << "Error parseHearMessage" << std::endl;
+                return false;
+            }
+        }
         else if ( ! std::strcmp( id, "message" ) )
         {
             // misc. message
@@ -329,13 +349,13 @@ DebugClientParser::parseV2( const char * msg,
 */
 int
 DebugClientParser::parseTime( const char * tok,
-                              DebugViewPtr & debug_view )
+                              DebugViewPtr & )
 {
     int n_read = 0;
 
-    long cycle = 0;
+    int cycle = 0;
     if ( std::sscanf( tok,
-                      " ( time %ld ) %n",
+                      " ( time %d ) %n",
                       &cycle,
                       &n_read ) != 1
          || n_read == 0 )
@@ -369,9 +389,9 @@ DebugClientParser::parseBall( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 'b'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 'b'
 
     const double x = std::strtod( tok, &next );
     if ( x == -HUGE_VAL || x == HUGE_VAL ) return 0;
@@ -380,7 +400,7 @@ DebugClientParser::parseBall( const char * tok,
     if ( y == -HUGE_VAL || y == HUGE_VAL ) return 0;
     tok = next;
 
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
 
     double vx = HUGE_VAL;
     double vy = HUGE_VAL;
@@ -395,7 +415,7 @@ DebugClientParser::parseBall( const char * tok,
         if ( vy == -HUGE_VAL || vy == HUGE_VAL ) return 0;
         tok = next;
 
-        while ( &tok != '\0' && *tok == ' ' ) ++tok;
+        while ( *tok != '\0' && *tok == ' ' ) ++tok;
     }
 
     char comment[256+1];
@@ -458,11 +478,11 @@ DebugClientParser::parseSelf( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 's'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 's'
 
-    while ( &tok != '\0' && *tok == ' ' ) ++tok; // goto <side>
+    while ( *tok != '\0' && *tok == ' ' ) ++tok; // goto <side>
     const char side = *tok;
     if ( side != 'l' && side != 'r' )
     {
@@ -679,9 +699,9 @@ DebugClientParser::parseLine( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 'line'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 'line'
 
     const double x1 = std::strtod( tok, &next );
     if ( x1 == -HUGE_VAL || x1 == HUGE_VAL ) return 0;
@@ -726,9 +746,9 @@ DebugClientParser::parseTriangle( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 'tri'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 'tri'
 
     const double x1 = std::strtod( tok, &next );
     if ( x1 == -HUGE_VAL || x1 == HUGE_VAL ) return 0;
@@ -774,9 +794,9 @@ DebugClientParser::parseRectangle( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 'rect'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 'rect'
 
     const double x1 = std::strtod( tok, &next );
     if ( x1 == -HUGE_VAL || x1 == HUGE_VAL ) return 0;
@@ -815,9 +835,9 @@ DebugClientParser::parseCircle( const char * tok,
     const char * start = tok;
     char * next;
 
-    while ( &tok != '\0' && *tok == '(' ) ++tok;
-    while ( &tok != '\0' && *tok == ' ' ) ++tok;
-    while ( &tok != '\0' && *tok != ' ' ) ++tok; // skip 'rect'
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' ) ++tok; // skip 'rect'
 
     const double x = std::strtod( tok, &next );
     if ( x == -HUGE_VAL || x == HUGE_VAL ) return 0;
@@ -916,11 +936,11 @@ DebugClientParser::parseMessage( const char * tok,
 {
     int n_read = 0;
 
-    char message[256+1];
+    char message[1024];
     std::strcpy( message, "" );
     if ( std::sscanf( tok,
-                      " ( message \"%256[^\"]\" ) %n" ,
-                      message ,
+                      " ( message \"%1023[^\"]\" ) %n" ,
+                      message,
                       &n_read ) != 1
          || n_read == 0 )
     {
@@ -935,5 +955,86 @@ DebugClientParser::parseMessage( const char * tok,
 #endif
     debug_view->setMessage( message );
 
+//     if ( std::strstr( message, "Say[" ) != NULL )
+//     {
+//         debug_view->setHasSayMessage( true );
+//     }
+
     return n_read;
 }
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+DebugClientParser::parseSayMessage( const char * tok,
+                                    DebugViewPtr & debug_view )
+{
+    int n_read = 0;
+
+    char message[1024];
+    std::strcpy( message, "" );
+    if ( std::sscanf( tok,
+                      " ( say \"%1023[^\"]\" ) %n" ,
+                      message,
+                      &n_read ) != 1
+         || n_read == 0 )
+    {
+        std::cout << "DebugClientParser::parseSayMessage. Failed. ["
+                  << std::string( tok, 16 )
+                 << "]"  << std::endl;
+        return 0;
+    }
+
+    debug_view->setSayMessage( message );
+
+    return n_read;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+int
+DebugClientParser::parseHearMessage( const char * tok,
+                                     DebugViewPtr & debug_view )
+{
+    const char * start = tok;
+
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok == '(' ) ++tok;
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    while ( *tok != '\0' && *tok != ' ' && *tok != ')' ) ++tok;  // skip 'hear'
+
+    while ( *tok != '\0' )
+    {
+        if ( *tok == ')' )
+        {
+            ++tok;
+            break;
+        }
+
+        int n_read = 0;
+        int unum = -1;
+        char msg[256];
+
+        if ( std::sscanf( tok, " ( %d \"%255[^\"]\" ) %n ",
+                          &unum, msg, &n_read ) != 2 )
+        {
+            std::cerr << "DebugClientParser::parseHearMessage()"
+                      << " Illegal message [" << std::string( start, 16 ) << "]"
+                      << std::endl;
+            return 0;
+        }
+        tok += n_read;
+
+        debug_view->addHearMessage( unum, msg );
+
+        while ( *tok != '\0' && *tok == ' ' ) ++tok;
+    }
+
+    while ( *tok != '\0' && *tok == ' ' ) ++tok;
+
+    return tok - start;
+}
