@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -138,25 +138,23 @@ DebugLogHolder::setDir( const std::string & team_name,
 
         std::streampos pos = 0;
         std::string buf;
-        long prev_cycle = -1;
-        long cycle;
+        int prev_cycle = -1;
+        int cycle;
 
-        long n_line = 0;
+        int n_line = 0;
         std::ifstream & fin = M_player_data[idx].fin_;
         while ( std::getline( fin, buf ) )
         {
             ++n_line;
             // first element should be cycle value.
-            if ( std::sscanf( buf.c_str(),
-                              " %ld ",
-                              &cycle ) == 1 )
+            if ( std::sscanf( buf.c_str(), " %d ", &cycle ) == 1 )
             {
                 if ( cycle > prev_cycle )
                 {
-                    M_player_data[idx].cycle_map_
-                        .insert( std::make_pair( cycle, pos ) );
+                    M_player_data[idx].cycle_map_.insert( std::make_pair( cycle, pos ) );
                     prev_cycle = cycle;
                 }
+
                 if ( cycle < prev_cycle )
                 {
                     std::cerr << "***ERROR*** DebugLogHolder::setDir()"
@@ -186,7 +184,7 @@ DebugLogHolder::setDir( const std::string & team_name,
 */
 bool
 DebugLogHolder::getCycleData( const int unum,
-                              const long & cycle,
+                              const int cycle,
                               boost::shared_ptr< DebugLogData > data_ptr )
 {
     if ( unum < 1 || 11 < unum )
@@ -197,7 +195,7 @@ DebugLogHolder::getCycleData( const int unum,
     const int idx = unum - 1;
 
     // get the start stream position of 'cycle'
-    std::map< long, std::streampos >::iterator it
+    std::map< int, std::streampos >::iterator it
         = M_player_data[idx].cycle_map_.find( cycle );
 
     if ( it == M_player_data[idx].cycle_map_.end() )
@@ -219,7 +217,7 @@ DebugLogHolder::getCycleData( const int unum,
     int n_line = 0;
     while ( fin.getline( buf, 8192 ) )
     {
-        if ( ++n_line > 4096 )
+        if ( ++n_line > 8192 * 4 )
         {
             std::cerr << "***ERROR*** DebugLogHolder::getCycleData()"
                       << " Too many lines! cycle ="
@@ -228,12 +226,9 @@ DebugLogHolder::getCycleData( const int unum,
             break;
         }
 
-        long read_cycle = 0;
+        int read_cycle = 0;
         int n_read = 0;
-        if ( std::sscanf( buf,
-                          " %ld %n",
-                          &read_cycle,
-                          &n_read ) != 1
+        if ( std::sscanf( buf, " %d %n", &read_cycle, &n_read ) != 1
              || n_read == 0 )
         {
             // invalide cycle line.
@@ -265,7 +260,7 @@ DebugLogHolder::getCycleData( const int unum,
 */
 bool
 DebugLogHolder::seekData( const int unum,
-                          const long & cycle )
+                          const int cycle )
 {
     if ( unum < 1 || 11 < unum )
     {
@@ -294,10 +289,10 @@ DebugLogHolder::seekData( const int unum,
 
     bool found = false;
     std::streampos first_pos = 0;
-    long first_cycle = std::max( 1L, cycle - MAX_SPAN + 1 );
+    int first_cycle = std::max( 1, cycle - MAX_SPAN + 1 );
     for ( ; first_cycle <= cycle; ++first_cycle )
     {
-        std::map< long, std::streampos >::iterator it
+        std::map< int, std::streampos >::iterator it
             = M_player_data[idx].cycle_map_.find( first_cycle );
         if ( it != M_player_data[idx].cycle_map_.end() )
         {
@@ -323,10 +318,8 @@ DebugLogHolder::seekData( const int unum,
     // reset old data
     M_player_data[idx].data_list_.clear();
 
-    const long last_cycle = cycle + MAX_SPAN;
-    for ( long cyc = first_cycle;
-          cyc < last_cycle;
-          ++cyc )
+    const int last_cycle = cycle + MAX_SPAN;
+    for ( int cyc = first_cycle; cyc < last_cycle; ++cyc )
     {
         boost::shared_ptr< DebugLogData > data( new DebugLogData( cyc ) );
         if ( getCycleData( unum, cyc, data ) )
@@ -398,7 +391,7 @@ DebugLogHolder::incDataCycle( const int unum )
         return false;
     }
 
-    long new_cycle = M_player_data[idx].data_list_.back()->cycle() + 1;
+    int new_cycle = M_player_data[idx].data_list_.back()->cycle() + 1;
     boost::shared_ptr< DebugLogData > next_data_ptr( new DebugLogData( new_cycle ) );
 
     // max trial is 10 cycles
@@ -476,7 +469,7 @@ DebugLogHolder::decDataCycle( const int unum )
         return false;
     }
 
-    long new_cycle = M_player_data[idx].data_list_.front()->cycle() - 1;
+    int new_cycle = M_player_data[idx].data_list_.front()->cycle() - 1;
     if ( new_cycle <= 0 )
     {
         return false;
