@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -58,7 +58,7 @@ private:
         rcsc::SideID side_;
         DataList data_list_; // data set
         // key: cycle, value: streampos at cycle is changed
-        std::map< long, std::streampos > cycle_map_;
+        std::map< int, std::streampos > cycle_map_;
 
         void clear()
           {
@@ -74,7 +74,7 @@ private:
 
     boost::int32_t M_level; //!< current log level;
 
-    long M_cycle; //!< current forcus cycle
+    int M_cycle; //!< current forcus cycle
 
     PlayerData M_player_data[11];
 
@@ -103,7 +103,7 @@ public:
     bool decDataCycle( const int unum );
 
     bool seekData( const int unum,
-                   const long & cycle );
+                   const int cycle );
 
     boost::int32_t level() const
       {
@@ -123,8 +123,7 @@ public:
           }
       }
 
-    const
-    long & getCurrentCycle() const
+    int getCurrentCycle() const
       {
           return M_cycle;
       }
@@ -140,7 +139,7 @@ public:
 private:
 
     bool getCycleData( const int unum,
-                       const long & cycle,
+                       const int cycle,
                        boost::shared_ptr< DebugLogData > data_ptr );
 
 };
