@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -118,6 +118,11 @@ public:
           return M_flag & HAVE_STAMINA;
       }
 
+    bool hasStaminaCapacity() const
+      {
+          return M_data.stamina_capacity_ >= 0.0f;
+      }
+
     bool hasView() const
       {
           return M_flag & HAVE_VIEW;
@@ -261,6 +266,11 @@ public:
           return M_data.recovery_;
       }
 
+    double staminaCapacity() const
+      {
+          return M_data.stamina_capacity_;
+      }
+
     // command count
 
     int kickCount() const
