@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -42,23 +42,23 @@
 
 #include "view_holder.h"
 
+#include "options.h"
 #include "debug_client_parser.h"
-#include "app_config.h"
 
 #include <rcsc/common/player_param.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/rcg/serializer.h>
-#include <rcsc/rcg/factory.h>
 
 #include <iterator>
 #include <sstream>
 #include <fstream>
 #include <iostream>
 #include <cassert>
+#include <cstring>
 
 const std::size_t RCG_SIZE_LIMIT = 6000 * 8;
 
-const std::string DEFAULT_DEBUG_VIEW_EXTENSION = ".log";
+const std::string DEFAULT_DEBUG_VIEW_EXTENSION = ".dcl";
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -122,8 +122,7 @@ ViewHolder::clear()
 std::ostream &
 ViewHolder::saveRCG( std::ostream & os ) const
 {
-    rcsc::rcg::SerializerPtr serializer
-        = rcsc::rcg::make_serializer( logVersion() );
+    rcsc::rcg::Serializer::Ptr serializer = rcsc::rcg::Serializer::create( logVersion() );
 
     if ( ! serializer )
     {
@@ -265,7 +264,7 @@ bool
 ViewHolder::saveDebugView( const std::string & dir_path,
                            const std::string & team_name,
                            const int unum,
-                           const std::map< long, DebugViewCont > & data ) const
+                           const std::map< int, DebugViewCont > & data ) const
 {
     std::string filepath = dir_path;
     {
@@ -289,12 +288,11 @@ ViewHolder::saveDebugView( const std::string & dir_path,
         return false;
     }
 
-    const long last_cycle = M_latest_view_data->cycle();
+    const int last_cycle = M_latest_view_data->cycle();
 
-    for ( long i = 0; i < last_cycle; ++i )
+    for ( int i = 0; i < last_cycle; ++i )
     {
-        std::map< long, DebugViewCont >::const_iterator it
-            = data.find( i );
+        std::map< int, DebugViewCont >::const_iterator it = data.find( i );
         if ( it != data.end()
              && it->second[unum - 1] )
         {
@@ -477,7 +475,7 @@ ViewHolder::addShortShowInfo2( const rcsc::rcg::short_showinfo_t2 & show2 )
   push back new data
 */
 bool
-ViewHolder::addMsgInfo( const rcsc::rcg::Int16 board,
+ViewHolder::addMsgInfo( const rcsc::rcg::Int16,
                         const std::string & msg )
 {
     //std::cerr << "ViewHolder::addMsgInfo" << std::endl;
@@ -522,17 +520,17 @@ ViewHolder::addPlayMode( const char pmode )
         if ( playmode == rcsc::PM_PenaltyScore_Left
              || playmode == rcsc::PM_PenaltyMiss_Left )
         {
-            long cycle = ( M_latest_view_data
-                           ? M_latest_view_data->cycle()
-                           : 0 );
+            int cycle = ( M_latest_view_data
+                          ? M_latest_view_data->cycle()
+                          : 0 );
             M_penalty_scores_left.push_back( std::make_pair( cycle, playmode ) );
         }
         else if ( playmode == rcsc::PM_PenaltyScore_Right
                   || playmode == rcsc::PM_PenaltyMiss_Right )
         {
-            long cycle = ( M_latest_view_data
-                           ? M_latest_view_data->cycle()
-                           : 0 );
+            int cycle = ( M_latest_view_data
+                          ? M_latest_view_data->cycle()
+                          : 0 );
             M_penalty_scores_right.push_back( std::make_pair( cycle, playmode ) );
         }
     }
@@ -633,7 +631,7 @@ ViewHolder::addPlayerParam( const rcsc::rcg::player_params_t & param )
 
 */
 bool
-ViewHolder::addShow( const int time,
+ViewHolder::addShow( const int,
                      const rcsc::rcg::ShowInfoT & show )
 {
     if ( M_monitor_view_cont.size() > RCG_SIZE_LIMIT )
@@ -655,19 +653,18 @@ ViewHolder::addShow( const int time,
 
 */
 bool
-ViewHolder::addMsg( const int time,
-                    const int board,
-                    const char * msg )
+ViewHolder::addMsg( const int,
+                    const int,
+                    const std::string & msg )
 {
-    if ( ! std::strncmp( msg,
-                         "(team_graphic_",
-                         std::strlen( "(team_graphic_" ) ) )
+    if ( ! msg.compare( 0, std::strlen( "(team_graphic_" ),
+                        "(team_graphic_" ) )
     {
         return analyzeTeamGraphic( msg );
     }
-    else if ( ! std::strncmp( msg,
-                              "(change_player_type",
-                              std::strlen( "(change_player_type" ) ) )
+    else if ( ! msg.compare( 0, std::strlen( "(change_player_type" ),
+                             "(change_player_type" ) )
+
     {
         return true;
     }
@@ -682,7 +679,7 @@ ViewHolder::addMsg( const int time,
 
 */
 bool
-ViewHolder::addPlayMode( const int time,
+ViewHolder::addPlayMode( const int,
                          const rcsc::PlayMode pm )
 {
     //std::cerr << "ViewHolder::addPlayMode " << time << " " << pm << std::endl;
@@ -691,17 +688,17 @@ ViewHolder::addPlayMode( const int time,
         if ( pm == rcsc::PM_PenaltyScore_Left
              || pm == rcsc::PM_PenaltyMiss_Left )
         {
-            long cycle = ( M_latest_view_data
-                           ? M_latest_view_data->cycle()
-                           : 0 );
+            int cycle = ( M_latest_view_data
+                          ? M_latest_view_data->cycle()
+                          : 0 );
             M_penalty_scores_left.push_back( std::make_pair( cycle, pm ) );
         }
         else if ( pm == rcsc::PM_PenaltyScore_Right
                   || pm == rcsc::PM_PenaltyMiss_Right )
         {
-            long cycle = ( M_latest_view_data
-                           ? M_latest_view_data->cycle()
-                           : 0 );
+            int cycle = ( M_latest_view_data
+                          ? M_latest_view_data->cycle()
+                          : 0 );
             M_penalty_scores_right.push_back( std::make_pair( cycle, pm ) );
         }
     }
@@ -715,7 +712,7 @@ ViewHolder::addPlayMode( const int time,
 
 */
 bool
-ViewHolder::addTeam( const int time,
+ViewHolder::addTeam( const int,
                      const rcsc::rcg::TeamT & team_l,
                      const rcsc::rcg::TeamT & team_r )
 {
@@ -815,8 +812,8 @@ ViewHolder::addMonitorViewData( MonitorViewPtr ptr )
                     M_monitor_view_cont.pop_back();
                 }
             }
-            else if ( AppConfig::instance().monitorClientMode()
-                      && ! AppConfig::instance().timeShiftReplay() )
+            else if ( Options::instance().monitorClientMode()
+                      && ! Options::instance().timeShiftReplay() )
             {
                 M_monitor_view_cont.pop_back();
             }
@@ -846,12 +843,12 @@ ViewHolder::pushBackLatestViewData()
 
 */
 void
-ViewHolder::addDebugView( const long & cycle,
+ViewHolder::addDebugView( const int cycle,
                           char side,
                           const int unum,
                           DebugViewPtr debug_view )
 {
-    typedef std::map< long, DebugViewCont >::iterator IterType;
+    typedef std::map< int, DebugViewCont >::iterator IterType;
 
     if ( ! debug_view )
     {
@@ -865,9 +862,9 @@ ViewHolder::addDebugView( const long & cycle,
         return;
     }
 
-    std::map< long, DebugViewCont > * view_map = ( side == 'l'
-                                                   ? &M_left_debug_view
-                                                   : &M_right_debug_view );
+    std::map< int, DebugViewCont > * view_map = ( side == 'l'
+                                                  ? &M_left_debug_view
+                                                  : &M_right_debug_view );
 
     IterType it = view_map->find( cycle );
 
@@ -907,6 +904,23 @@ ViewHolder::getViewData( const std::size_t idx ) const
     return M_monitor_view_cont[idx];
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+struct TimeCmp {
+    bool operator()( const MonitorViewPtr & lhs,
+                     const int rhs )
+      {
+          return lhs->cycle() < rhs;
+      }
+};
+
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -914,41 +928,53 @@ ViewHolder::getViewData( const std::size_t idx ) const
 std::size_t
 ViewHolder::getIndexOf( const int cycle ) const
 {
-    MonitorViewCont::size_type len = M_monitor_view_cont.size();
-    MonitorViewCont::size_type half;
-
-    MonitorViewCont::const_iterator first = M_monitor_view_cont.begin();
-    MonitorViewCont::const_iterator middle;
-
-    // lower bound algorithm
-    while ( len > 0 )
+    MonitorViewCont::const_iterator it
+        = std::lower_bound( M_monitor_view_cont.begin(),
+                            M_monitor_view_cont.end(),
+                            cycle,
+                            TimeCmp() );
+    if ( it == M_monitor_view_cont.end() )
     {
-        half = len >> 1;
-        middle = first;
-        middle += half;
-        if ( (*middle)->cycle() < cycle )
-        {
-            first = middle;
-            ++first;
-            len = len - half - 1;
-        }
-        else
-        {
-            len = half;
-        }
-    }
-
-    if ( first == M_monitor_view_cont.end() )
-    {
-        if ( cycle > 0
-             && ! M_monitor_view_cont.empty() )
-        {
-            return M_monitor_view_cont.size() - 1;
-        }
         return 0;
     }
 
-    return std::distance( M_monitor_view_cont.begin(), first );
+    return std::distance( M_monitor_view_cont.begin(), it );
+
+//     MonitorViewCont::size_type len = M_monitor_view_cont.size();
+//     MonitorViewCont::size_type half;
+
+//     MonitorViewCont::const_iterator first = M_monitor_view_cont.begin();
+//     MonitorViewCont::const_iterator middle;
+
+//     // lower bound algorithm
+//     while ( len > 0 )
+//     {
+//         half = len >> 1;
+//         middle = first;
+//         middle += half;
+//         if ( (*middle)->cycle() < cycle )
+//         {
+//             first = middle;
+//             ++first;
+//             len = len - half - 1;
+//         }
+//         else
+//         {
+//             len = half;
+//         }
+//     }
+
+//     if ( first == M_monitor_view_cont.end() )
+//     {
+//         if ( cycle > 0
+//              && ! M_monitor_view_cont.empty() )
+//         {
+//             return M_monitor_view_cont.size() - 1;
+//         }
+//         return 0;
+//     }
+
+//     return std::distance( M_monitor_view_cont.begin(), first );
 }
 
 /*-------------------------------------------------------------------*/
@@ -994,7 +1020,7 @@ ViewHolder::analyzeTeamGraphic( const std::string & msg )
 
     if ( side == 'l' )
     {
-        //std::cerr << "recv: team_graphic_l (" << x << ',' << y << ')'
+        //std::cerr << "recv team_graphic_l (" << x << ',' << y << ')'
         //          << std::endl;
         return M_team_graphic_left.parse( msg.c_str() );
     }
