@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,11 +37,11 @@
 
 #include "debug_message_window.h"
 
+#include "options.h"
 #include "main_data.h"
 #include "debug_log_holder.h"
 #include "debug_log_data.h"
 #include "debug_log_dir_dialog.h"
-#include "app_config.h"
 
 #include <cassert>
 #include <iostream>
@@ -49,7 +49,8 @@
 #include "xpm/debug_view_all.xpm"
 #include "xpm/debug_view_self.xpm"
 #include "xpm/debug_view_ball.xpm"
-#include "xpm/debug_view_player.xpm"
+#include "xpm/debug_view_teammate.xpm"
+#include "xpm/debug_view_opponent.xpm"
 #include "xpm/debug_view_comment.xpm"
 #include "xpm/debug_view_line.xpm"
 #include "xpm/debug_view_target.xpm"
@@ -58,6 +59,7 @@
 #include "xpm/debug_log_view.xpm"
 
 #include "xpm/open_dir.xpm"
+#include "xpm/clear.xpm"
 
 #include "xpm/sync.xpm"
 #include "xpm/logplayer_one_step_back.xpm"
@@ -79,6 +81,22 @@
 #include "xpm/num_14.xpm"
 #include "xpm/num_15.xpm"
 #include "xpm/num_16.xpm"
+#include "xpm/num_17.xpm"
+#include "xpm/num_18.xpm"
+#include "xpm/num_19.xpm"
+#include "xpm/num_20.xpm"
+#include "xpm/num_21.xpm"
+#include "xpm/num_22.xpm"
+#include "xpm/num_23.xpm"
+#include "xpm/num_24.xpm"
+#include "xpm/num_25.xpm"
+#include "xpm/num_26.xpm"
+#include "xpm/num_27.xpm"
+#include "xpm/num_28.xpm"
+#include "xpm/num_29.xpm"
+#include "xpm/num_30.xpm"
+#include "xpm/num_31.xpm"
+#include "xpm/num_32.xpm"
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -92,22 +110,27 @@ DebugMessageWindow::DebugMessageWindow( QWidget * parent,
     //this->setWindowTitle( tr( "Debug Message" ) ); // qt4
     this->setCaption( tr( "Debug Message" ) ); // qt3
 
-    M_debug_text[0] = tr( "System" );
-    M_debug_text[1] = tr( "Sensor" );
-    M_debug_text[2] = tr( "WorldModel" );
-    M_debug_text[3] = tr( "BasicAction" );
-    M_debug_text[4] = tr( "Intercept" );
-    M_debug_text[5] = tr( "Kick" );
-    M_debug_text[6] = tr( "Dribble" );
-    M_debug_text[7] = tr( "Pass" );
-    M_debug_text[8] = tr( "Cross" );
-    M_debug_text[9] = tr( "Shoot" );
-    M_debug_text[10] = tr( "Clear" );
-    M_debug_text[11] = tr( "Team" );
-    M_debug_text[12] = tr( "Role" );
-    //M_debug_text[13] = tr( "" );
-    //M_debug_text[14] = tr( "" );
-    //M_debug_text[15] = tr( "" );
+    //this->setUsesTextLabel( true );
+
+    int i = -1;
+    M_debug_text[++i] = tr( "System" );
+    M_debug_text[++i] = tr( "Sensor" );
+    M_debug_text[++i] = tr( "WorldMode" );
+    M_debug_text[++i] = tr( "BasicAction" );
+    M_debug_text[++i] = tr( "Intercept" );
+    M_debug_text[++i] = tr( "Kick" );
+    M_debug_text[++i] = tr( "Hold" );
+    M_debug_text[++i] = tr( "Dribble" );
+    M_debug_text[++i] = tr( "Pass" );
+    M_debug_text[++i] = tr( "Cross" );
+    M_debug_text[++i] = tr( "Shoot" );
+    M_debug_text[++i] = tr( "Clear" );
+    M_debug_text[++i] = tr( "Block" );
+    M_debug_text[++i] = tr( "Mark" );
+    M_debug_text[++i] = tr( "Role" );
+    M_debug_text[++i] = tr( "Team" );
+    M_debug_text[++i] = tr( "Analyzer" );
+    M_debug_text[++i] = tr( "Recursive" );
 
     createActions();
     createWindows();
@@ -115,6 +138,7 @@ DebugMessageWindow::DebugMessageWindow( QWidget * parent,
     readSettings();
 
     this->resize( 640, 480 );
+    //this->setWFlags( this->getWFlags() | Qt::WStyle_StaysOnTop | Qt::WX11BypassWM );
 }
 
 /*-------------------------------------------------------------------*/
@@ -146,38 +170,81 @@ DebugMessageWindow::readSettings()
         bool enable = false;
 
         enable = settings.readBoolEntry( tr( "debug_view_all" ), true, &ok );
-        if ( ok ) M_show_debug_view_all_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_all_act->setOn( enable );
+            toggleShowDebugViewAll( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_view_self" ), true, &ok );
-        if ( ok ) M_show_debug_view_self_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_self_act->setOn( enable );
+            toggleShowDebugViewSelf( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_view_ball" ), true, &ok );
-        if ( ok ) M_show_debug_view_ball_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_ball_act->setOn( enable );
+            toggleShowDebugViewBall( enable );
+        }
 
-        enable = settings.readBoolEntry( tr( "debug_view_players" ), true, &ok );
-        if ( ok ) M_show_debug_view_players_act->setOn( enable );
+        enable = settings.readBoolEntry( tr( "debug_view_teammates" ), true, &ok );
+        if ( ok )
+        {
+            M_show_debug_view_teammates_act->setOn( enable );
+            toggleShowDebugViewTeammates( enable );
+        }
+
+        enable = settings.readBoolEntry( tr( "debug_view_opponents" ), true, &ok );
+        if ( ok )
+        {
+            M_show_debug_view_opponents_act->setOn( enable );
+            toggleShowDebugViewOpponents( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_view_comment" ), true, &ok );
-        if ( ok ) M_show_debug_view_comment_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_comment_act->setOn( enable );
+            toggleShowDebugViewComment( enable );
+        }
 
-        enable = settings.readBoolEntry( tr( "debug_view_figures" ), true, &ok );
-        if ( ok ) M_show_debug_view_figure_act->setOn( enable );
+        enable = settings.readBoolEntry( tr( "debug_view_shape" ), true, &ok );
+        if ( ok )
+        {
+            M_show_debug_view_shape_act->setOn( enable );
+            toggleShowDebugViewShape( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_view_target" ), true, &ok );
-        if ( ok ) M_show_debug_view_target_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_target_act->setOn( enable );
+            toggleShowDebugViewTarget( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_view_message" ), true, &ok );
-        if ( ok ) M_show_debug_view_message_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_view_message_act->setOn( enable );
+            toggleShowDebugViewMessage( enable );
+        }
 
         enable = settings.readBoolEntry( tr( "debug_log_objects" ), true, &ok );
-        if ( ok ) M_show_debug_log_objects_act->setOn( enable );
+        if ( ok )
+        {
+            M_show_debug_log_objects_act->setOn( enable );
+            toggleShowDebugLogObjects( enable );
+        }
     }
 
     // read debug level switch status
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
         char key[32];
-        std::snprintf( key, 32, "debug%02d", i + 1 );
+        snprintf( key, 32, "debug%02d", i + 1 );
 
         //QVariant val = settings.value( QString::fromAscii( key ) );
         bool ok = false;
@@ -198,10 +265,10 @@ DebugMessageWindow::readSettings()
     }
 
     // read debug level text string
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
         char key[32];
-        std::snprintf( key, 32, "debug_text_%02d", i + 1 );
+        snprintf( key, 32, "debug_text_%02d", i + 1 );
 
         bool ok = false;
         QString val = settings.readEntry( QString::fromAscii( key ),
@@ -210,9 +277,12 @@ DebugMessageWindow::readSettings()
         if ( ok )
         {
             M_debug_text[i] = val;
-            QString text = tr( "Level %1 " ).arg( i + 1 ) + val;
-            M_debug_level_act[i]->setText( text );
-            M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + text );
+            //QString text = tr( "Level %1 " ).arg( i + 1 ) + val;
+            //M_debug_level_act[i]->setText( text );
+            //M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + text );
+            M_debug_level_act[i]->setText( M_debug_text[i] );
+            M_debug_level_act[i]->setMenuText( M_debug_text[i] );
+            M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + M_debug_text[i] );
         }
     }
 
@@ -236,30 +306,31 @@ DebugMessageWindow::saveSettings()
     settings.writeEntry( tr( "debug_view_all" ),  M_show_debug_view_all_act->isOn() );
     settings.writeEntry( tr( "debug_view_self" ), M_show_debug_view_self_act->isOn() );
     settings.writeEntry( tr( "debug_view_ball" ), M_show_debug_view_ball_act->isOn() );
-    settings.writeEntry( tr( "debug_view_players" ), M_show_debug_view_players_act->isOn() );
+    settings.writeEntry( tr( "debug_view_teammates" ), M_show_debug_view_teammates_act->isOn() );
+    settings.writeEntry( tr( "debug_view_opponents" ), M_show_debug_view_opponents_act->isOn() );
     settings.writeEntry( tr( "debug_view_comment" ), M_show_debug_view_comment_act->isOn() );
-    settings.writeEntry( tr( "debug_view_figures" ), M_show_debug_view_figure_act->isOn() );
+    settings.writeEntry( tr( "debug_view_shape" ), M_show_debug_view_shape_act->isOn() );
     settings.writeEntry( tr( "debug_view_target" ), M_show_debug_view_target_act->isOn() );
     settings.writeEntry( tr( "debug_view_message" ), M_show_debug_view_message_act->isOn() );
     settings.writeEntry( tr( "debug_log_objects" ), M_show_debug_log_objects_act->isOn() );
 
     // save debug level switches
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
         char key[32];
-        std::snprintf( key, 32, "debug%02d", i + 1 );
+        snprintf( key, 32, "debug%02d", i + 1 );
 
         settings.writeEntry( QString::fromAscii( key ),
                              M_debug_level_act[i]->isOn() );
     }
 
     // save debug level text
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
         if ( ! M_debug_text[i].isEmpty() )
         {
             char key[32];
-            std::snprintf( key, 16, "debug_text_%02d", i + 1 );
+            snprintf( key, 32, "debug_text_%02d", i + 1 );
             settings.writeEntry( QString::fromAscii( key ), M_debug_text[i] );
         }
     }
@@ -309,16 +380,27 @@ DebugMessageWindow::createActions()
     connect( M_show_debug_view_ball_act, SIGNAL( toggled( bool ) ),
              this, SLOT( toggleShowDebugViewBall( bool ) ) );
 
-    M_show_debug_view_players_act
-        = new QAction( QIconSet( QPixmap( debug_view_player_xpm ) ),
-                       tr( "Toggle debug view players" ),
+    M_show_debug_view_teammates_act
+        = new QAction( QIconSet( QPixmap( debug_view_teammate_xpm ) ),
+                       tr( "Toggle debug view teammates" ),
                        QKeySequence(),
                        this );
-    M_show_debug_view_players_act->setStatusTip( tr( "Show/Hide debug view players" ) );
-    M_show_debug_view_players_act->setToggleAction( true );
-    M_show_debug_view_players_act->setOn( true );
-    connect( M_show_debug_view_players_act, SIGNAL( toggled( bool ) ),
-             this, SLOT( toggleShowDebugViewPlayers( bool ) ) );
+    M_show_debug_view_teammates_act->setStatusTip( tr( "Show/Hide debug view teammates" ) );
+    M_show_debug_view_teammates_act->setToggleAction( true );
+    M_show_debug_view_teammates_act->setOn( true );
+    connect( M_show_debug_view_teammates_act, SIGNAL( toggled( bool ) ),
+             this, SLOT( toggleShowDebugViewTeammates( bool ) ) );
+
+    M_show_debug_view_opponents_act
+        = new QAction( QIconSet( QPixmap( debug_view_opponent_xpm ) ),
+                       tr( "Toggle debug view opponents" ),
+                       QKeySequence(),
+                       this );
+    M_show_debug_view_opponents_act->setStatusTip( tr( "Show/Hide debug view opponents" ) );
+    M_show_debug_view_opponents_act->setToggleAction( true );
+    M_show_debug_view_opponents_act->setOn( true );
+    connect( M_show_debug_view_opponents_act, SIGNAL( toggled( bool ) ),
+             this, SLOT( toggleShowDebugViewOpponents( bool ) ) );
 
     M_show_debug_view_comment_act
         = new QAction( QIconSet( QPixmap( debug_view_comment_xpm ) ),
@@ -331,16 +413,16 @@ DebugMessageWindow::createActions()
     connect( M_show_debug_view_comment_act, SIGNAL( toggled( bool ) ),
              this, SLOT( toggleShowDebugViewComment( bool ) ) );
 
-    M_show_debug_view_figure_act
+    M_show_debug_view_shape_act
         = new QAction( QIconSet( QPixmap( debug_view_line_xpm ) ),
-                       tr( "Toggle debug view figures" ),
+                       tr( "Toggle debug view shapes" ),
                        QKeySequence(),
                        this );
-    M_show_debug_view_figure_act->setStatusTip( tr( "Show/Hide debug view figures" ) );
-    M_show_debug_view_figure_act->setToggleAction( true );
-    M_show_debug_view_figure_act->setOn( true );
-    connect( M_show_debug_view_figure_act, SIGNAL( toggled( bool ) ),
-             this, SLOT( toggleShowDebugViewFigure( bool ) ) );
+    M_show_debug_view_shape_act->setStatusTip( tr( "Show/Hide debug view shapes" ) );
+    M_show_debug_view_shape_act->setToggleAction( true );
+    M_show_debug_view_shape_act->setOn( true );
+    connect( M_show_debug_view_shape_act, SIGNAL( toggled( bool ) ),
+             this, SLOT( toggleShowDebugViewShape( bool ) ) );
 
     M_show_debug_view_target_act
         = new QAction( QIconSet( QPixmap( debug_view_target_xpm ) ),
@@ -384,10 +466,18 @@ DebugMessageWindow::createActions()
                        tr( "Open log dir..." ),
                        QKeySequence(),
                        this );
-    M_open_debug_log_dir_act->setAccel( tr( "Ctrl+O" ) );
+    M_open_debug_log_dir_act->setAccel( Qt::CTRL + Qt::Key_O );
     M_open_debug_log_dir_act->setStatusTip( tr( "Open the debug log directory" ) );
     connect( M_open_debug_log_dir_act, SIGNAL( activated() ),
              this, SLOT( showDebugLogDirDialog() ) );
+    //
+    M_clear_data_act = new QAction( QIconSet( QPixmap( clear_xpm ) ),
+                                    tr( "Clear data" ),
+                                    QKeySequence(),
+                                    this );
+    M_clear_data_act->setStatusTip( tr( "Clear all data" ) );
+    connect( M_clear_data_act, SIGNAL( activated() ),
+             this, SLOT( clearAll() ) );
 
     //////////////////////////////////////////////
     // cycle control
@@ -396,7 +486,7 @@ DebugMessageWindow::createActions()
                        tr( "Sync" ),
                        QKeySequence(),
                        this );
-    M_sync_act->setAccel( tr( "Ctrl+S" ) );
+    M_sync_act->setAccel( Qt::CTRL + Qt::Key_S );
     M_sync_act->setStatusTip( tr( "Synchronize with field canvas" ) );
     connect( M_sync_act, SIGNAL( activated() ),
              this, SLOT( syncCycle() ) );
@@ -424,22 +514,42 @@ DebugMessageWindow::createActions()
     //////////////////////////////////////////////
     // debug level switches
 
-    char ** num_xpm[16] = { num_1_xpm, num_2_xpm, num_3_xpm, num_4_xpm,
-                            num_5_xpm, num_6_xpm, num_7_xpm, num_8_xpm,
-                            num_9_xpm, num_10_xpm, num_11_xpm, num_12_xpm,
-                            num_13_xpm, num_14_xpm, num_15_xpm, num_16_xpm };
+    const char ** num_xpm[] = { num_1_xpm, num_2_xpm, num_3_xpm, num_4_xpm,
+                                num_5_xpm, num_6_xpm, num_7_xpm, num_8_xpm,
+                                num_9_xpm, num_10_xpm, num_11_xpm, num_12_xpm,
+                                num_13_xpm, num_14_xpm, num_15_xpm, num_16_xpm,
+                                num_17_xpm, num_18_xpm, num_19_xpm, num_20_xpm,
+                                num_21_xpm, num_22_xpm, num_23_xpm, num_24_xpm,
+                                num_25_xpm, num_26_xpm, num_27_xpm, num_28_xpm,
+                                  num_29_xpm, num_30_xpm, num_31_xpm, num_32_xpm };
 
     QSignalMapper * debug_level_mapper = new QSignalMapper( this );
     connect( debug_level_mapper, SIGNAL( mapped( int ) ),
              this, SLOT( toggleDebugLevel( int ) ) );
 
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
-        QString menu_text = tr( "Level %1 " ).arg( i + 1 ) + M_debug_text[i];
-        M_debug_level_act[i] = new QAction( QIconSet( QPixmap( num_xpm[i] ) ),
-                                            menu_text,
-                                            QKeySequence(),
-                                            this ); //M_debug_level_act_group );
+        QString menu_text;
+        if ( M_debug_text[i].isEmpty() )
+        {
+            menu_text = tr( "Level %1 " ).arg( i + 1 );
+            M_debug_level_act[i] = new QAction( QIconSet( QPixmap( num_xpm[i] ) ),
+                                                menu_text,
+                                                QKeySequence(),
+                                                this ); //M_debug_level_act_group );
+            M_debug_level_act[i]->setText( menu_text );
+        }
+        else
+        {
+            menu_text = M_debug_text[i];
+            M_debug_level_act[i] = new QAction( QIconSet( QPixmap( num_xpm[i] ) ),
+                                                menu_text,
+                                                QKeySequence(),
+                                                this );
+            M_debug_level_act[i]->setText( M_debug_text[i] );
+            M_debug_level_act[i]->setMenuText( M_debug_text[i] );
+        }
+
         M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + menu_text );
         M_debug_level_act[i]->setToggleAction( true );
 
@@ -486,7 +596,7 @@ void
 DebugMessageWindow::createToolBar()
 {
     {
-        QToolBar * tbar = new QToolBar( this );
+        QToolBar * tbar = new QToolBar( tr( "Control" ), this );
 
         M_open_debug_log_dir_act->addTo( tbar );
 
@@ -507,7 +617,7 @@ DebugMessageWindow::createToolBar()
         // short cut
         {
             QAccel * accel = new QAccel( this );
-            accel->connectItem( accel->insertItem( tr( "Ctrl+F" ) ),
+            accel->connectItem( accel->insertItem( Qt::CTRL + Qt::Key_F ),
                                 M_find_box, SLOT( setFocus() ) );
         }
         //QToolTip::add( M_find_box, tr( "Find the matched string from the below window" ) );
@@ -528,14 +638,15 @@ DebugMessageWindow::createToolBar()
     }
 
     {
-        QToolBar * tbar = new QToolBar( this );
+        QToolBar * tbar = new QToolBar( tr( "Debug view & 1-16" ), this );
 
         M_show_debug_view_all_act->addTo( tbar );
         M_show_debug_view_self_act->addTo( tbar );
         M_show_debug_view_ball_act->addTo( tbar );
-        M_show_debug_view_players_act->addTo( tbar );
+        M_show_debug_view_teammates_act->addTo( tbar );
+        M_show_debug_view_opponents_act->addTo( tbar );
         M_show_debug_view_comment_act->addTo( tbar );
-        M_show_debug_view_figure_act->addTo( tbar );
+        M_show_debug_view_shape_act->addTo( tbar );
         M_show_debug_view_target_act->addTo( tbar );
         M_show_debug_view_message_act->addTo( tbar );
 
@@ -543,12 +654,31 @@ DebugMessageWindow::createToolBar()
 
         M_show_debug_log_objects_act->addTo( tbar );
 
-         for ( int i = 0; i < 16; ++i )
+        this->addDockWindow( tbar, Qt::DockTop );
+    }
+
+    const int max_visible_level = 18;
+    {
+        QToolBar * tbar = new QToolBar( tr( "Debug Level" ), this );
+
+         for ( int i = 0; i < max_visible_level; ++i )
          {
              M_debug_level_act[i]->addTo( tbar );
          }
 
-        this->addDockWindow( tbar, Qt::DockBottom );
+        this->addDockWindow( tbar, Qt::DockLeft );
+    }
+
+    {
+        QToolBar * tbar = new QToolBar( tr( "Debug Level Hidden" ), this );
+
+         for ( int i = max_visible_level; i < 32; ++i )
+         {
+             M_debug_level_act[i]->addTo( tbar );
+         }
+
+         this->addDockWindow( tbar, Qt::DockLeft, true );
+         tbar->hide();
     }
 
 }
@@ -561,7 +691,7 @@ void
 DebugMessageWindow::openDebugLogDir( const rcsc::SideID side,
                                      const std::string & dir_path )
 {
-    MonitorViewConstPtr ptr = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr ptr = M_main_data.getCurrentViewData();
     if ( ! ptr )
     {
         std::cerr << "No valid data!" << std::endl;
@@ -618,9 +748,9 @@ DebugMessageWindow::changeCurrentTab( QWidget * )
 void
 DebugMessageWindow::toggleShowDebugViewAll( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugView() != checked )
+    if ( Options::instance().showDebugView() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugView();
+        Options::instance().toggleShowDebugView();
         emit configured();
     }
 }
@@ -632,9 +762,9 @@ DebugMessageWindow::toggleShowDebugViewAll( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewSelf( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewSelf() != checked )
+    if ( Options::instance().showDebugViewSelf() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewSelf();
+        Options::instance().toggleShowDebugViewSelf();
         emit configured();
     }
 }
@@ -646,9 +776,9 @@ DebugMessageWindow::toggleShowDebugViewSelf( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewBall( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewBall() != checked )
+    if ( Options::instance().showDebugViewBall() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewBall();
+        Options::instance().toggleShowDebugViewBall();
         emit configured();
     }
 }
@@ -658,11 +788,25 @@ DebugMessageWindow::toggleShowDebugViewBall( bool checked )
 
 */
 void
-DebugMessageWindow::toggleShowDebugViewPlayers( bool checked )
+DebugMessageWindow::toggleShowDebugViewTeammates( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewPlayers() != checked )
+    if ( Options::instance().showDebugViewTeammates() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewPlayers();
+        Options::instance().toggleShowDebugViewTeammates();
+        emit configured();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+DebugMessageWindow::toggleShowDebugViewOpponents( bool checked )
+{
+    if ( Options::instance().showDebugViewOpponents() != checked )
+    {
+        Options::instance().toggleShowDebugViewOpponents();
         emit configured();
     }
 }
@@ -674,9 +818,9 @@ DebugMessageWindow::toggleShowDebugViewPlayers( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewComment( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewComment() != checked )
+    if ( Options::instance().showDebugViewComment() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewComment();
+        Options::instance().toggleShowDebugViewComment();
         emit configured();
     }
 }
@@ -686,11 +830,11 @@ DebugMessageWindow::toggleShowDebugViewComment( bool checked )
 
 */
 void
-DebugMessageWindow::toggleShowDebugViewFigure( bool checked )
+DebugMessageWindow::toggleShowDebugViewShape( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewFigure() != checked )
+    if ( Options::instance().showDebugViewShape() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewFigure();
+        Options::instance().toggleShowDebugViewShape();
         emit configured();
     }
 }
@@ -702,9 +846,9 @@ DebugMessageWindow::toggleShowDebugViewFigure( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewTarget( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewTarget() != checked )
+    if ( Options::instance().showDebugViewTarget() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewTarget();
+        Options::instance().toggleShowDebugViewTarget();
         emit configured();
     }
 }
@@ -716,9 +860,9 @@ DebugMessageWindow::toggleShowDebugViewTarget( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewMessage( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewMessage() != checked )
+    if ( Options::instance().showDebugViewMessage() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewMessage();
+        Options::instance().toggleShowDebugViewMessage();
         emit configured();
     }
 }
@@ -730,9 +874,9 @@ DebugMessageWindow::toggleShowDebugViewMessage( bool checked )
 void
 DebugMessageWindow::toggleShowDebugLogObjects( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugLogObjects() != checked )
+    if ( Options::instance().showDebugLogObjects() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugLogObjects();
+        Options::instance().toggleShowDebugLogObjects();
         emit configured();
     }
 }
@@ -761,7 +905,7 @@ DebugMessageWindow::showDebugLogDirDialog()
               << "  debug log dir = [" << dir_path_std << "]"
               << std::endl;
 
-    AppConfig::instance().setDebugLogDir( dir_path_std );
+    Options::instance().setDebugLogDir( dir_path_std );
 
     openDebugLogDir( side, dir_path_std );
 }
@@ -773,7 +917,7 @@ DebugMessageWindow::showDebugLogDirDialog()
 void
 DebugMessageWindow::syncCycle()
 {
-    MonitorViewConstPtr ptr = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr ptr = M_main_data.getCurrentViewData();
     if ( ! ptr )
     {
         //std::cerr << "sync debug cycle. no view data." << std::endl;
@@ -875,13 +1019,13 @@ DebugMessageWindow::toggleDebugLevel( int level )
 {
     //M_level ^= DebugLogData::LEVEL_1; // reverse only 1 bit
 
-    if ( level < 1 || 16 < level )
+    if ( level < 1 || 32 < level )
     {
         std::cerr << "Invalid level value " << level << std::endl;
         return;
     }
 
-    int level_flag = 1 << ( level - 1 );
+    boost::int32_t level_flag = 1 << ( level - 1 );
 
     M_main_data.getDebugLogHolder().setLevel( level_flag,
                                               M_debug_level_act[level-1]->isOn() );
@@ -929,7 +1073,7 @@ DebugMessageWindow::updateMessage()
         return;
     }
 
-    const long cur_cycle = M_main_data.getDebugLogHolder().getCurrentCycle();
+    const int cur_cycle = M_main_data.getDebugLogHolder().getCurrentCycle();
 
     QTextEdit * text_edit = M_message[ M_tab_widget->currentPageIndex() ];
 
