@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,6 +37,7 @@
 
 #include "detail_dialog.h"
 
+#include "options.h"
 #include "main_data.h"
 
 #include <rcsc/geom/vector_2d.h>
@@ -47,7 +48,7 @@
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 DetailDialog::DetailDialog( QWidget * parent,
                             const MainData & main_data )
     : QDialog( parent )
@@ -66,7 +67,7 @@ DetailDialog::DetailDialog( QWidget * parent,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 DetailDialog::~DetailDialog()
 {
     //std::cerr << "delete DetailDialog" << std::endl;
@@ -75,13 +76,14 @@ DetailDialog::~DetailDialog()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 DetailDialog::createWidgets()
 {
     QVBoxLayout * layout = new QVBoxLayout( this );
     layout->setMargin( 2 );
     layout->setSpacing( 2 );
+    layout->setResizeMode( QLayout::Fixed );
 
 
     layout->addWidget( createBallLabels(),
@@ -96,7 +98,7 @@ DetailDialog::createWidgets()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 QWidget *
 DetailDialog::createBallLabels()
 {
@@ -142,7 +144,7 @@ DetailDialog::createBallLabels()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 QWidget *
 DetailDialog::createPlayerLabels()
 {
@@ -229,6 +231,11 @@ DetailDialog::createPlayerLabels()
                        row, 0, Qt::AlignRight );
     M_stamina = new QLabel( tr( "    0.0" ), group_box );
     layout->addWidget( M_stamina, row, 1, Qt::AlignRight );
+    //    ++row;
+    //    layout->addWidget( new QLabel( tr( "Capacity " ), group_box ),
+    //                       row, 0, Qt::AlignRight );
+    M_stamina_capacity = new QLabel( tr( "   -1.0" ), group_box );
+    layout->addWidget( M_stamina_capacity, row, 2, Qt::AlignRight );
     ++row;
 
     layout->addWidget( new QLabel( tr( "Effort " ), group_box ),
@@ -285,7 +292,7 @@ DetailDialog::createPlayerLabels()
     layout->addWidget( M_move_count, row, 1, Qt::AlignRight );
     ++row;
 
-    layout->addWidget( new QLabel( tr( "ChbView " ), group_box ),
+    layout->addWidget( new QLabel( tr( "ChgView " ), group_box ),
                        row, 0, Qt::AlignRight );
     M_change_view_count = new QLabel( tr( "     0" ), group_box );
     layout->addWidget( M_change_view_count, row, 1, Qt::AlignRight );
@@ -329,9 +336,9 @@ DetailDialog::createPlayerLabels()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-DetailDialog::showEvent( QShowEvent * event )
+DetailDialog::showEvent( QShowEvent * )
 {
     updateLabels();
 }
@@ -339,7 +346,7 @@ DetailDialog::showEvent( QShowEvent * event )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 DetailDialog::updateLabels()
 {
@@ -348,14 +355,14 @@ DetailDialog::updateLabels()
         return;
     }
 
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
 
     if ( ! view )
     {
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
 
     MonitorViewConstPtr last_view
         = ( M_main_data.viewIndex() <= 0
@@ -364,7 +371,7 @@ DetailDialog::updateLabels()
 
     //std::cerr << "DetailDialog::updteLabels start" << std::endl;
 
-    const double reverse = vconf.reverseValue();
+    const double reverse = opt.reverseValue();
 
     char buf[64];
 
@@ -372,23 +379,23 @@ DetailDialog::updateLabels()
 
     const Ball & ball = view->ball();
 
-    std::snprintf( buf, 64, " % 6.2f, % 6.2f",
-                   ball.x() * reverse,
-                   ball.y() * reverse );
+    snprintf( buf, 64, " % 6.2f, % 6.2f",
+              ball.x() * reverse,
+              ball.y() * reverse );
     M_ball_pos->setText( QString::fromAscii( buf ) );
 
     if ( ball.hasDelta() )
     {
         rcsc::Vector2D vel( ball.deltaX(), ball.deltaY() );
         vel *= reverse;
-        std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       vel.x, vel.y );
-				M_ball_vel->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  vel.x, vel.y );
+        M_ball_vel->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, " (%7.3f, %6.1f)",
-                       vel.r(), vel.th().degree() );
- 				M_ball_polar_vel->setText( QString::fromAscii( buf ) );
-		}
+        snprintf( buf, 64, " (%7.3f, %6.1f)",
+                  vel.r(), vel.th().degree() );
+        M_ball_polar_vel->setText( QString::fromAscii( buf ) );
+    }
 
     if ( ! last_view )
     {
@@ -401,18 +408,18 @@ DetailDialog::updateLabels()
                              ball.y() - last_view->ball().y() );
         move *= reverse;
 
-        std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       move.x, move.y );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  move.x, move.y );
         M_ball_last_move->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, " (%7.3f, %6.1f)",
-                       move.r(), move.th().degree() );
+        snprintf( buf, 64, " (%7.3f, %6.1f)",
+                  move.r(), move.th().degree() );
         M_ball_last_polar_move->setText( QString::fromAscii( buf ) );
     }
 
     // update player
 
-    int number = M_main_data.viewConfig().selectedNumber();
+    int number = opt.selectedNumber();
     if ( number == 0 || std::abs( number ) > 11 )
     {
         return;
@@ -424,46 +431,46 @@ DetailDialog::updateLabels()
         = M_main_data.viewHolder().playerType( player.type() );
 
     // side, number
-    std::snprintf( buf, 64, "%8s  %2d",
-                   ( vconf.reverseSide()
-                     ? ( number > 0 ? "   Right" : "   Left" )
-                     : ( number > 0 ? "   Left" : "   Right" ) ),
-                   number > 0 ? number : -number );
+    snprintf( buf, 64, "%8s  %2d",
+              ( opt.reverseSide()
+                ? ( number > 0 ? "   Right" : "   Left" )
+                : ( number > 0 ? "   Left" : "   Right" ) ),
+              number > 0 ? number : -number );
     M_player_number->setText( QString::fromAscii( buf ) );
 
     // player type
-    std::snprintf( buf, 64, " %d" ,
-                   player.type() );
+    snprintf( buf, 64, " %d" ,
+              player.type() );
     M_player_type->setText( QString::fromAscii( buf ) );
 
     // pos
-    std::snprintf( buf, 64, " %6.2f, %6.2f",
-                   player.x() * reverse,
-                   player.y() * reverse );
-		M_player_pos->setText( QString::fromAscii( buf ) );
+    snprintf( buf, 64, " %6.2f, %6.2f",
+              player.x() * reverse,
+              player.y() * reverse );
+    M_player_pos->setText( QString::fromAscii( buf ) );
 
     // velocity, speed, speed dir
-		if ( player.hasDelta() )
-		{
-				rcsc::Vector2D vel( player.deltaX(), player.deltaY() );
+    if ( player.hasDelta() )
+    {
+        rcsc::Vector2D vel( player.deltaX(), player.deltaY() );
         vel *= reverse;
 
-				std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       vel.x, vel.y );
-				M_player_vel->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  vel.x, vel.y );
+        M_player_vel->setText( QString::fromAscii( buf ) );
 
-				std::snprintf( buf, 64, " (%7.3f, %6.1f)",
-                       vel.r(), vel.th().degree() );
-				M_player_polar_vel->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 64, " (%7.3f, %6.1f)",
+                  vel.r(), vel.th().degree() );
+        M_player_polar_vel->setText( QString::fromAscii( buf ) );
 
         // ball relative velocity
-        std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       ball.deltaX() - vel.x, ball.deltaY() - vel.y );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  ball.deltaX() - vel.x, ball.deltaY() - vel.y );
         M_player_ball_rel_vel->setText( QString::fromAscii( buf ) );
 
         // max turn moment
         double max_turn = player_type.effectiveTurn( 180.0, vel.r() );
-        std::snprintf( buf, 64, " %3.2f", max_turn );
+        snprintf( buf, 64, " %3.2f", max_turn );
         M_max_turn_moment->setText( QString::fromAscii( buf ) );
     }
 
@@ -479,21 +486,21 @@ DetailDialog::updateLabels()
                                   player.y() - last_player.y() );
         last_move *= reverse;
 
-				std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       last_move.x, last_move.y );
-				M_player_last_move->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  last_move.x, last_move.y );
+        M_player_last_move->setText( QString::fromAscii( buf ) );
 
-				std::snprintf( buf, 64, " (%7.3f, %6.1f)",
-                       last_move.r(), last_move.th().degree() );
-				M_player_last_polar_move->setText( QString::fromAscii( buf ) );
+        snprintf( buf, 64, " (%7.3f, %6.1f)",
+                  last_move.r(), last_move.th().degree() );
+        M_player_last_polar_move->setText( QString::fromAscii( buf ) );
     }
 
     // body angle
     rcsc::AngleDeg body = player.body();
-    if ( vconf.reverseSide() ) body += 180.0;
+    if ( opt.reverseSide() ) body += 180.0;
 
-    std::snprintf( buf, 64," %6.1f",
-                   body.degree() );
+    snprintf( buf, 64," %6.1f",
+              body.degree() );
     M_player_body->setText( QString::fromAscii( buf ) );
 
     // neck, head angle
@@ -501,19 +508,19 @@ DetailDialog::updateLabels()
     {
         rcsc::AngleDeg neck = player.head() - player.body();
         rcsc::AngleDeg head = player.head();
-        if ( vconf.reverseSide() ) head += 180.0;
+        if ( opt.reverseSide() ) head += 180.0;
 
-        std::snprintf( buf, 64, " %6.1f(%6.1f)",
-                       head.degree(), neck.degree() );
+        snprintf( buf, 64, " %6.1f(%6.1f)",
+                  head.degree(), neck.degree() );
         M_player_head->setText( QString::fromAscii( buf ) );
     }
 
     // pointto point
     if ( player.isPointing() )
     {
-        std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       player.data().point_x_ * reverse,
-                       player.data().point_y_ * reverse );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  player.data().point_x_ * reverse,
+                  player.data().point_y_ * reverse );
         M_player_pointto_pos->setText( QString::fromAscii( buf ) );
     }
     else
@@ -524,11 +531,11 @@ DetailDialog::updateLabels()
     // focus target
     if ( player.isFocusing() )
     {
-        std::snprintf( buf, 64, " %5s %2d",
-                       ( vconf.reverseSide()
-                         ? player.data().focus_side_ == 'l' ? "Left" : "Right"
-                         : player.data().focus_side_ == 'l' ? "Right" : "Left" ),
-                       player.data().focus_unum_ );
+        snprintf( buf, 64, " %5s %2d",
+                  ( opt.reverseSide()
+                    ? player.data().focus_side_ == 'l' ? "Right" : "Left"
+                    : player.data().focus_side_ == 'l' ? "Left" : "Right" ),
+                  player.data().focus_unum_ );
         M_player_focus_target->setText( QString::fromAscii( buf ) );
     }
     else
@@ -539,64 +546,68 @@ DetailDialog::updateLabels()
     // stamina, effort, recovery
     if ( player.hasStamina() )
     {
-        std::snprintf( buf, 64, " %6.1f",
-                       player.stamina() );
+        snprintf( buf, 64, " %6.1f",
+                  player.stamina() );
         M_stamina->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, " %5.3f / %5.3f",
-                       player.effort(), player_type.effortMax() );
+        snprintf( buf, 64, " %.0f",
+                  player.staminaCapacity() );
+        M_stamina_capacity->setText( QString::fromAscii( buf ) );
+
+        snprintf( buf, 64, " %5.3f / %5.3f",
+                  player.effort(), player_type.effortMax() );
         M_effort->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, " %5.3f / 1.0",
-                       player.recovery() );
+        snprintf( buf, 64, " %5.3f / 1.0",
+                  player.recovery() );
         M_recovery->setText( QString::fromAscii( buf ) );
     }
 
     // command count
     if ( player.hasCommandCount() )
     {
-        std::snprintf( buf, 64, "  %5d",
-                       player.kickCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.kickCount() );
         M_kick_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.dashCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.dashCount() );
         M_dash_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.turnCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.turnCount() );
         M_turn_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.sayCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.sayCount() );
         M_say_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.turnNeckCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.turnNeckCount() );
         M_turn_neck_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.catchCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.catchCount() );
         M_catch_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.moveCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.moveCount() );
         M_move_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.changeViewCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.changeViewCount() );
         M_change_view_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.tackleCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.tackleCount() );
         M_tackle_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.pointtoCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.pointtoCount() );
         M_pointto_count->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, "  %5d",
-                       player.attentiontoCount() );
+        snprintf( buf, 64, "  %5d",
+                  player.attentiontoCount() );
         M_attentionto_count->setText( QString::fromAscii( buf ) );
     }
 
@@ -606,12 +617,12 @@ DetailDialog::updateLabels()
                                        ball.y() - player.y() );
         player_to_ball *= reverse;
 
-        std::snprintf( buf, 64, " %6.2f, %6.2f",
-                       player_to_ball.x, player_to_ball.y );
+        snprintf( buf, 64, " %6.2f, %6.2f",
+                  player_to_ball.x, player_to_ball.y );
         M_player_ball_rel_pos->setText( QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 64, " (%7.3f, %6.1f)",
-                       player_to_ball.r(), player_to_ball.th().degree() );
+        snprintf( buf, 64, " (%7.3f, %6.1f)",
+                  player_to_ball.r(), player_to_ball.th().degree() );
         M_player_ball_rel_polar->setText( QString::fromAscii( buf ) );
     }
 }
