@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -71,6 +71,7 @@ const QColor DrawConfig::COLLIDE_PLAYER_COLOR = QColor( 105, 185, 255 );
 const QColor DrawConfig::TACKLE_COLOR = QColor( 255, 136, 127 );
 const QColor DrawConfig::TACKLE_FAULT_COLOR = QColor( 79, 159, 159 );
 const QColor DrawConfig::POINTTO_COLOR = QColor( 255, 136, 127 );
+//const QColor DrawConfig::ATTENTIONTO_COLOR = QColor( 0, 255, 0 );
 const QColor DrawConfig::MEASURE_COLOR = QColor( 0, 255, 255 );
 const QColor DrawConfig::MEASURE_FONT_COLOR = QColor( 255, 191, 191 );
 const QColor DrawConfig::DEBUG_BALL_COLOR = QColor( 255, 0, 127 );
@@ -84,7 +85,7 @@ const QColor DrawConfig::DEBUG_COMMENT_FONT_COLOR = QColor( 191, 191, 191 );
 const QColor DrawConfig::DEBUG_MESSAGE_FONT_COLOR = QColor( 255, 255, 255 );
 const QColor DrawConfig::DEBUG_TARGET_COLOR = QColor( 255, 127, 0 );
 const QColor DrawConfig::DEBUG_LOG_MESSAGE_FONT_COLOR = QColor( 0, 0, 0 );
-const QColor DrawConfig::DEBUG_FIGURE_COLOR = QColor( 255, 127, 0 );
+const QColor DrawConfig::DEBUG_SHAPE_COLOR = QColor( 255, 127, 0 );
 
 /*-------------------------------------------------------------------*/
 /*!
@@ -134,6 +135,7 @@ DrawConfig::DrawConfig()
     , M_tackle_brush( TACKLE_COLOR, Qt::SolidPattern )
     , M_tackle_fault_brush( TACKLE_FAULT_COLOR, Qt::SolidPattern )
     , M_pointto_pen( POINTTO_COLOR, 1, Qt::SolidLine )
+      //    , M_attentionto_pen( ATTENTIONTO_COLOR, 1, Qt::SolidLine )
     , M_kick_accel_pen( QColor( 0, 255, 0 ), 0, Qt::SolidLine )
     , M_tackle_area_pen( TACKLE_COLOR, 0, Qt::SolidLine )
     , M_measure_pen( MEASURE_COLOR, 1, Qt::SolidLine )
@@ -157,7 +159,7 @@ DrawConfig::DrawConfig()
     , M_debug_message_font_pen( DEBUG_MESSAGE_FONT_COLOR, 0, Qt::SolidLine )
     , M_debug_log_message_font( "Sans Serif", 8 )
     , M_debug_log_message_font_pen( DEBUG_LOG_MESSAGE_FONT_COLOR, 0, Qt::SolidLine )
-    , M_debug_figure_pen( DEBUG_FIGURE_COLOR, 1, Qt::SolidLine )
+    , M_debug_shape_pen( DEBUG_SHAPE_COLOR, 1, Qt::SolidLine )
 {
     //std::cerr << "create DrawConfig" << std::endl;
     readSettings();
@@ -224,6 +226,7 @@ DrawConfig::setDefaultColors()
     M_tackle_brush.setColor( TACKLE_COLOR );
     M_tackle_fault_brush.setColor( TACKLE_FAULT_COLOR );
     M_pointto_pen.setColor( POINTTO_COLOR );
+    //    M_attentionto_pen.setColor( ATTENTIONTO_COLOR );
     M_kick_accel_pen.setColor( QColor( 0, 255, 0 ) );
     M_tackle_area_pen.setColor( TACKLE_COLOR );
     M_measure_pen.setColor( MEASURE_COLOR );
@@ -243,7 +246,7 @@ DrawConfig::setDefaultColors()
     M_debug_comment_font_pen.setColor( DEBUG_COMMENT_FONT_COLOR );
     M_debug_message_font_pen.setColor( DEBUG_MESSAGE_FONT_COLOR );
     M_debug_log_message_font_pen.setColor( DEBUG_LOG_MESSAGE_FONT_COLOR );
-    M_debug_figure_pen.setColor( DEBUG_FIGURE_COLOR );
+    M_debug_shape_pen.setColor( DEBUG_SHAPE_COLOR );
 
 }
 
@@ -530,6 +533,12 @@ DrawConfig::readSettings()
         M_pointto_pen.setColor( toColor( val ) );
     }
 
+//     val = settings.readEntry( "attentionto_color" );
+//     if ( ! val.isEmpty() )
+//     {
+//         M_attentionto_pen.setColor( toColor( val ) );
+//     }
+
     val = settings.readEntry( "kick_accel_color" );
     if ( ! val.isEmpty() )
     {
@@ -605,10 +614,10 @@ DrawConfig::readSettings()
         M_debug_log_message_font_pen.setColor( toColor( val ) );
     }
 
-    val = settings.readEntry( "debug_figure_pen_color" );
+    val = settings.readEntry( "debug_shape_pen_color" );
     if ( ! val.isEmpty() )
     {
-        M_debug_figure_pen.setColor( toColor( val ) );
+        M_debug_shape_pen.setColor( toColor( val ) );
     }
 
     settings.endGroup();
@@ -714,6 +723,8 @@ DrawConfig::saveSettings()
                          toString( tackleFaultBrush().color() ) );
     settings.writeEntry( "pointto_color",
                          toString( pointtoPen().color() ) );
+//     settings.writeEntry( "attentionto_color",
+//                          toString( attentiontoPen().color() ) );
     settings.writeEntry( "kick_accel_color",
                          toString( kickAccelPen().color() ) );
 
@@ -741,8 +752,8 @@ DrawConfig::saveSettings()
 
     settings.writeEntry( "debug_log_message_font_color",
                          toString( debugLogMessageFontPen().color() ) );
-    settings.writeEntry( "debug_figure_pen_color",
-                         toString( debugFigurePen().color() ) );
+    settings.writeEntry( "debug_shape_pen_color",
+                         toString( debugShapePen().color() ) );
 
     settings.endGroup();
 }
