@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -38,7 +38,7 @@
 #include "monitor_client.h"
 
 #include "view_holder.h"
-#include "app_config.h"
+#include "options.h"
 
 #include <rcsc/trainer/trainer_command.h>
 #include <rcsc/monitor/monitor_command.h>
@@ -66,12 +66,14 @@ public:
 /*!
 
 */
-MonitorClient::MonitorClient( ViewHolder & view_holder,
+MonitorClient::MonitorClient( QObject * parent,
+                              ViewHolder & view_holder,
                               const char * hostname,
                               const int port,
                               const int version )
 
-    : M_impl( new MonitorClientImpl )
+    : QObject( parent )
+    , M_impl( new MonitorClientImpl )
     , M_view_holder( view_holder )
     , M_socket_device( static_cast< QSocketDevice * >( 0 ) )
     , M_socket_notifier( static_cast< QSocketNotifier * >( 0 ) )
@@ -86,9 +88,9 @@ MonitorClient::MonitorClient( ViewHolder & view_holder,
     {
         M_version = 1;
     }
-    if ( 3 < version )
+    if ( 4 < version )
     {
-        M_version = 3;
+        M_version = 4;
     }
 
     M_socket_device = new QSocketDevice( QSocketDevice::Datagram );
@@ -237,18 +239,24 @@ MonitorClient::tryConnecting()
 
 */
 void
-MonitorClient::handleReceive( int socket )
+MonitorClient::handleReceive( int /*socket*/ )
 {
     int receive_count = 0;
 
-    if ( M_version == 3 )
+    if ( M_version >= 3 )
     {
         char buf[8192];
         while ( M_socket_device->readBlock( buf, 8192 ) > 0 )
         {
             //std::cerr << "recv: " << std::strlen( buf )
             //          << " " << buf << std::endl;
-            if ( ! M_view_holder.addDisp3( buf ) )
+            if ( M_version == 4
+                 && ! M_view_holder.addDisp4( buf ) )
+            {
+                std::cerr << "recv: " << buf << std::endl;
+            }
+            else if ( M_version == 3
+                      && ! M_view_holder.addDisp3( buf ) )
             {
                 std::cerr << "recv: " << buf << std::endl;
             }
@@ -337,10 +345,10 @@ MonitorClient::handleTimer()
 
     //std::cerr << "handleTimer waited = " << M_waited_msec << std::endl;
 
-    if ( AppConfig::instance().waitSeconds() > 0
-         && M_waited_msec >= AppConfig::instance().waitSeconds() * 1000 )
+    if ( Options::instance().waitSeconds() > 0
+         && M_waited_msec >= Options::instance().waitSeconds() * 1000 )
     {
-        if ( AppConfig::instance().autoQuitMode() )
+        if ( Options::instance().autoQuitMode() )
         {
             std::cerr << "waited " << M_waited_msec * 1000 << " seconds.\n"
                       << "end..."
