@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -80,7 +80,8 @@ private:
     MonitorClient & operator=( const MonitorClient & );
 public:
     //! constructor
-    MonitorClient( ViewHolder & view_holder,
+    MonitorClient( QObject * parent,
+                   ViewHolder & view_holder,
                    const char * hostname,
                    const int port,
                    const int version );
