@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -41,6 +41,7 @@
 #include "draw_config.h"
 
 // model
+#include "options.h"
 #include "main_data.h"
 
 #include <rcsc/common/player_type.h>
@@ -85,23 +86,35 @@ const QBrush PlayerPainterRCSS::KICK_FAULT_BRUSH( QColor( "#0000ff" ),
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 inline
 PlayerPainterRCSS::Param::Param( const Player & player,
                                  const Ball & ball,
-                                 const ViewConfig & view_conf,
                                  const rcsc::PlayerType & ptype )
-    : x_( view_conf.screenX( player.x() ) )
-    , y_( view_conf.screenY( player.y() ) )
-    , body_( player.body() + ( view_conf.reverseSide() ? 180.0 : 0.0 ) )
-    , head_( player.head() + ( view_conf.reverseSide() ? 180.0 : 0.0 ) )
-    , body_radius_( view_conf.scale( ptype.playerSize() ) )
-    , kick_radius_( view_conf.scale( ptype.kickableArea() ) )
+    : x_( 0 )
+    , y_( 0 )
+    , body_( player.body() )
+    , head_( player.head() )
+    , body_radius_( 1 )
+    , kick_radius_( 5 )
     , have_full_effort_( player.hasFullEffort( ptype.effortMax() ) )
     , player_( player )
     , ball_( ball )
     , player_type_( ptype )
 {
+    const Options & opt = Options::instance();
+
+    x_ = opt.screenX( player.x() );
+    y_ = opt.screenY( player.y() );
+
+    if ( opt.reverseSide() )
+    {
+        body_ += 180.0;
+        head_ += 180.0;
+    }
+
+    body_radius_ = opt.scale( ptype.playerSize() );
+    kick_radius_ = opt.scale( ptype.kickableArea() );
 
     if ( body_radius_ < 1 ) body_radius_ = 1;
     if ( kick_radius_ < 5 ) kick_radius_ = 5;
@@ -112,7 +125,7 @@ PlayerPainterRCSS::Param::Param( const Player & player,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 PlayerPainterRCSS::PlayerPainterRCSS( const MainData & main_data )
     : M_main_data( main_data )
 {
@@ -122,27 +135,27 @@ PlayerPainterRCSS::PlayerPainterRCSS( const MainData & main_data )
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainterRCSS::draw( QPainter & painter )
 {
-    if ( ! M_main_data.viewConfig().isShownPlayers() )
+    const Options & opt = Options::instance();
+
+    if ( ! opt.showPlayers() )
     {
         return;
     }
 
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
 
     if ( ! view )
     {
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
-
     const Ball & ball = view->ball();
 
-    if ( vconf.selectedNumber() != 0 )
+    if ( opt.selectedNumber() != 0 )
     {
         const std::vector< Player >::const_iterator end = view->players().end();
         for ( std::vector< Player >::const_iterator it = view->players().begin();
@@ -154,15 +167,15 @@ PlayerPainterRCSS::draw( QPainter & painter )
                 break;
             }
 
-            if ( vconf.isSelectedPlayer( it->side(), it->unum() ) )
+            if ( opt.isSelectedPlayer( it->side(), it->unum() ) )
             {
-                drawViewArea( painter, *it );
+                drawViewAreaBackground( painter, *it );
                 break;
             }
         }
     }
 
-    if ( vconf.playerReverseDraw() )
+    if ( opt.playerReverseDraw() )
     {
         const std::vector< Player >::const_reverse_iterator end = view->players().rend();
         for ( std::vector< Player >::const_reverse_iterator it = view->players().rbegin();
@@ -187,37 +200,43 @@ PlayerPainterRCSS::draw( QPainter & painter )
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
-PlayerPainterRCSS::drawAll(  QPainter & painter,
-                             const Player & player,
-                             const Ball & ball ) const
+PlayerPainterRCSS::drawAll( QPainter & painter,
+                            const Player & player,
+                            const Ball & ball ) const
 {
+    const Options & opt = Options::instance();
     const Param param( player,
                        ball,
-                       M_main_data.viewConfig(),
                        M_main_data.viewHolder().playerType( player.type() ) );
 
     drawBody( painter, param );
     //drawShadow( painter, param );
     drawEdge( painter, param );
 
-    if ( M_main_data.viewConfig().isSelectedPlayer( player.side(), player.unum() )
-         && M_main_data.viewConfig().playerFutureCycle() > 0
+    if ( opt.isSelectedPlayer( player.side(), player.unum() )
+         && opt.playerFutureCycle() > 0
          && player.hasDelta() )
     {
         drawFuture( painter, param );
     }
 
     if ( player.hasView()
-         && M_main_data.viewConfig().isShownViewCone() )
+         && opt.showViewArea() )
     {
-        drawViewCone( painter, param );
+        drawViewArea( painter, param );
     }
 
-    if ( M_main_data.viewConfig().isShownControlArea() )
+    if ( player.isGoalie()
+         && opt.showCatchableArea() )
     {
-        drawControlArea( painter, param );
+        drawCatchableArea( painter, param );
+    }
+
+    if ( opt.showTackleArea() )
+    {
+        drawTackleArea( painter, param );
     }
 
     drawText( painter, param );
@@ -226,7 +245,7 @@ PlayerPainterRCSS::drawAll(  QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainterRCSS::drawBody( QPainter & painter,
                              const PlayerPainterRCSS::Param & param ) const
@@ -237,7 +256,7 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
     painter.setPen( PLAYER_PEN );
 
     rcsc::SideID side = param.player_.side();
-    if ( M_main_data.viewConfig().reverseSide() )
+    if ( Options::instance().reverseSide() )
     {
         side = static_cast< rcsc::SideID >( -1 * side );
     }
@@ -308,10 +327,10 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
     {
         painter.setBrush( BALL_COLLIDE_BRUSH );
     }
-//     if ( param.player_.isCollidedPlayer() )
-//     {
-//         painter.setBrush( dconf.collidePlayerBrush() );
-//     }
+    //     if ( param.player_.isCollidedPlayer() )
+    //     {
+    //         painter.setBrush( dconf.collidePlayerBrush() );
+    //     }
 
     painter.drawEllipse( param.x_ - param.kick_radius_ ,
                          param.y_ - param.kick_radius_ ,
@@ -323,10 +342,10 @@ PlayerPainterRCSS::drawBody( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
-PlayerPainterRCSS::drawShadow( QPainter & painter,
-                               const PlayerPainterRCSS::Param & param ) const
+PlayerPainterRCSS::drawShadow( QPainter &,
+                               const PlayerPainterRCSS::Param & ) const
 {
 
 }
@@ -334,12 +353,12 @@ PlayerPainterRCSS::drawShadow( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerPainterRCSS::drawEdge( QPainter & painter,
                              const PlayerPainterRCSS::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     // draw body edge
@@ -357,7 +376,8 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
     // draw stamina status if effort or recovery is decayed.
     if ( param.player_.hasStamina() )
     {
-        if ( param.player_.stamina() < 1500.0 )
+        //if ( param.player_.stamina() < 1500.0 )
+        if ( param.player_.stamina() < rcsc::ServerParam::i().staminaMax() * 0.375 )
         {
             painter.setBrush( dconf.shadowBrush() );
             painter.drawEllipse( param.x_ - param.body_radius_,
@@ -369,12 +389,12 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
 
     // body direction line
     {
-        int end_x = vconf.absScreenX( param.player_.x() * vconf.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = vconf.absScreenY( param.player_.y() * vconf.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
 
         painter.setPen( PLAYER_PEN );
         painter.setBrush( dconf.transparentBrush() );
@@ -387,31 +407,32 @@ PlayerPainterRCSS::drawEdge( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerPainterRCSS::drawFuture( QPainter & painter,
                                const PlayerPainterRCSS::Param & param ) const
 {
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     // draw future state
     rcsc::Vector2D ppos( param.player_.x(), param.player_.y() );
     rcsc::Vector2D pvel( param.player_.deltaX(), param.player_.deltaY() );
-    if ( M_main_data.viewConfig().reverseSide() )
+    if ( opt.reverseSide() )
     {
         ppos *= -1.0;
         pvel *= -1.0;
     }
 
-    const int last = M_main_data.viewConfig().playerFutureCycle();
+    const int last = opt.playerFutureCycle();
 
-    QPoint first_point( M_main_data.viewConfig().absScreenX( ppos.x ),
-                        M_main_data.viewConfig().absScreenY( ppos.y ) ) ;
+    QPoint first_point( opt.absScreenX( ppos.x ),
+                        opt.absScreenY( ppos.y ) ) ;
     QPoint last_point = first_point;
 
     // draw kickable area edge
     rcsc::SideID side = param.player_.side();
-    if ( M_main_data.viewConfig().reverseSide() )
+    if ( opt.reverseSide() )
     {
         side = static_cast< rcsc::SideID >( -1 * side );
     }
@@ -432,8 +453,8 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( M_main_data.viewConfig().absScreenX( ppos.x ),
-                   M_main_data.viewConfig().absScreenY( ppos.y ) );
+        QPoint pt( opt.absScreenX( ppos.x ),
+                   opt.absScreenY( ppos.y ) );
         if ( std::abs( last_point.x() - pt.x() ) < 1
              && std::abs( last_point.y() - pt.y() ) < 1 )
         {
@@ -461,26 +482,26 @@ PlayerPainterRCSS::drawFuture( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainterRCSS::drawViewArea( QPainter & painter,
-                                 const Player & player ) const
+PlayerPainterRCSS::drawViewAreaBackground( QPainter & painter,
+                                           const Player & player ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    const int x = vconf.screenX( player.x() );
-    const int y = vconf.screenY( player.y() );
-    const int visible_radius = vconf.scale( rcsc::ServerParam::i().visibleDistance() );
+    const int x = opt.screenX( player.x() );
+    const int y = opt.screenY( player.y() );
+    const int visible_radius = opt.scale( rcsc::ServerParam::i().visibleDistance() );
     const int view_start_angle
         = static_cast< int >
         ( rint( ( - player.head() - player.viewWidth()*0.5
-                  + ( vconf.reverseSide() ? 180.0 : 0.0 ) ) * 16 ) );
+                  + ( opt.reverseSide() ? 180.0 : 0.0 ) ) * 16 ) );
     const int view_width
         = static_cast< int >( rint( player.viewWidth() * 16 ) );
 
-    const int TEAM_FAR = vconf.scale( 40.0 );
-    const int TEAM_TOOFAR = vconf.scale( 60.0 );
+    const int TEAM_FAR = opt.scale( 40.0 );
+    const int TEAM_TOOFAR = opt.scale( 60.0 );
 
     painter.setRasterOp( Qt::OrROP );
     painter.setPen( dconf.transparentPen() );
@@ -514,24 +535,24 @@ PlayerPainterRCSS::drawViewArea( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainterRCSS::drawViewCone( QPainter & painter,
+PlayerPainterRCSS::drawViewArea( QPainter & painter,
                                  const PlayerPainterRCSS::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     // draw face direction
     painter.setPen( NECK_PEN );
     painter.setBrush( dconf.transparentBrush() );
 
-    int end_x = vconf.absScreenX( param.player_.x() * vconf.reverseValue()
-                                  + param.player_type_.kickableArea()
-                                  * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
-    int end_y = vconf.absScreenY( param.player_.y() * vconf.reverseValue()
-                                  + param.player_type_.kickableArea()
-                                  * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
+    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
+                                + param.player_type_.kickableArea()
+                                * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
+                                + param.player_type_.kickableArea()
+                                * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );;
 
     painter.drawLine( param.x_, param.y_, end_x, end_y );
 }
@@ -539,39 +560,49 @@ PlayerPainterRCSS::drawViewCone( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainterRCSS::drawControlArea( QPainter & painter,
-                                    const PlayerPainterRCSS::Param & param ) const
+PlayerPainterRCSS::drawCatchableArea( QPainter & painter,
+                                      const PlayerPainterRCSS::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    if ( param.player_.isGoalie() )
+    rcsc::SideID side = param.player_.side();
+    if ( opt.reverseSide() )
     {
-        rcsc::SideID side = param.player_.side();
-        if ( M_main_data.viewConfig().reverseSide() )
-        {
-            side = static_cast< rcsc::SideID >( -1 * side );
-        }
-
-        int catchable = vconf.scale( rcsc::ServerParam::i().catchableArea() );
-        painter.setPen( ( side == rcsc::LEFT )
-                        ? dconf.leftGoaliePen()
-                        : dconf.rightGoaliePen() );
-        painter.setBrush( dconf.transparentBrush() );
-
-        painter.drawEllipse( param.x_ - catchable,
-                             param.y_ - catchable,
-                             catchable * 2,
-                             catchable * 2 );
+        side = static_cast< rcsc::SideID >( -1 * side );
     }
 
+    int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
+    painter.setPen( ( side == rcsc::LEFT )
+                    ? dconf.leftGoaliePen()
+                    : dconf.rightGoaliePen() );
+    painter.setBrush( dconf.transparentBrush() );
+
+    painter.drawEllipse( param.x_ - catchable,
+                         param.y_ - catchable,
+                         catchable * 2,
+                         catchable * 2 );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+PlayerPainterRCSS::drawTackleArea( QPainter & painter,
+                                   const PlayerPainterRCSS::Param & param ) const
+{
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
+    const DrawConfig & dconf = DrawConfig::instance();
+
     rcsc::Vector2D ppos( param.player_.x(),
                          param.player_.y() );
     rcsc::Vector2D bpos( param.ball_.x(),
                          param.ball_.y() );
-    if ( vconf.reverseSide() )
+    if ( opt.reverseSide() )
     {
         ppos *= -1.0;
         bpos *= -1.0;
@@ -582,34 +613,34 @@ PlayerPainterRCSS::drawControlArea( QPainter & painter,
 
     // draw tackle area & probability
     double tackle_dist = ( player_to_ball.x > 0.0
-                           ? rcsc::ServerParam::i().tackleDist()
-                           : rcsc::ServerParam::i().tackleBackDist() );
+                           ? SP.tackleDist()
+                           : SP.tackleBackDist() );
     double tackle_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
-                                     rcsc::ServerParam::i().tackleExponent() )
-                           + std::pow( player_to_ball.absY() / rcsc::ServerParam::i().tackleWidth(),
-                                       rcsc::ServerParam::i().tackleExponent() ) );
+                                     SP.tackleExponent() )
+                           + std::pow( player_to_ball.absY() / SP.tackleWidth(),
+                                       SP.tackleExponent() ) );
     if ( tackle_prob < 1.0 )
     {
         rcsc::AngleDeg body_angle = param.body_;
         rcsc::AngleDeg body_angle_side = body_angle + 90.0;
         double body_x = body_angle.cos();
         double body_y = body_angle.sin();
-        double forward_x = rcsc::ServerParam::i().tackleDist() * body_x;
-        double forward_y = rcsc::ServerParam::i().tackleDist() * body_y;
-        double back_x = rcsc::ServerParam::i().tackleBackDist() * -body_x;
-        double back_y = rcsc::ServerParam::i().tackleBackDist() * -body_y;
-        double right_x = rcsc::ServerParam::i().tackleWidth() * body_angle_side.cos();
-        double right_y = rcsc::ServerParam::i().tackleWidth() * body_angle_side.sin();
+        double forward_x = SP.tackleDist() * body_x;
+        double forward_y = SP.tackleDist() * body_y;
+        double back_x = SP.tackleBackDist() * -body_x;
+        double back_y = SP.tackleBackDist() * -body_y;
+        double right_x = SP.tackleWidth() * body_angle_side.cos();
+        double right_y = SP.tackleWidth() * body_angle_side.sin();
 
         QPointArray pts( 4 );
-        pts[0].setX( vconf.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( vconf.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( vconf.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( vconf.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( vconf.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( vconf.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( vconf.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( vconf.absScreenY( ppos.y + back_y + right_y ) );
+        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -626,53 +657,59 @@ PlayerPainterRCSS::drawControlArea( QPainter & painter,
 
         painter.setFont( dconf.playerFont() );
         painter.setPen( dconf.tacklePen() );
-
-        char msg[32];
-        std::snprintf( msg, 32, "TakleProb=%.3f", 1.0 - tackle_prob );
         painter.drawText( param.x_ + text_radius,
                           param.y_ + 4 + painter.fontMetrics().ascent(),
-                          QString::fromAscii( msg ) );
+                          QString( "TackleProb=%1" ).arg( 1.0 - tackle_prob, 0, 'g', 3 ) );
     }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerPainterRCSS::drawText( QPainter & painter,
                              const PlayerPainterRCSS::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
 
-    char main_buf[64];
-    std::memset( main_buf, 0, 64 );
+    QString str;
 
-    if ( vconf.isShownPlayerNumber() )
+    if ( opt.showPlayerNumber() )
     {
-        char buf[8];
-        std::snprintf( buf, 8, "%X", param.player_.unum() );
-        std::strcat( main_buf, buf );
+        str += QString::number( param.player_.unum(), 16 ).upper();
     }
 
-    if ( param.player_.hasStamina()
-         && vconf.isShownStamina() )
+    if ( param.player_.hasStamina() )
     {
-        char buf[16];
-        std::snprintf( buf, 16, "%4.0f", param.player_.stamina() );
-        if ( main_buf[0] != '\0' ) std::strcat( main_buf, " " );
-        std::strcat( main_buf, buf );
+        if ( opt.showStamina() )
+        {
+            if ( ! str.isEmpty() ) str += ' ';
+            str += QString::number( static_cast< int >( rint ( param.player_.stamina() ) ) );
+        }
+
+        if ( opt.showStaminaCapacity()
+             && param.player_.hasStaminaCapacity() )
+        {
+            if ( ! str.isEmpty() ) str += '/';
+            str += QString::number( static_cast< int >( rint( param.player_.staminaCapacity() ) ) );
+        }
     }
 
-    if ( vconf.isShownHeteroNumber() )
+    if ( opt.showPlayerType() )
     {
-        char buf[8];
-        std::snprintf( buf, 8, "t%d", param.player_.type() );
-        if ( main_buf[0] != '\0' ) std::strcat( main_buf, ", " );
-        strcat( main_buf, buf );
+        if ( ! str.isEmpty() )
+        {
+            str += ", t";
+        }
+        else
+        {
+            str += 't';
+        }
+        str += QString::number( param.player_.type() );
     }
 
-    if ( main_buf[0] != '\0' )
+    if ( ! str.isEmpty() )
     {
         static bool first = true;
         static QFont font( "6x13bold", 9, QFont::Bold );
@@ -689,7 +726,7 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         painter.setFont( font );
 
         rcsc::SideID side = param.player_.side();
-        if ( M_main_data.viewConfig().reverseSide() )
+        if ( opt.reverseSide() )
         {
             side = static_cast< rcsc::SideID >( -1 * side );
         }
@@ -697,8 +734,6 @@ PlayerPainterRCSS::drawText( QPainter & painter,
         painter.setPen( side == rcsc::LEFT
                         ? LEFT_TEAM_PEN
                         : RIGHT_TEAM_PEN );
-        painter.drawText( param.x_,
-                          param.y_,
-                          QString::fromAscii( main_buf ) );
+        painter.drawText( param.x_, param.y_, str );
     }
 }
