@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -72,7 +72,7 @@ public:
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 PlayerTypeDialog::PlayerTypeDialog( QWidget * parent,
                                     const MainData & main_data )
     : QDialog( parent )
@@ -91,7 +91,7 @@ PlayerTypeDialog::PlayerTypeDialog( QWidget * parent,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 PlayerTypeDialog::~PlayerTypeDialog()
 {
     //std::cerr << "delete PlayerTypeDialog" << std::endl;
@@ -100,13 +100,17 @@ PlayerTypeDialog::~PlayerTypeDialog()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerTypeDialog::createTable()
 {
     QBoxLayout * layout = new QBoxLayout( this, QBoxLayout::LeftToRight );
 
     M_player_type_table = new QListView( this );
+    //M_player_type_table->setResizeMode( QListView::AllColumns );
+    M_player_type_table->setMinimumWidth( 640 );
+    M_player_type_table->setHScrollBarMode( QScrollView::AlwaysOff );
+    M_player_type_table->setVScrollBarMode( QScrollView::AlwaysOff );
 
     //QFont font = M_player_type_table->font();
     //font.setPointSize( 8 );
@@ -118,55 +122,98 @@ PlayerTypeDialog::createTable()
         int col = 0;
         M_player_type_table->addColumn( tr( "id" ),
                                         metrics.width( tr( "0" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Speed Real/Max" ),
                                         metrics.width( tr( "1.000 / 1.200" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "AccelStep" ),
                                         metrics.width( tr( "000" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "AccelMax" ),
                                         metrics.width( tr( "0.6000" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "StamInc" ),
                                         metrics.width( tr( "45.00" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Consume" ),
                                         metrics.width( tr( "55.00" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Kickable" ),
                                         metrics.width( tr( "1.085" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Decay" ),
                                         metrics.width( tr( "0.400" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Inertia" ),
                                         metrics.width( tr( "5.00" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "DashRate" ),
                                         metrics.width( tr( "0.00600" ) ) + 12 );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
         M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
         M_player_type_table->addColumn( tr( "Size" ),
                                         metrics.width( tr( "0.300" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "KickMargin" ),
                                         metrics.width( tr( "0.700" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "KickRand" ),
                                         metrics.width( tr( "0.00" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "ExtStam" ),
                                         metrics.width( tr( "100.00" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
+        //
         M_player_type_table->addColumn( tr( "Effort Max-Min" ),
                                         metrics.width( tr( "1.000 - 1.000" ) ) + 12 );
-        M_player_type_table->setColumnAlignment( col++, Qt::AlignRight );
+        M_player_type_table->setColumnWidthMode( col, QListView::Maximum );
+        M_player_type_table->setColumnAlignment( col, Qt::AlignRight );
+        ++col;
     }
 
     createListViewItems();
 
-    M_player_type_table->setHScrollBarMode( QScrollView::AlwaysOff );
-    M_player_type_table->setVScrollBarMode( QScrollView::AlwaysOff );
+    M_player_type_table->header()->setMovingEnabled( false );
+    M_player_type_table->header()->setResizeEnabled( true );
+    //M_player_type_table->header()->setStretchEnabled( true, -1 );
 
     layout->addWidget( M_player_type_table );
     //this->setLayout( layout ); // qt4
@@ -175,7 +222,7 @@ PlayerTypeDialog::createTable()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerTypeDialog::createListViewItems()
 {
@@ -213,9 +260,9 @@ PlayerTypeDialog::createListViewItems()
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerTypeDialog::showEvent( QShowEvent * event )
+PlayerTypeDialog::showEvent( QShowEvent * )
 {
     updateData();
 
@@ -237,7 +284,7 @@ PlayerTypeDialog::showEvent( QShowEvent * event )
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerTypeDialog::updateData()
 {
@@ -251,8 +298,6 @@ PlayerTypeDialog::updateData()
         createListViewItems();
     }
 
-    char buf[32];
-
     const int size = M_player_types.size();
     for ( int i = 0; i < size; ++i )
     {
@@ -266,53 +311,54 @@ PlayerTypeDialog::updateData()
         const rcsc::PlayerType & param = it->second;
 
         int col = 0;
+        char buf[32];
 
-        std::snprintf( buf, 32, "%d", param.id() );
+        snprintf( buf, 32, "%d", param.id() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%5.3f / %5.3f",
-                       param.realSpeedMax(), param.playerSpeedMax() );
+        snprintf( buf, 32, "%5.3f / %5.3f",
+                  param.realSpeedMax(), param.playerSpeedMax() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%d", param.cyclesToReachMaxSpeed() );
+        snprintf( buf, 32, "%d", param.cyclesToReachMaxSpeed() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.4f",
-                       rcsc::ServerParam::i().maxPower() * param.dashPowerRate() * param.effortMax() );
+        snprintf( buf, 32, "%.4f",
+                  rcsc::ServerParam::i().maxPower() * param.dashPowerRate() * param.effortMax() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f", param.staminaIncMax() );
+        snprintf( buf, 32, "%.2f", param.staminaIncMax() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f",
-                       param.getOneStepStaminaComsumption( rcsc::ServerParam::i() ) );
+        snprintf( buf, 32, "%.2f",
+                  param.getOneStepStaminaComsumption( rcsc::ServerParam::i() ) );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.3f", param.playerSize() + param.kickableMargin() + 0.085 );
+        snprintf( buf, 32, "%.3f", param.playerSize() + param.kickableMargin() + 0.085 );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.3f", param.playerDecay() );
+        snprintf( buf, 32, "%.3f", param.playerDecay() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f", param.inertiaMoment() );
+        snprintf( buf, 32, "%.2f", param.inertiaMoment() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.5f", param.dashPowerRate() );
+        snprintf( buf, 32, "%.5f", param.dashPowerRate() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.3f", param.playerSize() );
+        snprintf( buf, 32, "%.3f", param.playerSize() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.3f", param.kickableMargin() );
+        snprintf( buf, 32, "%.3f", param.kickableMargin() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f", param.kickRand() );
+        snprintf( buf, 32, "%.2f", param.kickRand() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.2f", param.extraStamina() );
+        snprintf( buf, 32, "%.2f", param.extraStamina() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
 
-        std::snprintf( buf, 32, "%.3f - %.3f", param.effortMax(), param.effortMin() );
+        snprintf( buf, 32, "%.3f - %.3f", param.effortMax(), param.effortMin() );
         M_player_types[i]->setText( col++, QString::fromAscii( buf ) );
     }
 
