@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -41,7 +41,8 @@
 #include "debug_log_holder.h"
 #include "debug_log_data.h"
 #include "debug_log_dir_dialog.h"
-#include "app_config.h"
+
+#include "options.h"
 
 #include <cassert>
 #include <iostream>
@@ -49,7 +50,8 @@
 #include "xpm/debug_view_all.xpm"
 #include "xpm/debug_view_self.xpm"
 #include "xpm/debug_view_ball.xpm"
-#include "xpm/debug_view_player.xpm"
+#include "xpm/debug_view_teammate.xpm"
+#include "xpm/debug_view_opponent.xpm"
 #include "xpm/debug_view_comment.xpm"
 #include "xpm/debug_view_line.xpm"
 #include "xpm/debug_view_target.xpm"
@@ -58,6 +60,7 @@
 #include "xpm/debug_log_view.xpm"
 
 #include "xpm/open_dir.xpm"
+#include "xpm/clear.xpm"
 
 #include "xpm/sync.xpm"
 #include "xpm/logplayer_one_step_back.xpm"
@@ -79,6 +82,22 @@
 #include "xpm/num_14.xpm"
 #include "xpm/num_15.xpm"
 #include "xpm/num_16.xpm"
+#include "xpm/num_17.xpm"
+#include "xpm/num_18.xpm"
+#include "xpm/num_19.xpm"
+#include "xpm/num_20.xpm"
+#include "xpm/num_21.xpm"
+#include "xpm/num_22.xpm"
+#include "xpm/num_23.xpm"
+#include "xpm/num_24.xpm"
+#include "xpm/num_25.xpm"
+#include "xpm/num_26.xpm"
+#include "xpm/num_27.xpm"
+#include "xpm/num_28.xpm"
+#include "xpm/num_29.xpm"
+#include "xpm/num_30.xpm"
+#include "xpm/num_31.xpm"
+#include "xpm/num_32.xpm"
 
 class TabWidget
     : public QTabWidget {
@@ -116,29 +135,34 @@ DebugMessageWindow::DebugMessageWindow( QWidget * parent,
 {
     this->setWindowTitle( tr( "Debug Message" ) );
 
-    M_debug_text[0] = tr( "System" );
-    M_debug_text[1] = tr( "Sensor" );
-    M_debug_text[2] = tr( "WorldModel" );
-    M_debug_text[3] = tr( "BasicAction" );
-    M_debug_text[4] = tr( "Intercept" );
-    M_debug_text[5] = tr( "Kick" );
-    M_debug_text[6] = tr( "Dribble" );
-    M_debug_text[7] = tr( "Pass" );
-    M_debug_text[8] = tr( "Cross" );
-    M_debug_text[9] = tr( "Shoot" );
-    M_debug_text[10] = tr( "Clear" );
-    M_debug_text[11] = tr( "Team" );
-    M_debug_text[12] = tr( "Role" );
-    //M_debug_text[13] = tr( "" );
-    //M_debug_text[14] = tr( "" );
-    //M_debug_text[15] = tr( "" );
+    int i = -1;
+    M_debug_text[++i] = tr( "System" );
+    M_debug_text[++i] = tr( "Sensor" );
+    M_debug_text[++i] = tr( "WM" );
+    M_debug_text[++i] = tr( "Action" );
+    M_debug_text[++i] = tr( "Intercept" );
+    M_debug_text[++i] = tr( "Kick" );
+    M_debug_text[++i] = tr( "Hold" );
+    M_debug_text[++i] = tr( "Dribble" );
+    M_debug_text[++i] = tr( "Pass" );
+    M_debug_text[++i] = tr( "Cross" );
+    M_debug_text[++i] = tr( "Shoot" );
+    M_debug_text[++i] = tr( "Clear" );
+    M_debug_text[++i] = tr( "Block" );
+    M_debug_text[++i] = tr( "Mark" );
+    M_debug_text[++i] = tr( "Role" );
+    M_debug_text[++i] = tr( "Team" );
+    M_debug_text[++i] = tr( "Analyzer" );
+    M_debug_text[++i] = tr( "Recursive" );
+
 
     createActions();
     createWindows();
 
     readSettings();
 
-    this->resize( 640, 480 );
+    this->resize( 800, 600 );
+    this->setWindowFlags( this->windowFlags() | Qt::WindowStaysOnTopHint );
 }
 
 /*-------------------------------------------------------------------*/
@@ -158,8 +182,13 @@ DebugMessageWindow::~DebugMessageWindow()
 void
 DebugMessageWindow::readSettings()
 {
+#ifndef Q_WS_WIN
     QSettings settings( QDir::homePath() + "/.soccerwindow2-qt4",
                         QSettings::IniFormat );
+#else
+    QSettings settings( QDir::currentPath() + "/soccerwindow2.ini",
+                        QSettings::IniFormat );
+#endif
     //QSettings settings( QSettings::IniFormat,
     //                    QSettings::UserScope,
     //                    PACKAGE_NAME );
@@ -171,41 +200,82 @@ DebugMessageWindow::readSettings()
         QVariant val;
 
         val = settings.value( tr( "debug_view_all" ) );
-        if ( val.isValid() ) M_show_debug_view_all_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_all_act->setChecked( val.toBool() );
+            toggleShowDebugViewAll( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_view_self" ) );
-        if ( val.isValid() ) M_show_debug_view_self_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_self_act->setChecked( val.toBool() );
+            toggleShowDebugViewSelf( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_view_ball" ) );
-        if ( val.isValid() ) M_show_debug_view_ball_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_ball_act->setChecked( val.toBool() );
+            toggleShowDebugViewBall( val.toBool() );
+        }
 
-        val = settings.value( tr( "debug_view_players" ) );
-        if ( val.isValid() ) M_show_debug_view_players_act->setChecked( val.toBool() );
+        val = settings.value( tr( "debug_view_teammates" ) );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_teammates_act->setChecked( val.toBool() );
+            toggleShowDebugViewTeammates( val.toBool() );
+        }
+
+        val = settings.value( tr( "debug_view_opponents" ) );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_opponents_act->setChecked( val.toBool() );
+            toggleShowDebugViewOpponents( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_view_comment" ) );
-        if ( val.isValid() ) M_show_debug_view_comment_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_comment_act->setChecked( val.toBool() );
+            toggleShowDebugViewComment( val.toBool() );
+        }
 
-        val = settings.value( tr( "debug_view_figures" ) );
-        if ( val.isValid() ) M_show_debug_view_figure_act->setChecked( val.toBool() );
+        val = settings.value( tr( "debug_view_shape" ) );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_shape_act->setChecked( val.toBool() );
+            toggleShowDebugViewShape( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_view_target" ) );
-        if ( val.isValid() ) M_show_debug_view_target_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_target_act->setChecked( val.toBool() );
+            toggleShowDebugViewTarget( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_view_message" ) );
-        if ( val.isValid() ) M_show_debug_view_message_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_view_message_act->setChecked( val.toBool() );
+            toggleShowDebugViewMessage( val.toBool() );
+        }
 
         val = settings.value( tr( "debug_log_objects" ) );
-        if ( val.isValid() ) M_show_debug_log_objects_act->setChecked( val.toBool() );
+        if ( val.isValid() )
+        {
+            M_show_debug_log_objects_act->setChecked( val.toBool() );
+            toggleShowDebugLogObjects( val.toBool() );
+        }
 
     }
 
     // read debug switch status
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
-        char key[16];
-        std::snprintf( key, 16, "debug%02d", i + 1 );
-
-        QVariant val = settings.value( QString::fromAscii( key ) );
+        //snprintf( key, 16, "debug%02d", i + 1 );
+        QVariant val = settings.value( QString( "debug%1" ).arg( i + 1, 2, 10, QChar( '0' ) ) );
         if ( val.isValid() )
         {
             bool enable = val.toBool();
@@ -222,19 +292,19 @@ DebugMessageWindow::readSettings()
     }
 
     // read debug text string
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
-        char key[32];
-        std::snprintf( key, 32, "debug_text_%02d", i + 1 );
-
-        QVariant val = settings.value( QString::fromAscii( key ) );
+        //snprintf( key, 32, "debug_text_%02d", i + 1 );
+        QVariant val = settings.value( QString( "debug_text_%1" ).arg( i + 1, 2, 10, QChar( '0' ) ) );
         if ( val.isValid() )
         {
             M_debug_text[i] = val.toString();
-            QString text = tr( "Level %1 " ).arg( i + 1 );
-            text += M_debug_text[i];
-            M_debug_level_act[i]->setText( text );
-            M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + text );
+            //QString text = tr( "Level %1 " ).arg( i + 1 );
+            //text += M_debug_text[i];
+            //M_debug_level_act[i]->setText( text );
+            //M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + text );
+            M_debug_level_act[i]->setText( M_debug_text[i] );
+            M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + M_debug_text[i] );
         }
     }
 
@@ -248,8 +318,13 @@ DebugMessageWindow::readSettings()
 void
 DebugMessageWindow::saveSettings()
 {
+#ifndef Q_WS_WIN
     QSettings settings( QDir::homePath() + "/.soccerwindow2-qt4",
                         QSettings::IniFormat );
+#else
+    QSettings settings( QDir::currentPath() + "/soccerwindow2.ini",
+                        QSettings::IniFormat );
+#endif
     //QSettings settings( QSettings::IniFormat,
     //                    QSettings::UserScope,
     //                    PACKAGE_NAME );
@@ -260,31 +335,30 @@ DebugMessageWindow::saveSettings()
     settings.setValue( tr( "debug_view_all" ),  M_show_debug_view_all_act->isChecked() );
     settings.setValue( tr( "debug_view_self" ), M_show_debug_view_self_act->isChecked() );
     settings.setValue( tr( "debug_view_ball" ), M_show_debug_view_ball_act->isChecked() );
-    settings.setValue( tr( "debug_view_players" ), M_show_debug_view_players_act->isChecked() );
+    settings.setValue( tr( "debug_view_teammates" ), M_show_debug_view_teammates_act->isChecked() );
+    settings.setValue( tr( "debug_view_opponents" ), M_show_debug_view_opponents_act->isChecked() );
     settings.setValue( tr( "debug_view_comment" ), M_show_debug_view_comment_act->isChecked() );
-    settings.setValue( tr( "debug_view_figures" ), M_show_debug_view_figure_act->isChecked() );
+    settings.setValue( tr( "debug_view_shape" ), M_show_debug_view_shape_act->isChecked() );
     settings.setValue( tr( "debug_view_target" ), M_show_debug_view_target_act->isChecked() );
     settings.setValue( tr( "debug_view_message" ), M_show_debug_view_message_act->isChecked() );
     settings.setValue( tr( "debug_log_objects" ), M_show_debug_log_objects_act->isChecked() );
 
     // save debug level switches
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
-        char key[16];
-        std::snprintf( key, 16, "debug%02d", i + 1 );
-
-        settings.setValue( QString::fromAscii( key ),
+        //snprintf( key, 16, "debug%02d", i + 1 );
+        settings.setValue( QString( "debug%1" ).arg( i + 1, 2, 10, QChar( '0' ) ),
                            M_debug_level_act[i]->isChecked() );
     }
 
     // save debug level text
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
         if ( ! M_debug_text[i].isEmpty() )
         {
-            char key[32];
-            std::snprintf( key, 16, "debug_text_%02d", i + 1 );
-            settings.setValue( QString::fromAscii( key ), M_debug_text[i] );
+            //snprintf( key, 32, "debug_text_%02d", i + 1 );
+            settings.setValue( QString( "debug_text_%1" ).arg( i + 1, 2, 10, QChar( '0' ) ),
+                               M_debug_text[i] );
         }
     }
 
@@ -324,13 +398,21 @@ DebugMessageWindow::createActions()
     connect( M_show_debug_view_ball_act, SIGNAL( triggered( bool ) ),
              this, SLOT( toggleShowDebugViewBall( bool ) ) );
 
-    M_show_debug_view_players_act = new QAction( QIcon( QPixmap( debug_view_player_xpm ) ),
-                                              tr( "Toggle debug view players" ), this );
-    M_show_debug_view_players_act->setStatusTip( tr( "Show/Hide debug view players" ) );
-    M_show_debug_view_players_act->setCheckable( true );
-    M_show_debug_view_players_act->setChecked( true );
-    connect( M_show_debug_view_players_act, SIGNAL( triggered( bool ) ),
-             this, SLOT( toggleShowDebugViewPlayers( bool ) ) );
+    M_show_debug_view_teammates_act = new QAction( QIcon( QPixmap( debug_view_teammate_xpm ) ),
+                                                   tr( "Toggle debug view teammates" ), this );
+    M_show_debug_view_teammates_act->setStatusTip( tr( "Show/Hide debug view teammates" ) );
+    M_show_debug_view_teammates_act->setCheckable( true );
+    M_show_debug_view_teammates_act->setChecked( true );
+    connect( M_show_debug_view_teammates_act, SIGNAL( triggered( bool ) ),
+             this, SLOT( toggleShowDebugViewTeammates( bool ) ) );
+
+    M_show_debug_view_opponents_act = new QAction( QIcon( QPixmap( debug_view_opponent_xpm ) ),
+                                                   tr( "Toggle debug view opponents" ), this );
+    M_show_debug_view_opponents_act->setStatusTip( tr( "Show/Hide debug view opponents" ) );
+    M_show_debug_view_opponents_act->setCheckable( true );
+    M_show_debug_view_opponents_act->setChecked( true );
+    connect( M_show_debug_view_opponents_act, SIGNAL( triggered( bool ) ),
+             this, SLOT( toggleShowDebugViewOpponents( bool ) ) );
 
     M_show_debug_view_comment_act = new QAction( QIcon( QPixmap( debug_view_comment_xpm ) ),
                                               tr( "Toggle debug view comment" ), this );
@@ -340,13 +422,13 @@ DebugMessageWindow::createActions()
     connect( M_show_debug_view_comment_act, SIGNAL( triggered( bool ) ),
              this, SLOT( toggleShowDebugViewComment( bool ) ) );
 
-    M_show_debug_view_figure_act = new QAction( QIcon( QPixmap( debug_view_line_xpm ) ),
-                                              tr( "Toggle debug view figures" ), this );
-    M_show_debug_view_figure_act->setStatusTip( tr( "Show/Hide debug view figures" ) );
-    M_show_debug_view_figure_act->setCheckable( true );
-    M_show_debug_view_figure_act->setChecked( true );
-    connect( M_show_debug_view_figure_act, SIGNAL( triggered( bool ) ),
-             this, SLOT( toggleShowDebugViewFigure( bool ) ) );
+    M_show_debug_view_shape_act = new QAction( QIcon( QPixmap( debug_view_line_xpm ) ),
+                                              tr( "Toggle debug view shape" ), this );
+    M_show_debug_view_shape_act->setStatusTip( tr( "Show/Hide debug view shape" ) );
+    M_show_debug_view_shape_act->setCheckable( true );
+    M_show_debug_view_shape_act->setChecked( true );
+    connect( M_show_debug_view_shape_act, SIGNAL( triggered( bool ) ),
+             this, SLOT( toggleShowDebugViewShape( bool ) ) );
 
     M_show_debug_view_target_act = new QAction( QIcon( QPixmap( debug_view_target_xpm ) ),
                                               tr( "Toggle debug view target" ), this );
@@ -365,10 +447,9 @@ DebugMessageWindow::createActions()
              this, SLOT( toggleShowDebugViewMessage( bool ) ) );
     //////////////////////////////////////////////
     // debug log view
-    M_show_debug_log_objects_act
-        = new QAction( QIcon( QPixmap( debug_log_view_xpm ) ),
-                       tr( "Toggle debug log objects" ),
-                       this );
+    M_show_debug_log_objects_act = new QAction( QIcon( QPixmap( debug_log_view_xpm ) ),
+                                                tr( "Toggle debug log objects" ),
+                                                this );
     M_show_debug_log_objects_act->setStatusTip( tr( "Show/Hide debug log objects" ) );
     M_show_debug_log_objects_act->setCheckable( true );
     M_show_debug_log_objects_act->setChecked( true );
@@ -380,22 +461,28 @@ DebugMessageWindow::createActions()
     M_open_debug_log_dir_act = new QAction( QIcon( QPixmap( open_dir_xpm ) ),
                                             tr( "Open log dir..." ), this );
 #ifdef Q_WS_MAC
-    M_open_debug_log_dir_act->setShortcut( tr( "Meta+O" ) );
+    M_open_debug_log_dir_act->setShortcut( Qt::META + Qt::Key_O );
 #else
-    M_open_debug_log_dir_act->setShortcut( tr( "Ctrl+O" ) );
+    M_open_debug_log_dir_act->setShortcut( Qt::CTRL + Qt::Key_O );
 #endif
     M_open_debug_log_dir_act->setStatusTip( tr( "Open the debug log directory" ) );
     connect( M_open_debug_log_dir_act, SIGNAL( triggered() ),
              this, SLOT( showDebugLogDirDialog() ) );
+    //
+    M_clear_data_act = new QAction( QIcon( QPixmap( clear_xpm ) ),
+                                    tr( "Clear data" ), this );
+    M_clear_data_act->setStatusTip( tr( "Clear all data" ) );
+    connect( M_clear_data_act, SIGNAL( triggered() ),
+             this, SLOT( clearAll() ) );
 
     //////////////////////////////////////////////
     // cycle control
     M_sync_act = new QAction( QIcon( QPixmap( sync_xpm ) ),
                               tr( "Sync" ), this );
 #ifdef Q_WS_MAC
-    M_sync_act->setShortcut( tr( "Meta+S" ) );
+    M_sync_act->setShortcut( Qt::META + Qt::Key_S );
 #else
-    M_sync_act->setShortcut( tr( "Ctrl+S" ) );
+    M_sync_act->setShortcut( Qt::CTRL + Qt::Key_S );
 #endif
     M_sync_act->setStatusTip( tr( "Synchronize with field canvas" ) );
     connect( M_sync_act, SIGNAL( triggered() ),
@@ -406,7 +493,7 @@ DebugMessageWindow::createActions()
 #ifdef Q_WS_MAC
     M_decrement_act->setShortcut( Qt::META + Qt::Key_Left );
 #else
-    M_decrement_act->setShortcut( Qt::META + Qt::Key_Left );
+    M_decrement_act->setShortcut( Qt::CTRL + Qt::Key_Left );
 #endif
     M_decrement_act->setStatusTip( tr( "Decrement message cycle" ) );
     connect( M_decrement_act, SIGNAL( triggered() ),
@@ -417,7 +504,7 @@ DebugMessageWindow::createActions()
 #ifdef Q_WS_MAC
     M_increment_act->setShortcut( Qt::META + Qt::Key_Right );
 #else
-    M_increment_act->setShortcut( Qt::META + Qt::Key_Right );
+    M_increment_act->setShortcut( Qt::CTRL + Qt::Key_Right );
 #endif
     M_increment_act->setStatusTip( tr( "Increment message cycle" ) );
     connect( M_increment_act, SIGNAL( triggered() ),
@@ -425,21 +512,34 @@ DebugMessageWindow::createActions()
 
     //////////////////////////////////////////////
     // debug level switches
-    char ** num_xpm[16] = { num_1_xpm, num_2_xpm, num_3_xpm, num_4_xpm,
-                            num_5_xpm, num_6_xpm, num_7_xpm, num_8_xpm,
-                            num_9_xpm, num_10_xpm, num_11_xpm, num_12_xpm,
-                            num_13_xpm, num_14_xpm, num_15_xpm, num_16_xpm };
+    const char ** num_xpm[] = { num_1_xpm, num_2_xpm, num_3_xpm, num_4_xpm,
+                                num_5_xpm, num_6_xpm, num_7_xpm, num_8_xpm,
+                                num_9_xpm, num_10_xpm, num_11_xpm, num_12_xpm,
+                                num_13_xpm, num_14_xpm, num_15_xpm, num_16_xpm,
+                                num_17_xpm, num_18_xpm, num_19_xpm, num_20_xpm,
+                                num_21_xpm, num_22_xpm, num_23_xpm, num_24_xpm,
+                                num_25_xpm, num_26_xpm, num_27_xpm, num_28_xpm,
+                                num_29_xpm, num_30_xpm, num_31_xpm, num_32_xpm };
 
     QSignalMapper * debug_level_mapper = new QSignalMapper( this );
     connect( debug_level_mapper, SIGNAL( mapped( int ) ),
              this, SLOT( toggleDebugLevel( int ) ) );
 
-    for ( int i = 0; i < 16; ++i )
+    for ( int i = 0; i < 32; ++i )
     {
-        QString menu_text = tr( "Level %1 " ).arg( i + 1 );
-        menu_text += M_debug_text[i];
-        M_debug_level_act[i] = new QAction( QIcon( QPixmap( num_xpm[i] ) ),
-                                            menu_text, this );
+        QString menu_text;
+        if ( M_debug_text[i].isEmpty() )
+        {
+            menu_text = tr( "Level %1 " ).arg( i + 1 );
+            M_debug_level_act[i] = new QAction( QIcon( QPixmap( num_xpm[i] ) ),
+                                                menu_text, this );
+        }
+        else
+        {
+            menu_text = M_debug_text[i];
+            M_debug_level_act[i] = new QAction( menu_text, this );
+        }
+
         M_debug_level_act[i]->setStatusTip( tr( "Debug " ) + menu_text );
         M_debug_level_act[i]->setCheckable( true );
 
@@ -484,10 +584,11 @@ void
 DebugMessageWindow::createToolBar()
 {
     {
-        QToolBar * tbar = new QToolBar( this );
+        QToolBar * tbar = new QToolBar( tr( "Control" ), this );
         tbar->setIconSize( QSize( 16, 16 ) );
 
         tbar->addAction( M_open_debug_log_dir_act );
+        tbar->addAction( M_clear_data_act );
 
         tbar->addSeparator();
 
@@ -508,9 +609,9 @@ DebugMessageWindow::createToolBar()
         {
             QAction * act = new QAction( tr( "Focus Find Box" ), this );
 #ifdef Q_WS_MAC
-            act->setShortcut( tr( "Meta+F" ) );
+            act->setShortcut( Qt::META + Qt::Key_F );
 #else
-            act->setShortcut( tr( "Ctrl+F" ) );
+            act->setShortcut( Qt::CTRL + Qt::Key_F );
 #endif
             connect( act, SIGNAL( triggered() ),
                      M_find_box, SLOT( setFocus() ) );
@@ -543,15 +644,16 @@ DebugMessageWindow::createToolBar()
     }
 
     {
-        QToolBar * tbar = new QToolBar( this );
+        QToolBar * tbar = new QToolBar( tr( "Debug view" ), this );
         tbar->setIconSize( QSize( 16, 16 ) );
 
         tbar->addAction( M_show_debug_view_all_act );
         tbar->addAction( M_show_debug_view_self_act );
         tbar->addAction( M_show_debug_view_ball_act );
-        tbar->addAction( M_show_debug_view_players_act );
+        tbar->addAction( M_show_debug_view_teammates_act );
+        tbar->addAction( M_show_debug_view_opponents_act );
         tbar->addAction( M_show_debug_view_comment_act );
-        tbar->addAction( M_show_debug_view_figure_act );
+        tbar->addAction( M_show_debug_view_shape_act );
         tbar->addAction( M_show_debug_view_target_act );
         tbar->addAction( M_show_debug_view_message_act );
 
@@ -559,12 +661,33 @@ DebugMessageWindow::createToolBar()
 
         tbar->addAction( M_show_debug_log_objects_act );
 
-        for ( int i = 0; i < 16; ++i )
+        this->addToolBar( Qt::TopToolBarArea, tbar );
+    }
+
+    const int max_visible_level = 18;
+    // left tool bar
+    {
+        QToolBar * tbar = new QToolBar( tr( "Debug Level" ), this );
+        tbar->setIconSize( QSize( 16, 16 ) );
+        for ( int i = 0; i < max_visible_level; ++i )
         {
             tbar->addAction( M_debug_level_act[i] );
         }
 
-        this->addToolBar( Qt::BottomToolBarArea, tbar );
+        this->addToolBar( Qt::LeftToolBarArea, tbar );
+    }
+    //this->addToolBarBreak( Qt::LeftToolBarArea );
+    {
+        QToolBar * tbar = new QToolBar( tr( "Debug Level Hidden" ), this );
+        tbar->setIconSize( QSize( 16, 16 ) );
+
+        for ( int i = max_visible_level; i < 32; ++i )
+        {
+            tbar->addAction( M_debug_level_act[i] );
+        }
+
+        this->addToolBar( Qt::LeftToolBarArea, tbar );
+        tbar->hide();
     }
 }
 
@@ -576,7 +699,7 @@ void
 DebugMessageWindow::openDebugLogDir( const rcsc::SideID side,
                                      const std::string & dir_path )
 {
-    MonitorViewConstPtr ptr = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr ptr = M_main_data.getCurrentViewData();
     if ( ! ptr )
     {
         std::cerr << "No valid data!" << std::endl;
@@ -629,9 +752,9 @@ DebugMessageWindow::changeCurrentTab( int index )
 void
 DebugMessageWindow::toggleShowDebugViewAll( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugView() != checked )
+    if ( Options::instance().showDebugView() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugView();
+        Options::instance().toggleShowDebugView();
         emit configured();
     }
 }
@@ -643,9 +766,9 @@ DebugMessageWindow::toggleShowDebugViewAll( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewSelf( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewSelf() != checked )
+    if ( Options::instance().showDebugViewSelf() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewSelf();
+        Options::instance().toggleShowDebugViewSelf();
         emit configured();
     }
 }
@@ -657,9 +780,9 @@ DebugMessageWindow::toggleShowDebugViewSelf( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewBall( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewBall() != checked )
+    if ( Options::instance().showDebugViewBall() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewBall();
+        Options::instance().toggleShowDebugViewBall();
         emit configured();
     }
 }
@@ -669,11 +792,25 @@ DebugMessageWindow::toggleShowDebugViewBall( bool checked )
 
 */
 void
-DebugMessageWindow::toggleShowDebugViewPlayers( bool checked )
+DebugMessageWindow::toggleShowDebugViewTeammates( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewPlayers() != checked )
+    if ( Options::instance().showDebugViewTeammates() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewPlayers();
+        Options::instance().toggleShowDebugViewTeammates();
+        emit configured();
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+DebugMessageWindow::toggleShowDebugViewOpponents( bool checked )
+{
+    if ( Options::instance().showDebugViewOpponents() != checked )
+    {
+        Options::instance().toggleShowDebugViewOpponents();
         emit configured();
     }
 }
@@ -685,9 +822,9 @@ DebugMessageWindow::toggleShowDebugViewPlayers( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewComment( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewComment() != checked )
+    if ( Options::instance().showDebugViewComment() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewComment();
+        Options::instance().toggleShowDebugViewComment();
         emit configured();
     }
 }
@@ -697,11 +834,11 @@ DebugMessageWindow::toggleShowDebugViewComment( bool checked )
 
 */
 void
-DebugMessageWindow::toggleShowDebugViewFigure( bool checked )
+DebugMessageWindow::toggleShowDebugViewShape( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewFigure() != checked )
+    if ( Options::instance().showDebugViewShape() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewFigure();
+        Options::instance().toggleShowDebugViewShape();
         emit configured();
     }
 }
@@ -713,9 +850,9 @@ DebugMessageWindow::toggleShowDebugViewFigure( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewTarget( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewTarget() != checked )
+    if ( Options::instance().showDebugViewTarget() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewTarget();
+        Options::instance().toggleShowDebugViewTarget();
         emit configured();
     }
 }
@@ -727,9 +864,9 @@ DebugMessageWindow::toggleShowDebugViewTarget( bool checked )
 void
 DebugMessageWindow::toggleShowDebugViewMessage( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugViewMessage() != checked )
+    if ( Options::instance().showDebugViewMessage() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugViewMessage();
+        Options::instance().toggleShowDebugViewMessage();
         emit configured();
     }
 }
@@ -741,9 +878,9 @@ DebugMessageWindow::toggleShowDebugViewMessage( bool checked )
 void
 DebugMessageWindow::toggleShowDebugLogObjects( bool checked )
 {
-    if ( M_main_data.viewConfig().isShownDebugLogObjects() != checked )
+    if ( Options::instance().showDebugLogObjects() != checked )
     {
-        M_main_data.getViewConfig().toggleShowDebugLogObjects();
+        Options::instance().toggleShowDebugLogObjects();
         emit configured();
     }
 }
@@ -772,7 +909,7 @@ DebugMessageWindow::showDebugLogDirDialog()
               << "  debug log dir = [" << dir_path_std << "]"
               << std::endl;
 
-    AppConfig::instance().setDebugLogDir( dir_path_std );
+    Options::instance().setDebugLogDir( dir_path_std );
 
     openDebugLogDir( side, dir_path_std );
 }
@@ -784,10 +921,10 @@ DebugMessageWindow::showDebugLogDirDialog()
 void
 DebugMessageWindow::syncCycle()
 {
-    MonitorViewConstPtr ptr = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr ptr = M_main_data.getCurrentViewData();
     if ( ! ptr )
     {
-        std::cerr << "sync debug cycle. no view data." << std::endl;
+        //std::cerr << "sync debug cycle. no view data." << std::endl;
         return;
     }
 
@@ -885,7 +1022,7 @@ DebugMessageWindow::toggleDebugLevel( int level )
 {
     //M_level ^= DebugLogData::LEVEL_1; // reverse only 1 bit
 
-    if ( level < 1 || 16 < level )
+    if ( level < 1 || 32 < level )
     {
         std::cerr << "Invalid level " << level << std::endl;
         return;
@@ -939,7 +1076,7 @@ DebugMessageWindow::updateMessage()
         return;
     }
 
-    const long cur_cycle = M_main_data.getDebugLogHolder().getCurrentCycle();
+    const int cur_cycle = M_main_data.getDebugLogHolder().getCurrentCycle();
 
     QTextEdit * text_edit = M_message[ M_tab_widget->currentIndex() ];
 
