@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -39,6 +39,7 @@
 
 #include "draw_config.h"
 // model
+#include "options.h"
 #include "main_data.h"
 
 #include <rcsc/common/server_param.h>
@@ -46,6 +47,98 @@
 #include <iostream>
 #include <cmath>
 
+namespace {
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+void
+draw_communication( QPainter & painter,
+                    const rcsc::SideID self_side,
+                    const MonitorViewData & monitor_view,
+                    const DebugViewCont & debug_view_cont,
+                    const DebugViewData & debug_view )
+{
+    const Options & opt = Options::instance();
+    const DrawConfig & dconf = DrawConfig::instance();
+
+    const double reverse = ( self_side == rcsc::RIGHT
+                             ? -1.0
+                             : 1.0 );
+
+    const int say_r = std::max( 3, opt.scale( 3.5 ) );
+    const int hear_r = std::max( 2, opt.scale( 3.0 ) );
+
+    //
+    // draw say message sender marks
+    //
+
+    painter.setPen( DrawConfig::instance().debugCommentFontPen() );
+    painter.setBrush( DrawConfig::instance().transparentBrush() );
+
+    for ( size_t i = 0; i < debug_view_cont.size(); ++i )
+    {
+        const DebugViewPtr v = debug_view_cont[i];
+        if ( ! v ) continue;
+        if ( v->sayMessage().empty() ) continue;
+
+        const boost::shared_ptr< DebugViewData::SelfT > self = v->self();
+        if ( ! self ) continue;
+
+        const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+
+        painter.drawEllipse( ix - say_r, iy - say_r, say_r * 2, say_r * 2 );
+    }
+
+    //
+    // draw heard message sender marks
+    //
+    if ( ! debug_view.hearMessages().empty() )
+    {
+        const boost::shared_ptr< DebugViewData::SelfT > self = debug_view.self();
+        if ( ! self )
+        {
+            return;
+        }
+
+        const int self_x = opt.screenX( self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int self_y = opt.screenY( self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+
+        const std::vector< Player > & players = monitor_view.players();
+        const std::vector< Player >::const_iterator p_end = players.end();
+
+        //painter.setPen( dconf.debugShapePen() );
+        painter.setPen( Qt::darkMagenta );
+        painter.setBrush( dconf.transparentBrush() );
+
+        for ( std::map< int, std::string >::const_iterator it = debug_view.hearMessages().begin();
+              it != debug_view.hearMessages().end();
+              ++it )
+        {
+            for ( std::vector< Player >::const_iterator p = players.begin();
+                  p != p_end;
+                  ++p )
+            {
+                if ( p->side() == self_side
+                     && p->unum() == it->first )
+                {
+                    int px = opt.screenX( p->x() );
+                    int py = opt.screenY( p->y() );
+
+                    painter.drawEllipse( px - hear_r, py - hear_r,
+                                         hear_r * 2, hear_r * 2 );
+                    painter.drawLine( px, py, self_x, self_y );
+                    break;
+                }
+            }
+        }
+    }
+}
+
+}
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -53,14 +146,14 @@
 void
 DebugPainter::draw( QPainter & painter )
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
 
-    if ( ! vconf.isShownDebugView() )
+    if ( ! opt.showDebugView() )
     {
         return;
     }
 
-    int number = vconf.selectedNumber();
+    int number = opt.selectedNumber();
     if ( number == 0 )
     {
         return;
@@ -68,20 +161,18 @@ DebugPainter::draw( QPainter & painter )
 
     const rcsc::SideID self_side = ( number < 0 ? rcsc::RIGHT : rcsc::LEFT );
 
-    const std::map< long, DebugViewCont > & view_map
+    const std::map< int, DebugViewCont > & view_map
         = ( self_side == rcsc::RIGHT
             ? M_main_data.viewHolder().rightDebugView()
             : M_main_data.viewHolder().leftDebugView() );
 
-    MonitorViewConstPtr monitor_view
-        = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr monitor_view = M_main_data.getCurrentViewData();
     if ( ! monitor_view )
     {
         return;
     }
 
-    std::map< long, DebugViewCont >::const_iterator it
-        = view_map.find( monitor_view->cycle() );
+    std::map< int, DebugViewCont >::const_iterator it = view_map.find( monitor_view->cycle() );
 
     if ( it == view_map.end() )
     {
@@ -98,6 +189,10 @@ DebugPainter::draw( QPainter & painter )
         return;
     }
 
+    //
+    // draw selected player's debug data
+    //
+
     const DebugViewPtr view = it->second[ number - 1 ];
     if ( ! view )
     {
@@ -111,34 +206,44 @@ DebugPainter::draw( QPainter & painter )
     const DrawConfig & dconf = DrawConfig::instance();
 
     // draw self
-    if ( vconf.isShownDebugViewSelf() )
+    if ( opt.showDebugViewSelf() )
     {
         drawSelf( painter, self_side, *view );
     }
 
     // draw players
-    if ( vconf.isShownDebugViewPlayers() )
+    if ( opt.showDebugViewTeammates() )
     {
         drawPlayers( painter,
                      self_side,
                      *view,
                      view->teammates(),
-                     ( vconf.isShownDebugViewTarget()
+                     ( opt.showDebugViewTarget()
                        ? view->targetTeammate()
                        : -1 ),
                      dconf.debugTeammateBrush() );
-        drawPlayers( painter,
-                     self_side,
-                     *view,
-                     view->opponents(),
-                     -1,
-                     dconf.debugOpponentBrush() );
         drawPlayers( painter,
                      self_side,
                      *view,
                      view->unknownTeammates(),
                      -1,
                      dconf.debugUnknownTeammateBrush() );
+
+        draw_communication( painter,
+                            self_side,
+                            *monitor_view,
+                            it->second,
+                            *view );
+    }
+
+    if ( opt.showDebugViewOpponents() )
+    {
+        drawPlayers( painter,
+                     self_side,
+                     *view,
+                     view->opponents(),
+                     -1,
+                     dconf.debugOpponentBrush() );
         drawPlayers( painter,
                      self_side,
                      *view,
@@ -154,25 +259,44 @@ DebugPainter::draw( QPainter & painter )
     }
 
     // draw ball
-    if ( vconf.isShownDebugViewBall()
+    if ( opt.showDebugViewBall()
          && view->ball() )
     {
         drawBall( painter, self_side, *view );
     }
 
-    // draw lines
-    if ( vconf.isShownDebugViewFigure() )
+    // draw shapes
+    if ( opt.showDebugViewShape() )
     {
+        if ( opt.antiAliasing() )
+        {
+            painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                                   //QPainter::HighQualityAntialiasing |
+#endif
+                                   QPainter::Antialiasing,
+                                   false );
+        }
+
         drawLines( painter, self_side, *view );
         drawTriangles( painter, self_side, *view );
         drawRectangles( painter, self_side, *view );
         drawCircles( painter, self_side, *view );
+
+        if ( opt.antiAliasing() )
+        {
+            painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                                   //QPainter::HighQualityAntialiasing |
+#endif
+                                   QPainter::Antialiasing,
+                                   true );
+        }
     }
 
-    if ( vconf.isShownDebugViewMessage()
-         && ! view->message().empty() )
+    if ( opt.showDebugViewMessage() )
     {
-        drawMessage( painter, view->message() );
+        drawMessage( painter, *view );
     }
 }
 
@@ -192,25 +316,25 @@ DebugPainter::drawSelf( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
-    const int ix = vconf.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-    const int iy = vconf.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const int ix = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+    const int iy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
     const int r = std::max( 2,
-                            ( vconf.isEnlarged()
-                              ? vconf.scale( 1.0 )
-                              : vconf.scale( 0.3 ) ) );
+                            ( opt.enlargeMode()
+                              ? opt.scale( 1.0 )
+                              : opt.scale( 0.3 ) ) );
 
     // draw targets
-    if ( vconf.isShownDebugViewTarget()
+    if ( opt.showDebugViewTarget()
          &&  view.targetPoint() )
     {
-        int tx = vconf.screenX( view.targetPoint()->x * reverse );
-        int ty = vconf.screenY( view.targetPoint()->y * reverse );
+        int tx = opt.screenX( view.targetPoint()->x * reverse );
+        int ty = opt.screenY( view.targetPoint()->y * reverse );
 
         painter.setPen( dconf.debugTargetPen() );
         painter.setBrush( dconf.transparentBrush() );
@@ -223,7 +347,7 @@ DebugPainter::drawSelf( QPainter & painter,
 
     // draw body half circle
     double body_start_dir = - ( (double)self->body_ + 90.0 );
-    if ( vconf.reverseSide() ) body_start_dir += 180.0;
+    if ( opt.reverseSide() ) body_start_dir += 180.0;
     if ( self_side == rcsc::RIGHT ) body_start_dir -= 180.0;
 
     painter.setPen( dconf.transparentPen() );
@@ -247,7 +371,7 @@ DebugPainter::drawSelf( QPainter & painter,
                          r * 2 );
 
     // draw comment
-    if ( vconf.isShownDebugViewComment()
+    if ( opt.showDebugViewComment()
          && ! self->comment_.empty() )
     {
         painter.setFont( dconf.debugCommentFont() );
@@ -275,7 +399,7 @@ DebugPainter::drawBall( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
@@ -288,12 +412,12 @@ DebugPainter::drawBall( QPainter & painter,
                          (double)ball->vy_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
     const double ball_decay = rcsc::ServerParam::i().ballDecay();
-    const int ix = vconf.screenX( bpos.x );
-    const int iy = vconf.screenY( bpos.y );
+    const int ix = opt.screenX( bpos.x );
+    const int iy = opt.screenY( bpos.y );
     const int r = std::max( 2,
-                            ( vconf.isEnlarged()
-                              ? vconf.scale( 0.25 )
-                              : vconf.scale( 0.07 ) ) );
+                            ( opt.enlargeMode()
+                              ? opt.scale( 0.25 )
+                              : opt.scale( 0.07 ) ) );
 
     painter.setPen( dconf.transparentPen() );
     painter.setBrush( dconf.debugBallBrush() );
@@ -303,6 +427,36 @@ DebugPainter::drawBall( QPainter & painter,
                          r * 2,
                          r * 2 );
 
+
+    if ( opt.antiAliasing() )
+    {
+        painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                               //QPainter::HighQualityAntialiasing |
+#endif
+                               QPainter::Antialiasing,
+                               false );
+    }
+
+#ifdef USE_GLWIDGET
+    {
+        painter.setPen( dconf.debugBallTracePen() );
+        painter.setBrush( dconf.debugBallTraceBrush() );
+
+        int bx = ix;
+        int by = iy;
+        for ( int i = 0; i < 10; ++i )
+        {
+            bpos += bvel;
+            bvel *= ball_decay;
+            bx = opt.screenX( bpos.x );
+            by = opt.screenY( bpos.y );
+            painter.drawEllipse( bx - 2, by - 2, 4, 4 );
+        }
+
+        painter.drawLine( ix, iy, bx, by );
+    }
+#else
     {
         QPainterPath path;
         painter.setPen( dconf.debugBallTracePen() );
@@ -314,8 +468,8 @@ DebugPainter::drawBall( QPainter & painter,
         {
             bpos += bvel;
             bvel *= ball_decay;
-            bx = vconf.screenX( bpos.x );
-            by = vconf.screenY( bpos.y );
+            bx = opt.screenX( bpos.x );
+            by = opt.screenY( bpos.y );
             path.addEllipse( bx, by, 4, 4 );
         }
         path.moveTo( ix, iy );
@@ -323,9 +477,21 @@ DebugPainter::drawBall( QPainter & painter,
 
         painter.drawPath( path );
     }
+#endif
+
+
+    if ( opt.antiAliasing() )
+    {
+        painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                               //QPainter::HighQualityAntialiasing |
+#endif
+                               QPainter::Antialiasing,
+                               true );
+    }
 
     // draw comment
-    if ( vconf.isShownDebugViewComment()
+    if ( opt.showDebugViewComment()
          && ! ball->comment_.empty() )
     {
         painter.setFont( dconf.debugCommentFont() );
@@ -349,30 +515,30 @@ DebugPainter::drawPlayers( QPainter & painter,
                            const int target_number,
                            const QBrush & body_brush ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
-    const bool comment = vconf.isShownDebugViewComment();
+    const bool comment = opt.showDebugViewComment();
     const int r = std::max( 2,
-                            ( vconf.isEnlarged()
-                              ? vconf.scale( 1.0 )
-                              : vconf.scale( 0.3 ) ) );
+                            ( opt.enlargeMode()
+                              ? opt.scale( 1.0 )
+                              : opt.scale( 0.3 ) ) );
 
     const DebugViewData::PlayerCont::const_iterator end = players.end();
     for ( DebugViewData::PlayerCont::const_iterator it = players.begin();
           it != end;
           ++it )
     {
-        const int ix = vconf.screenX( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-        const int iy = vconf.screenY( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int ix = opt.screenX( (double)(*it)->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+        const int iy = opt.screenY( (double)(*it)->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
 
         if ( (*it)->body_ != -360 )
         {
             double body_start_dir = - ( (double)(*it)->body_ + 90.0 );
-            if ( vconf.reverseSide() ) body_start_dir += 180.0;
+            if ( opt.reverseSide() ) body_start_dir += 180.0;
             if ( self_side == rcsc::RIGHT ) body_start_dir -= 180.0;
             //double body_end_dir = body_start_dir + 180.0;
             // draw body half circle
@@ -412,8 +578,8 @@ DebugPainter::drawPlayers( QPainter & painter,
             const boost::shared_ptr< DebugViewData::SelfT > self = view.self();
             if ( self )
             {
-                const int sx = vconf.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
-                const int sy = vconf.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+                const int sx = opt.screenX( (double)self->x_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
+                const int sy = opt.screenY( (double)self->y_ / rcsc::rcg::SHOWINFO_SCALE2 * reverse );
                 painter.drawLine( sx, sy, ix, iy );
             }
 
@@ -449,16 +615,28 @@ DebugPainter::drawLines( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
 
-    painter.setPen( dconf.debugFigurePen() );
+    painter.setPen( dconf.debugShapePen() );
     painter.setBrush( dconf.transparentBrush() );
 
+#ifdef USE_GL_WIDGET
+    const std::list< DebugViewData::LineT >::const_iterator end = view.lines().end();
+    for ( std::list< DebugViewData::LineT >::const_iterator it = view.lines().begin();
+          it != end;
+          ++it )
+    {
+        painter.drawLine( opt.screenX( it->x1_ * reverse ),
+                          opt.screenY( it->y1_ * reverse ),
+                          opt.screenX( it->x2_ * reverse ),
+                          opt.screenY( it->y2_ * reverse ) );
+    }
+#else
     QPainterPath path;
 
     const std::list< DebugViewData::LineT >::const_iterator end = view.lines().end();
@@ -466,13 +644,14 @@ DebugPainter::drawLines( QPainter & painter,
           it != end;
           ++it )
     {
-        path.moveTo( vconf.screenX( it->x1_ * reverse ),
-                     vconf.screenY( it->y1_ * reverse ) );
-        path.lineTo( vconf.screenX( it->x2_ * reverse ),
-                     vconf.screenY( it->y2_ * reverse ) );
+        path.moveTo( opt.screenX( it->x1_ * reverse ),
+                     opt.screenY( it->y1_ * reverse ) );
+        path.lineTo( opt.screenX( it->x2_ * reverse ),
+                     opt.screenY( it->y2_ * reverse ) );
     }
 
     painter.drawPath( path );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -489,16 +668,31 @@ DebugPainter::drawTriangles( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
 
-    painter.setPen( dconf.debugFigurePen() );
+    painter.setPen( dconf.debugShapePen() );
     painter.setBrush( dconf.transparentBrush() );
 
+#ifdef USE_GLWIDGET
+    const std::list< DebugViewData::TriangleT >::const_iterator end = view.triangles().end();
+    for ( std::list< DebugViewData::TriangleT >::const_iterator it = view.triangles().begin();
+          it != end;
+          ++it )
+    {
+        QPoint points[3] = { QPoint( opt.screenX( it->x1_ * reverse ),
+                                     opt.screenY( it->y1_ * reverse ) ),
+                             QPoint( opt.screenX( it->x2_ * reverse ),
+                                     opt.screenY( it->y2_ * reverse ) ),
+                             QPoint( opt.screenX( it->x3_ * reverse ),
+                                     opt.screenY( it->y3_ * reverse ) ) };
+        painter.drawLines( points, 3 );
+    }
+#else
     QPainterPath path;
 
     const std::list< DebugViewData::TriangleT >::const_iterator end = view.triangles().end();
@@ -506,12 +700,12 @@ DebugPainter::drawTriangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int x1 = vconf.screenX( it->x1_ * reverse );
-        int y1 = vconf.screenY( it->y1_ * reverse );
-        int x2 = vconf.screenX( it->x2_ * reverse );
-        int y2 = vconf.screenY( it->y2_ * reverse );
-        int x3 = vconf.screenX( it->x3_ * reverse );
-        int y3 = vconf.screenY( it->y3_ * reverse );
+        int x1 = opt.screenX( it->x1_ * reverse );
+        int y1 = opt.screenY( it->y1_ * reverse );
+        int x2 = opt.screenX( it->x2_ * reverse );
+        int y2 = opt.screenY( it->y2_ * reverse );
+        int x3 = opt.screenX( it->x3_ * reverse );
+        int y3 = opt.screenY( it->y3_ * reverse );
 
         path.moveTo( x1, y1 );
         path.lineTo( x2, y2 );
@@ -520,6 +714,7 @@ DebugPainter::drawTriangles( QPainter & painter,
     }
 
     painter.drawPath( path );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -536,16 +731,33 @@ DebugPainter::drawRectangles( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
 
-    painter.setPen( dconf.debugFigurePen() );
+    painter.setPen( dconf.debugShapePen() );
     painter.setBrush( dconf.transparentBrush() );
 
+#ifdef USE_GLWIDGET
+    const std::list< DebugViewData::RectT >::const_iterator end = view.rectangles().end();
+    for ( std::list< DebugViewData::RectT >::const_iterator it = view.rectangles().begin();
+          it != end;
+          ++it )
+    {
+        int left_x = opt.screenX( it->left_x_ * reverse );
+        int top_y = opt.screenY( it->top_y_ * reverse );
+        int right_x = opt.screenX( it->right_x_ * reverse );
+        int bottom_y = opt.screenY( it->bottom_y_ * reverse );
+
+        painter.drawRect( left_x,
+                          top_y,
+                          ( right_x - left_x ),
+                          ( bottom_y - top_y ) );
+    }
+#else
     QPainterPath path;
 
     const std::list< DebugViewData::RectT >::const_iterator end = view.rectangles().end();
@@ -553,10 +765,10 @@ DebugPainter::drawRectangles( QPainter & painter,
           it != end;
           ++it )
     {
-        int left_x = vconf.screenX( it->left_x_ * reverse );
-        int top_y = vconf.screenY( it->top_y_ * reverse );
-        int right_x = vconf.screenX( it->right_x_ * reverse );
-        int bottom_y = vconf.screenY( it->bottom_y_ * reverse );
+        int left_x = opt.screenX( it->left_x_ * reverse );
+        int top_y = opt.screenY( it->top_y_ * reverse );
+        int right_x = opt.screenX( it->right_x_ * reverse );
+        int bottom_y = opt.screenY( it->bottom_y_ * reverse );
 
         path.addRect( left_x,
                       top_y,
@@ -565,6 +777,7 @@ DebugPainter::drawRectangles( QPainter & painter,
     }
 
     painter.drawPath( path );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -581,16 +794,29 @@ DebugPainter::drawCircles( QPainter & painter,
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double reverse = ( self_side == rcsc::RIGHT
                              ? -1.0
                              : 1.0 );
 
-    painter.setPen( dconf.debugFigurePen() );
+    painter.setPen( dconf.debugShapePen() );
     painter.setBrush( dconf.transparentBrush() );
 
+#ifdef USE_GLWIDGET
+    const std::list< DebugViewData::CircleT >::const_iterator end = view.circles().end();
+    for ( std::list< DebugViewData::CircleT >::const_iterator it = view.circles().begin();
+          it != end;
+          ++it )
+    {
+        int r = opt.scale( it->radius_ );
+        painter.drawEllipse( opt.screenX( it->center_x_ * reverse ) - r,
+                             opt.screenY( it->center_y_ * reverse ) - r,
+                             r * 2,
+                             r * 2 );
+    }
+#else
     QPainterPath path;
 
     const std::list< DebugViewData::CircleT >::const_iterator end = view.circles().end();
@@ -598,14 +824,15 @@ DebugPainter::drawCircles( QPainter & painter,
           it != end;
           ++it )
     {
-        int r = vconf.scale( it->radius_ );
-        path.addEllipse( vconf.screenX( it->center_x_ * reverse ) - r,
-                         vconf.screenY( it->center_y_ * reverse ) - r,
+        int r = opt.scale( it->radius_ );
+        path.addEllipse( opt.screenX( it->center_x_ * reverse ) - r,
+                         opt.screenY( it->center_y_ * reverse ) - r,
                          r * 2,
                          r * 2 );
     }
 
     painter.drawPath( path );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -614,15 +841,44 @@ DebugPainter::drawCircles( QPainter & painter,
 */
 void
 DebugPainter::drawMessage( QPainter & painter,
-                           const std::string & message ) const
+                           const DebugViewData & view ) const
 {
-    const DrawConfig & dconf = DrawConfig::instance();
+    painter.setFont( DrawConfig::instance().debugMessageFont() );
+    painter.setPen( DrawConfig::instance().debugMessageFontPen() );
 
-    painter.setFont( dconf.debugMessageFont() );
-    painter.setPen( dconf.debugMessageFontPen() );
+    int height = painter.fontMetrics().ascent() + 2;
+    int y = Options::instance().scoreBoardHeight() + height;
 
-    painter.drawText( 10,
-                      M_main_data.viewConfig().scoreBoardHeight()
-                      + painter.fontMetrics().ascent() + 2,
-                      QString::fromStdString( message ) );
+    if ( ! view.message().empty() )
+    {
+        painter.drawText( 10, y,
+                          QString::fromStdString( view.message() ) );
+        y += height;
+    }
+
+    if ( ! view.sayMessage().empty() )
+    {
+        QString text = QObject::tr( "Say: " );
+        text += QString::fromStdString( view.sayMessage() );
+        //text += QObject::tr( "\"" );
+
+        painter.drawText( 10, y, text );
+        y+= height;
+    }
+
+    if ( ! view.hearMessages().empty() )
+    {
+        QString text = QObject::tr( "Hear: " );
+
+        for ( std::map< int, std::string >::const_iterator it = view.hearMessages().begin();
+              it != view.hearMessages().end();
+              ++it )
+        {
+            text += QString( "(%1" ).arg( it->first );
+            text += QString::fromStdString( it->second );
+            text += QObject::tr( ")" );
+        }
+
+        painter.drawText( 10, y, text );
+    }
 }
