@@ -75,6 +75,7 @@ public:
     static const QColor TACKLE_COLOR;
     static const QColor TACKLE_FAULT_COLOR;
     static const QColor POINTTO_COLOR;
+    static const QColor ATTENTIONTO_COLOR;
     static const QColor MEASURE_COLOR;
     static const QColor MEASURE_FONT_COLOR;
     static const QColor DEBUG_BALL_COLOR;
@@ -88,7 +89,7 @@ public:
     static const QColor DEBUG_MESSAGE_FONT_COLOR;
     static const QColor DEBUG_TARGET_COLOR;
     static const QColor DEBUG_LOG_MESSAGE_FONT_COLOR;
-    static const QColor DEBUG_FIGURE_COLOR;
+    static const QColor DEBUG_SHAPE_COLOR;
 
 private:
 
@@ -188,6 +189,7 @@ private:
     QBrush M_tackle_brush; //!< body paint brush when tackling
     QBrush M_tackle_fault_brush; //!< body paint brush when tackle fault
     QPen M_pointto_pen; //!< pointto painting
+    //    QPen M_attentionto_pen; //!< attentionto painting
 
     //! kick accel info pen
     QPen M_kick_accel_pen;
@@ -238,7 +240,7 @@ private:
     QPen M_debug_log_message_font_pen;
 
     // line, triangle, rectanble, circle
-    QPen M_debug_figure_pen;
+    QPen M_debug_shape_pen;
 
     //! constructor
     DrawConfig();
@@ -696,7 +698,7 @@ public:
           return M_tackle_fault_brush;
       }
 
-   QPen * pointtoPenPtr()
+    QPen * pointtoPenPtr()
       {
           return &M_pointto_pen;
       }
@@ -706,6 +708,16 @@ public:
           return M_pointto_pen;
       }
 
+    //     QPen * attentiontoPenPtr()
+    //       {
+    //           return &M_pointto_pen;
+    //       }
+    //     const
+    //     QPen & attentiontoPen() const
+    //       {
+    //           return M_attentionto_pen;
+    //       }
+
     QPen * kickAccelPenPtr()
       {
           return &M_kick_accel_pen;
@@ -936,14 +948,14 @@ public:
           return M_debug_log_message_font_pen;
       }
 
-    QPen * debugFigurePenPtr()
+    QPen * debugShapePenPtr()
       {
-          return &M_debug_figure_pen;
+          return &M_debug_shape_pen;
       }
     const
-    QPen & debugFigurePen() const
+    QPen & debugShapePen() const
       {
-          return M_debug_figure_pen;
+          return M_debug_shape_pen;
       }
 };
 
