@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -56,7 +56,7 @@
 
 // model
 #include "main_data.h"
-#include "app_config.h"
+#include "options.h"
 
 #include <rcsc/common/server_param.h>
 
@@ -64,22 +64,21 @@
 #include <cmath>
 #include <cassert>
 
-namespace {
-int cursor_timeout = 5000;
-}
-
 /*-------------------------------------------------------------------*/
 /*!
 
 */
 FieldCanvas::FieldCanvas( MainData & main_data )
-    : M_main_data( main_data )
-    , M_redraw_all( true )
-    , M_normal_menu( static_cast< QMenu * >( 0 ) )
-    , M_system_menu( static_cast< QMenu * >( 0 ) )
-    , M_monitor_menu( static_cast< QMenu * >( 0 ) )
-    , M_cursor_timer( new QTimer( this ) )
-    , M_draw_type( -1 )
+    :
+#ifdef USE_GLWIDGET
+    QGLWidget( QGLFormat( QGL::SampleBuffers ) ),
+#endif
+    M_main_data( main_data ),
+    M_redraw_all( true ),
+    M_normal_menu( static_cast< QMenu * >( 0 ) ),
+    M_system_menu( static_cast< QMenu * >( 0 ) ),
+    M_monitor_menu( static_cast< QMenu * >( 0 ) ),
+    M_cursor_timer( new QTimer( this ) )
 {
     //this->setPalette( M_main_data.drawConfig().fieldBrush().color() );
     //this->setAutoFillBackground( true );
@@ -115,20 +114,20 @@ FieldCanvas::~FieldCanvas()
 void
 FieldCanvas::createPainters()
 {
-    ViewConfig::DrawType draw_type = M_main_data.viewConfig().drawType();
-    if ( M_draw_type == static_cast< int >( draw_type ) )
+    static int s_paint_style = -1;
+
+    Options::PaintStyle paint_style = Options::instance().paintStyle();
+    if ( s_paint_style == static_cast< int >( paint_style ) )
     {
         return;
     }
 
-    M_draw_type = static_cast< int >( draw_type );
+    s_paint_style = static_cast< int >( paint_style );
     M_painters.clear();
 
-    if ( draw_type == ViewConfig::DRAW_RCSSMONITOR )
+    if ( paint_style == Options::PAINT_RCSSMONITOR )
     {
-        M_field_painter
-            = boost::shared_ptr< FieldPainter >
-            ( new FieldPainter( M_main_data.viewConfig() ) );
+        M_field_painter = boost::shared_ptr< FieldPainter >( new FieldPainter() );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
                               ( new TeamGraphicPainter( M_main_data ) ) );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
@@ -154,16 +153,15 @@ FieldCanvas::createPainters()
         return;
     }
 
-    if ( draw_type != ViewConfig::DRAW_DEFAULT )
+    if ( paint_style != Options::PAINT_DEFAULT )
     {
-        std::cerr << "***WARNING*** Unsupported draw type id=" << draw_type
+        std::cerr << "***WARNING*** Unsupported paint style : " << paint_style
                   << std::endl;
     }
 
     {
         M_field_painter
-            = boost::shared_ptr< FieldPainter >
-            ( new FieldPainter( M_main_data.viewConfig() ) );
+            = boost::shared_ptr< FieldPainter >( new FieldPainter() );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
                               ( new TeamGraphicPainter( M_main_data ) ) );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
@@ -176,8 +174,6 @@ FieldCanvas::createPainters()
                               ( new BallPainter( M_main_data ) ) );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
                               ( new OffsideLinePainter( M_main_data ) ) );
-        M_painters.push_back( boost::shared_ptr< PainterInterface >
-                              ( new PlayerControlPainter( M_main_data ) ) );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
                               ( new VoronoiDiagramPainter( M_main_data ) ) );
         M_painters.push_back( boost::shared_ptr< PainterInterface >
@@ -291,16 +287,30 @@ FieldCanvas::freeKickRight()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+FieldCanvas::changePlayMode( int mode )
+{
+    emit playModeChanged( mode, M_mouse_state[0].pressedPoint() );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 FieldCanvas::paintEvent( QPaintEvent * )
 {
     QPainter painter( this );
-    if ( M_main_data.viewConfig().antiAliasing() )
+
+    if ( Options::instance().antiAliasing() )
     {
-        painter.setRenderHint( QPainter::Antialiasing );
-        // for QGLWidget
-        //painter.setRenderHint( QPainter::HighQualityAntialiasing );
+        painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                               //QPainter::HighQualityAntialiasing |
+#endif
+                               QPainter::Antialiasing,
+                               true );
     }
 
     M_main_data.update( this->width(),
@@ -335,9 +345,9 @@ FieldCanvas::mouseDoubleClickEvent( QMouseEvent * event )
         emit focusChanged( event->pos() );
     }
 
-    if ( M_main_data.viewConfig().cursorHide() )
+    if ( Options::instance().cursorHide() )
     {
-        M_cursor_timer->start( cursor_timeout );
+        M_cursor_timer->start( Options::CURSOR_TIMEOUT );
     }
 }
 
@@ -396,7 +406,7 @@ FieldCanvas::mousePressEvent( QMouseEvent * event )
         }
     }
 
-    if ( M_main_data.viewConfig().cursorHide() )
+    if ( Options::instance().cursorHide() )
     {
         M_cursor_timer->stop();
     }
@@ -421,7 +431,7 @@ FieldCanvas::mouseReleaseEvent( QMouseEvent * event )
         }
         else if ( M_mouse_state[0].pressedPoint() == event->pos() )
         {
-            if ( AppConfig::instance().monitorClientMode() )
+            if ( Options::instance().monitorClientMode() )
             {
                 if ( M_monitor_menu
                      && ! M_monitor_menu->exec( event->globalPos() ) )
@@ -453,7 +463,7 @@ FieldCanvas::mouseReleaseEvent( QMouseEvent * event )
         }
         else if ( M_mouse_state[2].pressedPoint() == event->pos() )
         {
-            if ( AppConfig::instance().monitorClientMode() )
+            if ( Options::instance().monitorClientMode() )
             {
                 if ( M_system_menu
                      && ! M_system_menu->exec( event->globalPos() ) )
@@ -472,9 +482,9 @@ FieldCanvas::mouseReleaseEvent( QMouseEvent * event )
         }
     }
 
-    if ( M_main_data.viewConfig().cursorHide() )
+    if ( Options::instance().cursorHide() )
     {
-        M_cursor_timer->start( cursor_timeout );
+        M_cursor_timer->start( Options::CURSOR_TIMEOUT );
     }
 }
 
@@ -503,8 +513,8 @@ FieldCanvas::mouseMoveEvent( QMouseEvent * event )
                 this->setCursor( QCursor( Qt::ClosedHandCursor ) );
             }
 
-            int new_x = M_main_data.viewConfig().absScreenX( M_main_data.viewConfig().focusPoint().x );
-            int new_y = M_main_data.viewConfig().absScreenY( M_main_data.viewConfig().focusPoint().y );
+            int new_x = Options::instance().absScreenX( Options::instance().focusPoint().x );
+            int new_y = Options::instance().absScreenY( Options::instance().focusPoint().y );
             new_x -= ( event->pos().x() - M_mouse_state[0].draggedPoint().x() );
             new_y -= ( event->pos().y() - M_mouse_state[0].draggedPoint().y() );
             emit focusChanged( QPoint( new_x, new_y ) );
@@ -532,7 +542,7 @@ FieldCanvas::mouseMoveEvent( QMouseEvent * event )
         s_last_rect = new_rect;
     }
 
-    if ( M_main_data.viewConfig().cursorHide() )
+    if ( Options::instance().cursorHide() )
     {
         if ( M_mouse_state[0].isPressed()
              || M_mouse_state[1].isPressed()
@@ -542,7 +552,7 @@ FieldCanvas::mouseMoveEvent( QMouseEvent * event )
         }
         else
         {
-            M_cursor_timer->start( cursor_timeout );
+            M_cursor_timer->start( Options::CURSOR_TIMEOUT );
         }
     }
 
@@ -558,10 +568,9 @@ FieldCanvas::draw( QPainter & painter )
 {
     createPainters();
 
-    M_field_painter->setRedraw( M_redraw_all );
     M_field_painter->draw( painter );
 
-    if ( ! M_main_data.getViewData( M_main_data.viewIndex() ) )
+    if ( ! M_main_data.getCurrentViewData() )
     {
         return;
     }
@@ -584,7 +593,8 @@ FieldCanvas::draw( QPainter & painter )
 void
 FieldCanvas::drawMouseMeasure( QPainter & painter )
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
 
     QPoint start_point = M_mouse_state[2].pressedPoint();
     QPoint end_point = M_mouse_state[2].draggedPoint();
@@ -604,28 +614,28 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
                           4,
                           4 );
 
-    rcsc::Vector2D start_real( vconf.fieldX( start_point.x() ),
-                               vconf.fieldY( start_point.y() ) );
-    rcsc::Vector2D end_real( vconf.fieldX( end_point.x() ),
-                             vconf.fieldY( end_point.y() ) );
+    rcsc::Vector2D start_real( opt.fieldX( start_point.x() ),
+                               opt.fieldY( start_point.y() ) );
+    rcsc::Vector2D end_real( opt.fieldX( end_point.x() ),
+                             opt.fieldY( end_point.y() ) );
 
     // ball travel marks
     {
-        double ball_speed = rcsc::ServerParam::i().ballSpeedMax();
+        double ball_speed = SP.ballSpeedMax();
 
         if ( ! M_measure_ball_speed_point.isNull()
              && M_measure_ball_speed_point != start_point )
         {
             rcsc::Vector2D vel( std::abs( M_measure_ball_speed_point.x()
                                           - start_point.x() )
-                                / vconf.fieldScale(),
+                                / opt.fieldScale(),
                                 std::abs( M_measure_ball_speed_point.y()
                                           - start_point.y() )
-                                / vconf.fieldScale() );
+                                / opt.fieldScale() );
             ball_speed = vel.r();
-            if ( ball_speed > rcsc::ServerParam::i().ballSpeedMax() )
+            if ( ball_speed > SP.ballSpeedMax() )
             {
-                ball_speed = rcsc::ServerParam::i().ballSpeedMax();
+                ball_speed = SP.ballSpeedMax();
             }
         }
 
@@ -637,14 +647,14 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
         double total_travel = 0.0;
 
         ball_pos += ball_vel;
-        ball_vel *= rcsc::ServerParam::i().ballDecay();
+        ball_vel *= SP.ballDecay();
         total_travel += ball_vel.r();
         QPoint last_pt( start_point.x() - 2,
                         start_point.y() - 2 );
         while ( total_travel < max_length )
         {
-            QPoint pt( vconf.absScreenX( ball_pos.x ) - 2,
-                       vconf.absScreenY( ball_pos.y ) - 2 );
+            QPoint pt( opt.absScreenX( ball_pos.x ) - 2,
+                       opt.absScreenY( ball_pos.y ) - 2 );
             if ( std::abs( pt.x() - last_pt.x() ) < 1
                  && std::abs( pt.y() - last_pt.y() ) < 1 )
             {
@@ -656,8 +666,8 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
                                   4,
                                   4 );
             ball_pos += ball_vel;
-            ball_vel *= rcsc::ServerParam::i().ballDecay();
-            ball_speed *= rcsc::ServerParam::i().ballDecay();
+            ball_vel *= SP.ballDecay();
+            ball_speed *= SP.ballDecay();
             total_travel += ball_speed;
         }
     }
@@ -671,10 +681,10 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
     //painter.setBackgroundMode( Qt::TransparentMode );
 
     char buf[64];
-    std::snprintf( buf, 64,
-                   "(%.2f,%.2f)",
-                   start_real.x,
-                   start_real.y );
+    snprintf( buf, 64,
+              "(%.2f,%.2f)",
+              start_real.x,
+              start_real.y );
 
     painter.drawText( start_point,
                       QString::fromAscii( buf ) );
@@ -685,10 +695,10 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
         return;
     }
 
-    std::snprintf( buf, 64,
-                   "(%.2f,%.2f)",
-                   end_real.x,
-                   end_real.y );
+    snprintf( buf, 64,
+              "(%.2f,%.2f)",
+              end_real.x,
+              end_real.y );
 
     painter.drawText( end_point.x(),
                       end_point.y(),
@@ -696,9 +706,9 @@ FieldCanvas::drawMouseMeasure( QPainter & painter )
 
     painter.setPen( QColor( 224, 224, 192 ) );
     rcsc::Vector2D rel( end_real - start_real );
-    std::snprintf( buf, 64,
-                   "rel(%.2f,%.2f) r%.2f th%.1f",
-                   rel.x, rel.y, rel.r(), rel.th().degree() );
+    snprintf( buf, 64,
+              "rel(%.2f,%.2f) r%.2f th%.1f",
+              rel.x, rel.y, rel.r(), rel.th().degree() );
 
     int dist_add_y = ( end_point.y() > start_point.y()
                        ? + painter.fontMetrics().height()
@@ -724,9 +734,7 @@ FieldCanvas::drawDraggedPlayer( QPainter & painter )
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
-
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
 
     if ( ! view )
     {
@@ -750,7 +758,7 @@ FieldCanvas::drawDraggedPlayer( QPainter & painter )
     const Player & player = view->players()[index];
     const rcsc::PlayerType & type = M_main_data.viewHolder().playerType( player.type() );
 
-    int draw_radius = vconf.scale( type.kickableArea() );
+    int draw_radius = Options::instance().scale( type.kickableArea() );
 
     const DrawConfig & dconf = DrawConfig::instance();
 
@@ -770,16 +778,16 @@ FieldCanvas::drawDraggedPlayer( QPainter & painter )
 void
 FieldCanvas::grabPlayer( const QPoint & point )
 {
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
     if ( ! view )
     {
         return;
     }
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
 
-    const rcsc::Vector2D field_pos( vconf.fieldX( point.x() ),
-                                    vconf.fieldY( point.y() ) );
+    const rcsc::Vector2D field_pos( opt.fieldX( point.x() ),
+                                    opt.fieldY( point.y() ) );
 
     int player_index = -1;
     double min_dist = 100000.0;
@@ -789,12 +797,12 @@ FieldCanvas::grabPlayer( const QPoint & point )
           p != end;
           ++p, ++index )
     {
-        double size = ( vconf.playerSize() > 0.0
-                        ? vconf.playerSize()
+        double size = ( opt.playerSize() > 0.0
+                        ? opt.playerSize()
                         : M_main_data.viewHolder().playerType( p->type() ).kickableArea() );
 
         rcsc::Vector2D ppos( p->x(), p->y() );
-        if ( vconf.reverseSide() ) ppos *= -1.0;
+        if ( opt.reverseSide() ) ppos *= -1.0;
 
         double dist = field_pos.dist( ppos );
         if ( dist < size
@@ -841,10 +849,10 @@ FieldCanvas::dragPlayer( const QPoint & point )
 
     M_player_dragged_point = point;
 
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
 
-    const rcsc::Vector2D field_pos( vconf.fieldX( point.x() ),
-                                    vconf.fieldY( point.y() ) );
+    const rcsc::Vector2D field_pos( opt.fieldX( point.x() ),
+                                    opt.fieldY( point.y() ) );
 
     M_main_data.getTrainerData().setPlayer( data.draggedPlayerSide(),
                                             data.draggedPlayerNumber(),
@@ -852,9 +860,9 @@ FieldCanvas::dragPlayer( const QPoint & point )
                                             0.0 );
 
     // calculate an update area
-    int player_size = ( vconf.playerSize() > 0.0
-                        ? vconf.scale( vconf.playerSize() + 0.3 )
-                        : vconf.scale( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) );
+    int player_size = ( opt.playerSize() > 0.0
+                        ? opt.scale( opt.playerSize() + 0.3 )
+                        : opt.scale( rcsc::ServerParam::i().defaultKickableArea() + 0.3 ) );
 
     QRect new_rect( point.x() - player_size,
                     point.y() - player_size,
