@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -93,6 +93,49 @@ public:
 
 };
 
+
+class CycleSlider
+    : public QSlider {
+private:
+
+public:
+
+    CycleSlider( Qt::Orientation orientation,
+                 QWidget * parent = 0 )
+        : QSlider( orientation, parent )
+      { }
+
+protected:
+#if 0
+    void mousePressEvent( QMouseEvent * event )
+      {
+          if ( event->button() == Qt::LeftButton )
+          {
+              double rate = 0;
+              if ( this->orientation() == Qt::Horizontal )
+              {
+                  rate = static_cast< double >( event->pos().x() )
+                      / static_cast< double >( this->width() );
+
+              }
+              else
+              {
+                  rate = static_cast< double >( this->height() - event->pos().y() )
+                      / static_cast< double >( this->height() );
+              }
+
+              int val = this->minimum()
+                  + static_cast< int >( rint( ( this->maximum() - this->minimum() ) * rate ) );
+
+              this->setValue( val );
+          }
+
+          QSlider::mousePressEvent( event );
+      }
+#endif
+};
+
+
 /*-------------------------------------------------------------------*/
 /*!
 
@@ -143,108 +186,120 @@ LogPlayerToolBar::createControls( LogPlayer * log_player,
 {
     // visible actions
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_go_first_xpm ) ),
-                           tr( "Go first" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_go_first_xpm ) ),
+                                     tr( "Go first" ), this );
         act->setShortcut( Qt::Key_Home );
-        act->setStatusTip( tr( "Go to the first.(Home)" ) );
+        act->setStatusTip( tr( "Go to the first cycle.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( goToFirst() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_go_prev_score_xpm ) ),
-                           tr( "Go prev score" ), this );
-        act->setShortcut( tr( "Ctrl+G" ) );
-        act->setStatusTip( tr( "Go to the previous goal scene.(Ctrl+G)" ) );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_go_prev_score_xpm ) ),
+                                     tr( "Go prev goal" ), this );
+#ifdef Q_WS_MAC
+        act->setShortcut( Qt::META + Qt::Key_G );
+#else
+        act->setShortcut( Qt::CTRL + Qt::Key_G );
+#endif
+        act->setStatusTip( tr( "Go to the previous goal scene.(" )
+                           + act->shortcut().toString()
+                           + (")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( goToPrevScore() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_one_step_back_xpm ) ),
-                           tr( "Step Back" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_one_step_back_xpm ) ),
+                                     tr( "Step Back" ), this );
         act->setShortcut( Qt::Key_Left );
-        act->setStatusTip( tr( "One step back the log player. (Left)" ) );
+        act->setStatusTip( tr( "One step back the log player. (" )
+                           + act->shortcut().toString()
+                           + tr ( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( stepBack() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_play_or_stop_xpm ) ),
-                           tr( "Play/Stop." ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_play_or_stop_xpm ) ),
+                                     tr( "Play/Stop." ), this );
         act->setShortcut( Qt::Key_Space );
-        act->setStatusTip( tr( "Play or Stop the log player.(Space)" ) );
+        act->setStatusTip( tr( "Play or Stop the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( playOrStop() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_one_step_forward_xpm ) ),
-                           tr( "Step Forward" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_one_step_forward_xpm ) ),
+                                     tr( "Step Forward" ), this );
         act->setShortcut( Qt::Key_Right );
-        act->setStatusTip( tr( "One step forward the log player.(Right)" ) );
+        act->setStatusTip( tr( "One step forward the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( stepForward() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_go_next_score_xpm ) ),
-                           tr( "Go next score" ), this );
-        act->setShortcut( tr( "G" ) );
-        act->setStatusTip( tr( "Go to the next goal scene.(G)" ) );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_go_next_score_xpm ) ),
+                                     tr( "Go next score" ), this );
+        act->setShortcut( Qt::Key_G );
+        act->setStatusTip( tr( "Go to the next goal scene.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( goToNextScore() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_go_last_xpm ) ),
-                           tr( "Go last" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_go_last_xpm ) ),
+                                     tr( "Go last" ), this );
         act->setShortcut( Qt::Key_End );
-        act->setStatusTip( tr( "Go to the last.(End)" ) );
+        act->setStatusTip( tr( "Go to the last.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( goToLast() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_shift_down_xpm ) ),
-                           tr( "Decelerate" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_shift_down_xpm ) ),
+                                     tr( "Decelerate" ), this );
 #ifdef Q_WS_MAC
         act->setShortcut( Qt::META + Qt::Key_Left );
 #else
         act->setShortcut( Qt::CTRL + Qt::Key_Left );
 #endif
-
-        act->setStatusTip( "Decelerate the log player.(Ctrl+Left)" );
+        act->setStatusTip( tr( "Decelerate the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( decelerate() ) );
         this->addAction( act );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_shift_up_xpm ) ),
-                           tr( "Accelerate" ), this );
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_shift_up_xpm ) ),
+                                     tr( "Accelerate" ), this );
 #ifdef Q_WS_MAC
         act->setShortcut( Qt::META + Qt::Key_Right );
 #else
         act->setShortcut( Qt::CTRL + Qt::Key_Right );
 #endif
-        act->setStatusTip( "Accelerate the log player.(Ctrl+Right)" );
+        act->setStatusTip( tr ( "Accelerate the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( accelerate() ) );
         this->addAction( act );
@@ -253,25 +308,29 @@ LogPlayerToolBar::createControls( LogPlayer * log_player,
 
     // invisible actions
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_stop_xpm ) ),
-                           tr( "Stop" ), this );
+        //(void)new QShortcut( Qt::Key_Down, this, SLOT( stop() ) );
+
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_stop_xpm ) ),
+                                     tr( "Stop" ), this );
         act->setShortcut( Qt::Key_Down );
-        act->setStatusTip( tr( "Stop the log player. (Down)" ) );
+        act->setStatusTip( tr( "Stop the log player. (" )
+                           + act->shortcut().toString()
+                           + tr ( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( stop() ) );
         //this->addAction( act );
         //act->setVisible( false );
 
-        //new QShortcut( Qt::Key_Down,
-        //               this, SLOT( stop() ) );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_play_back_xpm ) ),
-                           tr( "Play Backward" ), this );
-        act->setStatusTip( tr( "Play backward the log player.(Ctrl+Down)" ) );
+        //new QShortcut( Qt::CTRL + Qt::Key_Down, this, SLOT( playBack() ) );
+
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_play_back_xpm ) ),
+                                     tr( "Play Backward" ), this );
+        act->setStatusTip( tr( "Play backward the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
 #ifdef Q_WS_MAC
         act->setShortcut( Qt::META + Qt::Key_Down );
 #else
@@ -282,29 +341,28 @@ LogPlayerToolBar::createControls( LogPlayer * log_player,
         //this->addAction( act );
         //act->setVisible( false );
 
-        //new QShortcut( Qt::CTRL + Qt::Key_Down,
-        //               this, SLOT( playBack() ) );
         main_win->addAction( act );
     }
     {
-        QAction * act
-            = new QAction( QIcon( QPixmap( logplayer_play_forward_xpm ) ),
-                           tr( "Play Forward" ), this );
+        //(void)new QShortcut( Qt::Key_Up, this, SLOT( playForward() ) );
+
+        QAction * act = new QAction( QIcon( QPixmap( logplayer_play_forward_xpm ) ),
+                                     tr( "Play Forward" ), this );
         act->setShortcut( Qt::Key_Up );
-        act->setStatusTip( tr( "Play forward the log player.(Up)" ) );
+        act->setStatusTip( tr( "Play forward the log player.(" )
+                           + act->shortcut().toString()
+                           + tr( ")" ) );
         connect( act, SIGNAL( triggered() ),
                  log_player, SLOT( playForward() ) );
         //this->addAction( act );
         //act->setVisible( false );
 
-        //new QShortcut( Qt::Key_Up,
-        //               this, SLOT( playForward() ) );
         main_win->addAction( act );
     }
 
     // visible widgets
 
-    M_cycle_slider = new QSlider( this->orientation() );
+    M_cycle_slider = new CycleSlider( this->orientation() );
     connect( this, SIGNAL( orientationChanged( Qt::Orientation ) ),
              M_cycle_slider, SLOT( setOrientation( Qt::Orientation ) ) );
     M_cycle_slider->setStatusTip( tr( "You can select the cycle by this slider." ) );
@@ -315,7 +373,9 @@ LogPlayerToolBar::createControls( LogPlayer * log_player,
     M_cycle_slider->setMaximumSize( 640, 640 );
     M_cycle_slider->setMinimumWidth( 200 );
     //M_cycle_slider->setMinimumSize( 400, 400 );
-    connect( M_cycle_slider, SIGNAL( sliderMoved( int ) ),
+//     connect( M_cycle_slider, SIGNAL( sliderMoved( int ) ),
+//              log_player, SLOT( goToIndex( int ) ) );
+    connect( M_cycle_slider, SIGNAL( valueChanged( int ) ),
              log_player, SLOT( goToIndex( int ) ) );
 
     this->addWidget( M_cycle_slider );
@@ -329,18 +389,18 @@ LogPlayerToolBar::createControls( LogPlayer * log_player,
     this->addWidget( M_cycle_edit );
 
     // invisible action
-    {
-        QAction * act = new QAction( tr( "Focus Cycle Edit" ), this );
-#ifdef Q_WS_MAC
-        act->setShortcut( tr( "Meta+F" ) );
-#else
-        act->setShortcut( tr( "Ctrl+F" ) );
-#endif
-        connect( act, SIGNAL( triggered() ),
-                 M_cycle_edit, SLOT( setFocus() ) );
+//     {
+//         QAction * act = new QAction( tr( "Focus Cycle Edit" ), this );
+// #ifdef Q_WS_MAC
+//         act->setShortcut( Qt::META + Qt::Key_F );
+// #else
+//         act->setShortcut( Qt::CTRL + Qt::Key_F );
+// #endif
+//         connect( act, SIGNAL( triggered() ),
+//                  M_cycle_edit, SLOT( setFocus() ) );
 
-        main_win->addAction( act );
-    }
+//         main_win->addAction( act );
+//     }
 
     connect( this, SIGNAL( orientationChanged( Qt::Orientation ) ),
              this, SLOT( changeOrientation( Qt::Orientation ) ) );
