@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -37,6 +37,7 @@
 
 #include "monitor_move_dialog.h"
 
+#include "options.h"
 #include "main_data.h"
 #include "trainer_data.h"
 
@@ -303,10 +304,11 @@ MonitorMoveDialog::createLeftTeamBox()
     for ( int i = 0; i < 11; ++i )
     {
         int col = 0;
+        char buf[64];
 
-        QString text;
-        text.sprintf( "%2d:", i + 1 );
-        layout->addWidget( new QLabel( text ), i + 1, col++ );
+        snprintf( buf, 64, "%2d:", i + 1 );
+        layout->addWidget( new QLabel( QString::fromAscii( buf ) ),
+                           i + 1, col++ );
         //
         M_left_cb[i] = new QCheckBox();
         M_left_cb[i]->setChecked( false );
@@ -315,8 +317,8 @@ MonitorMoveDialog::createLeftTeamBox()
         cb_mapper->setMapping( M_left_cb[i], i );
         layout->addWidget( M_left_cb[i], i + 1, col++ );
         //
-        text.sprintf( "%7.3f", -3.0 * (i+1) );
-        M_left_x[i] = new QLineEdit( text );
+        snprintf( buf, 64, "%7.3f", -3.0 * (i+1) );
+        M_left_x[i] = new QLineEdit( QString::fromAscii( buf ) );
         M_left_x[i]->setValidator( new QDoubleValidator( -57.5, 57.5, 3,
                                                          M_left_x[i] ) );
         M_left_x[i]->setMaximumSize( 64, 24 );
@@ -380,10 +382,11 @@ MonitorMoveDialog::createRightTeamBox()
     for ( int i = 0; i < 11; ++i )
     {
         int col = 0;
+        char buf[64];
 
-        QString text;
-        text.sprintf( "%2d:", i + 1 );
-        layout->addWidget( new QLabel( text ), i + 1, col++ );
+        snprintf( buf, 64, "%2d:", i + 1 );
+        layout->addWidget( new QLabel( QString::fromAscii( buf ) ),
+                           i + 1, col++ );
         //
         M_right_cb[i] = new QCheckBox();
         M_right_cb[i]->setChecked( true );
@@ -392,8 +395,8 @@ MonitorMoveDialog::createRightTeamBox()
         cb_mapper->setMapping( M_right_cb[i], i );
         layout->addWidget( M_right_cb[i], i + 1, col++ );
         //
-        text.sprintf( "%7.3f", 3.0 * (i+1) );
-        M_right_x[i] = new QLineEdit( text );
+        snprintf( buf, 64, "%7.3f", 3.0 * (i+1) );
+        M_right_x[i] = new QLineEdit( QString::fromAscii( buf ) );
         M_right_x[i]->setValidator( new QDoubleValidator( -57.5, 57.5, 3,
                                                           M_right_x[i] ) );
         M_right_x[i]->setMaximumSize( 64, 24 );
@@ -523,15 +526,15 @@ MonitorMoveDialog::toggleRightCheck( int index )
 void
 MonitorMoveDialog::readFieldStatus()
 {
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
 
     if ( ! view )
     {
         return;
     }
 
-    const bool reverse = M_main_data.viewConfig().reverseSide();
-    const double rval =  M_main_data.viewConfig().reverseValue();
+    const bool reverse = Options::instance().reverseSide();
+    const double rval =  Options::instance().reverseValue();
 
     // playmode
     if ( view->playmode().isLeftSetPlay() )
@@ -561,25 +564,25 @@ MonitorMoveDialog::readFieldStatus()
         M_drop_ball_rb->setChecked( true );
     }
 
-    QString text;
+    char buf[64];
 
     // ball
     if ( M_ball_cb->isChecked() )
     {
-        text.sprintf( "%.3f", view->ball().x() * rval );
-        M_ball_x->setText( text );
+        snprintf( buf, 64, "%.3f", view->ball().x() * rval );
+        M_ball_x->setText( QString::fromAscii( buf ) );
 
-        text.sprintf( "%.3f", view->ball().y() * rval );
-        M_ball_y->setText( text );
+        snprintf( buf, 64, "%.3f", view->ball().y() * rval );
+        M_ball_y->setText( QString::fromAscii( buf ) );
 
         if ( M_ball_vel_cb->isChecked()
              && view->ball().hasDelta() )
         {
-            text.sprintf( "%.3f", view->ball().deltaX() * rval );
-            M_ball_vx->setText( text );
+            snprintf( buf, 64, "%.3f", view->ball().deltaX() * rval );
+            M_ball_vx->setText( QString::fromAscii( buf ) );
 
-            text.sprintf( "%.3f", view->ball().deltaY() * rval );
-            M_ball_vy->setText( text );
+            snprintf( buf, 64, "%.3f", view->ball().deltaY() * rval );
+            M_ball_vy->setText( QString::fromAscii( buf ) );
         }
     }
 
@@ -594,11 +597,11 @@ MonitorMoveDialog::readFieldStatus()
         {
             if ( ! M_left_cb[idx]->isChecked() ) continue;
 
-            text.sprintf( "%.3f", players[i].x() * rval );
-            M_left_x[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", players[i].x() * rval );
+            M_left_x[idx]->setText( QString::fromAscii( buf ) );
 
-            text.sprintf( "%.3f", players[i].y() * rval );
-            M_left_y[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", players[i].y() * rval );
+            M_left_y[idx]->setText( QString::fromAscii( buf ) );
 
             double body = players[i].body();
             if ( reverse )
@@ -606,8 +609,8 @@ MonitorMoveDialog::readFieldStatus()
                 body += 180.0;
                 if ( body > 180.0 ) body -= 360.0;
             }
-            text.sprintf( "%.3f", body );
-            M_left_body[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", body );
+            M_left_body[idx]->setText( QString::fromAscii( buf ) );
         }
     }
 
@@ -619,11 +622,11 @@ MonitorMoveDialog::readFieldStatus()
         {
             if ( ! M_right_cb[idx]->isChecked() ) continue;
 
-            text.sprintf( "%.3f", players[i].x() * rval );
-            M_right_x[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", players[i].x() * rval );
+            M_right_x[idx]->setText( QString::fromAscii( buf ) );
 
-            text.sprintf( "%.3f", players[i].y() * rval );
-            M_right_y[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", players[i].y() * rval );
+            M_right_y[idx]->setText( QString::fromAscii( buf ) );
 
             double body = players[i].body();
             if ( reverse )
@@ -631,8 +634,8 @@ MonitorMoveDialog::readFieldStatus()
                 body += 180.0;
                 if ( body > 180.0 ) body -= 360.0;
             }
-            text.sprintf( "%.3f", body );
-            M_right_body[idx]->setText( text );
+            snprintf( buf, 64, "%.3f", body );
+            M_right_body[idx]->setText( QString::fromAscii( buf ) );
         }
     }
 
@@ -950,10 +953,9 @@ MonitorMoveDialog::save()
 void
 MonitorMoveDialog::sendCommand()
 {
-    //std::cerr << "send move command" << std::endl;
-    const bool reverse = M_main_data.viewConfig().reverseSide();
+    const bool reverse = Options::instance().reverseSide();
 
-     {
+    {
         if ( M_free_kick_left_rb->isChecked() )
         {
             M_trainer_data.setPlayMode( reverse
@@ -1041,7 +1043,7 @@ MonitorMoveDialog::sendCommand()
     }
 
     // right
-    side = ( M_main_data.viewConfig().reverseSide()
+    side = ( reverse
              ? rcsc::LEFT
              : rcsc::RIGHT );
     for ( int i = 0; i < 11; ++i )
