@@ -12,7 +12,7 @@
 
  This code is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
+ the Free Software Foundation; either version 3, or (at your option)
  any later version.
 
  This code is distributed in the hope that it will be useful,
@@ -39,11 +39,13 @@
 
 #include "draw_config.h"
 // model
+#include "options.h"
 #include "main_data.h"
 
 #include <rcsc/common/player_type.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/geom/angle_deg.h>
+#include <rcsc/geom/circle_2d.h>
 
 #include <cstring>
 #include <cstdio>
@@ -52,32 +54,44 @@
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 inline
 PlayerPainter::Param::Param( const Player & player,
                              const Ball & ball,
-                             const ViewConfig & view_conf,
                              const rcsc::PlayerType & ptype )
-    : x_( view_conf.screenX( player.x() ) )
-    , y_( view_conf.screenY( player.y() ) )
-    , body_( player.body() + ( view_conf.reverseSide() ? 180.0 : 0.0 ) )
-    , head_( player.head() + ( view_conf.reverseSide() ? 180.0 : 0.0 ) )
-    , body_radius_( view_conf.scale( ptype.playerSize() ) )
-    , kick_radius_( view_conf.scale( ptype.kickableArea() ) )
+    : x_( 0 )
+    , y_( 0 )
+    , body_( player.body() )
+    , head_( player.head() )
+    , body_radius_( 1 )
+    , kick_radius_( 5 )
     , have_full_effort_( player.hasFullEffort( ptype.effortMax() ) )
     , player_( player )
     , ball_( ball )
     , player_type_( ptype )
 {
+    const Options & opt = Options::instance();
+
+    x_ = opt.screenX( player.x() );
+    y_ = opt.screenY( player.y() );
+
+    if ( opt.reverseSide() )
+    {
+        body_ += 180.0;
+        head_ += 180.0;
+    }
+
+    body_radius_ = opt.scale( ptype.playerSize() );
+    kick_radius_ = opt.scale( ptype.kickableArea() );
 
     if ( body_radius_ < 1 ) body_radius_ = 1;
     if ( kick_radius_ < 5 ) kick_radius_ = 5;
 
-    if ( view_conf.isEnlarged() )
+    if ( opt.enlargeMode() )
     {
-        if ( view_conf.playerSize() > 0.0 )
+        if ( opt.playerSize() > 0.0 )
         {
-            draw_radius_ = view_conf.scale( view_conf.playerSize() );
+            draw_radius_ = opt.scale( opt.playerSize() );
             if ( draw_radius_ < 1 ) draw_radius_ = 1;
         }
         else
@@ -94,7 +108,7 @@ PlayerPainter::Param::Param( const Player & player,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 PlayerPainter::PlayerPainter( const MainData & main_data )
     : M_main_data( main_data )
 {
@@ -104,16 +118,18 @@ PlayerPainter::PlayerPainter( const MainData & main_data )
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainter::draw( QPainter & painter )
 {
-    if ( ! M_main_data.viewConfig().isShownPlayers() )
+    const Options & opt = Options::instance();
+
+    if ( ! opt.showPlayers() )
     {
         return;
     }
 
-    MonitorViewConstPtr view = M_main_data.getViewData( M_main_data.viewIndex() );
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
 
     if ( ! view )
     {
@@ -122,7 +138,7 @@ PlayerPainter::draw( QPainter & painter )
 
     const Ball & ball = view->ball();
 
-    if ( M_main_data.viewConfig().playerReverseDraw() )
+    if ( opt.playerReverseDraw() )
     {
         const std::vector< Player >::const_reverse_iterator end = view->players().rend();
         for ( std::vector< Player >::const_reverse_iterator it = view->players().rbegin();
@@ -147,41 +163,70 @@ PlayerPainter::draw( QPainter & painter )
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainter::drawAll( QPainter & painter,
                         const Player & player,
                         const Ball & ball ) const
 {
+    const Options & opt = Options::instance();
+
+    const bool selected = opt.isSelectedPlayer( player.side(), player.unum() );
     const Param param( player,
                        ball,
-                       M_main_data.viewConfig(),
                        M_main_data.viewHolder().playerType( player.type() ) );
 
     drawBody( painter, param );
-    if ( M_main_data.viewConfig().isShownBodyShadow() )
+    if ( opt.showBodyShadow() )
     {
         drawShadow( painter, param );
     }
     drawEdge( painter, param );
 
-    if ( M_main_data.viewConfig().isSelectedPlayer( player.side(), player.unum() )
-         && M_main_data.viewConfig().playerFutureCycle() > 0
+    if ( selected
+         && opt.playerFutureCycle() > 0
          && player.hasDelta() )
     {
         drawFuture( painter, param );
     }
 
-    if ( player.hasView()
-         && M_main_data.viewConfig().isShownViewCone() )
+    if ( player.hasView() )
     {
-        drawViewCone( painter, param );
+        if ( opt.showViewArea() )
+        {
+            drawViewArea( painter, param );
+        }
+        else
+        {
+            drawViewDir( painter, param );
+        }
     }
 
-    if ( M_main_data.viewConfig().isShownControlArea() )
+    if ( player.isGoalie()
+         && opt.showCatchableArea() )
     {
-        drawControlArea( painter, param );
-        drawPointTo( painter, param );
+        drawCatchableArea( painter, param );
+    }
+
+    if ( opt.showTackleArea() )
+    {
+        drawTackleArea( painter, param );
+    }
+
+    if ( selected
+         && opt.showKickAccelArea() )
+    {
+        drawKickAccelArea( painter, param );
+    }
+
+    if ( opt.showPointto() )
+    {
+        drawPointto( painter, param );
+    }
+
+    if ( opt.showAttentionto() )
+    {
+        drawAttentionto( painter, param );
     }
 
     drawText( painter, param );
@@ -190,7 +235,7 @@ PlayerPainter::drawAll( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainter::drawBody( QPainter & painter,
                          const PlayerPainter::Param & param ) const
@@ -201,7 +246,7 @@ PlayerPainter::drawBody( QPainter & painter,
     painter.setPen( dconf.playerPen() );
 
     rcsc::SideID side = param.player_.side();
-    if ( M_main_data.viewConfig().reverseSide() )
+    if ( Options::instance().reverseSide() )
     {
         side = static_cast< rcsc::SideID >( -1 * side );
     }
@@ -277,15 +322,15 @@ PlayerPainter::drawBody( QPainter & painter,
         painter.setBrush( dconf.collidePlayerBrush() );
     }
 
-    if ( M_main_data.viewConfig().gradient() )
+    if ( Options::instance().gradient() )
     {
         QRadialGradient gradient( param.x_,
                                   param.y_,
                                   param.draw_radius_,
                                   param.x_ - param.draw_radius_,
                                   param.y_ - param.draw_radius_ );
-                                  //param.x_ - param.draw_radius_*0.8,
-                                  //param.y_ - param.draw_radius_*0.8 );
+        //param.x_ - param.draw_radius_*0.8,
+        //param.y_ - param.draw_radius_*0.8 );
         //gradient.setColorAt( 0.0, painter.brush().color().lighter( 170 ) );
         gradient.setColorAt( 0.0, painter.brush().color().lighter( 160 ) );
         gradient.setColorAt( 1.0, painter.brush().color() );
@@ -301,7 +346,7 @@ PlayerPainter::drawBody( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*
 
-*/
+ */
 void
 PlayerPainter::drawShadow( QPainter & painter,
                            const PlayerPainter::Param & param ) const
@@ -347,7 +392,7 @@ PlayerPainter::drawShadow( QPainter & painter,
     painter.setBrush( dconf.shadowBrush( level ) );
 #endif
 
-    if ( M_main_data.viewConfig().gradient() )
+    if ( Options::instance().gradient() )
     {
         QRadialGradient gradient( param.x_,
                                   param.y_,
@@ -374,7 +419,7 @@ PlayerPainter::drawShadow( QPainter & painter,
 #else
     // set angle
     int shadow_start_dir
-            = static_cast< int >( rint( ( - param.body_ + 90.0 ) * 16 ) );
+        = static_cast< int >( rint( ( - param.body_ + 90.0 ) * 16 ) );
     //double shadow_end_dir = shadow_start_dir + 180.0;
     painter.drawPie( param.x_ - shadow_radius, // left x
                      param.y_ - shadow_radius, // top y
@@ -388,18 +433,18 @@ PlayerPainter::drawShadow( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerPainter::drawEdge( QPainter & painter,
                          const PlayerPainter::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     int edge_radius = 1;
-    if ( vconf.isEnlarged() )
+    if ( opt.enlargeMode() )
     {
-        if ( vconf.isShownBodyShadow() )
+        if ( opt.showBodyShadow() )
         {
             painter.setPen( dconf.realBodyPen() );
         }
@@ -414,7 +459,7 @@ PlayerPainter::drawEdge( QPainter & painter,
     {
         // draw kickable area edge
         rcsc::SideID side = param.player_.side();
-        if ( M_main_data.viewConfig().reverseSide() )
+        if ( opt.reverseSide() )
         {
             side = static_cast< rcsc::SideID >( -1 * side );
         }
@@ -438,14 +483,14 @@ PlayerPainter::drawEdge( QPainter & painter,
                          edge_radius * 2 );
 
     // body direction line
-    if ( ! vconf.isShownBodyShadow() )
+    if ( ! opt.showBodyShadow() )
     {
-        int end_x = vconf.absScreenX( param.player_.x() * vconf.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
-        int end_y = vconf.absScreenY( param.player_.y() * vconf.reverseValue()
-                                      + param.player_type_.kickableArea()
-                                      * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::cos( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
+        int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
+                                    + param.player_type_.kickableArea()
+                                    * std::sin( param.body_ * rcsc::AngleDeg::DEG2RAD ) );
         // same pen & brush
         painter.drawLine( param.x_, param.y_, end_x, end_y );
     }
@@ -480,37 +525,42 @@ PlayerPainter::drawEdge( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
 PlayerPainter::drawFuture( QPainter & painter,
                            const PlayerPainter::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    if ( vconf.antiAliasing() )
+    if ( opt.antiAliasing() )
     {
-        painter.setRenderHint( QPainter::Antialiasing, false );
+        painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                               //QPainter::HighQualityAntialiasing |
+#endif
+                               QPainter::Antialiasing,
+                               false );
     }
 
     // draw future state
     rcsc::Vector2D ppos( param.player_.x(), param.player_.y() );
     rcsc::Vector2D pvel( param.player_.deltaX(), param.player_.deltaY() );
-    if ( vconf.reverseSide() )
+    if ( opt.reverseSide() )
     {
         ppos *= -1.0;
         pvel *= -1.0;
     }
 
-    const int last = vconf.playerFutureCycle();
+    const int last = opt.playerFutureCycle();
 
-    QPoint first_point( vconf.absScreenX( ppos.x ),
-                        vconf.absScreenY( ppos.y ) ) ;
+    QPoint first_point( opt.absScreenX( ppos.x ),
+                        opt.absScreenY( ppos.y ) ) ;
     QPoint last_point = first_point;
 
     // draw kickable area edge
     rcsc::SideID side = param.player_.side();
-    if ( vconf.reverseSide() )
+    if ( opt.reverseSide() )
     {
         side = static_cast< rcsc::SideID >( -1 * side );
     }
@@ -532,8 +582,8 @@ PlayerPainter::drawFuture( QPainter & painter,
         ppos += pvel;
         pvel *= param.player_type_.playerDecay();
 
-        QPoint pt( vconf.absScreenX( ppos.x ),
-                   vconf.absScreenY( ppos.y ) );
+        QPoint pt( opt.absScreenX( ppos.x ),
+                   opt.absScreenY( ppos.y ) );
         if ( std::abs( last_point.x() - pt.x() ) < 1
              && std::abs( last_point.y() - pt.y() ) < 1 )
         {
@@ -558,39 +608,42 @@ PlayerPainter::drawFuture( QPainter & painter,
     painter.setPen( dconf.debugTargetPen() );
     painter.drawLine( first_point, last_point );
 
-    if ( vconf.antiAliasing() )
+    if ( opt.antiAliasing() )
     {
-        painter.setRenderHint( QPainter::Antialiasing );
+        painter.setRenderHints(
+#ifdef USE_GLWIDGET
+                               //QPainter::HighQualityAntialiasing |
+#endif
+                               QPainter::Antialiasing,
+                               true );
     }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainter::drawViewCone( QPainter & painter,
+PlayerPainter::drawViewArea( QPainter & painter,
                              const PlayerPainter::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const double view_width = param.player_.viewWidth();
 
     const double visible_dist = rcsc::ServerParam::i().visibleDistance();
-    const int visible_radius = vconf.scale( visible_dist );
+    const int visible_radius = opt.scale( visible_dist );
 
     const double view_start_angle = -param.head_ - view_width * 0.5;
-    const double view_start_angle_real = ( param.head_ - view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
-    const double view_end_angle_real = ( param.head_ + view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
 
     // draw enlarged view cone and feel area
-    if ( vconf.isSelectedPlayer( param.player_.side(),
-                                 param.player_.unum() ) )
+    if ( opt.isSelectedPlayer( param.player_.side(),
+                               param.player_.unum() ) )
     {
-        const int UNUM_FAR = vconf.scale( 20.0 );
-        const int TEAM_FAR = vconf.scale( 40.0 );
-        const int TEAM_TOOFAR = vconf.scale( 60.0 );
+        const int UNUM_FAR = opt.scale( 20.0 );
+        const int TEAM_FAR = opt.scale( 40.0 );
+        const int TEAM_TOOFAR = opt.scale( 60.0 );
 
         painter.setPen( dconf.linePen() );
         //painter.setBrush( dconf.transparentBrush() );
@@ -626,18 +679,21 @@ PlayerPainter::drawViewCone( QPainter & painter,
                          visible_radius * 2,
                          0,
                          360 * 16 );
+
+        const double view_start_angle_real = ( param.head_ - view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
+        const double view_end_angle_real = ( param.head_ + view_width * 0.5 ) * rcsc::AngleDeg::DEG2RAD;
         // left side view cone end point x
         int lx = param.x_
-            + vconf.scale( 60.0 * std::cos( view_start_angle_real ) );
+            + opt.scale( 60.0 * std::cos( view_start_angle_real ) );
         // left side view cone end point y
         int ly = param.y_
-            + vconf.scale( 60.0  * std::sin( view_start_angle_real ) );
+            + opt.scale( 60.0  * std::sin( view_start_angle_real ) );
         // right side view cone end point x
         int rx = param.x_
-            + vconf.scale( 60.0 * std::cos( view_end_angle_real ) );
+            + opt.scale( 60.0 * std::cos( view_end_angle_real ) );
         // right side view cone end point y
         int ry = param.y_
-            + vconf.scale( 60.0 * std::sin( view_end_angle_real ) );
+            + opt.scale( 60.0 * std::sin( view_end_angle_real ) );
         painter.drawLine( lx, ly, param.x_, param.y_ );
         painter.drawLine( rx, ry, param.x_, param.y_ );
     }
@@ -664,39 +720,74 @@ PlayerPainter::drawViewCone( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainter::drawControlArea( QPainter & painter,
-                                const PlayerPainter::Param & param ) const
+PlayerPainter::drawViewDir( QPainter & painter,
+                            const PlayerPainter::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
-    if ( param.player_.isGoalie() )
+    double r = ( opt.enlargeMode()
+                 ? param.player_type_.kickableArea()
+                 : param.player_type_.playerSize() );
+
+    int end_x = opt.absScreenX( param.player_.x() * opt.reverseValue()
+                                + r * std::cos( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+    int end_y = opt.absScreenY( param.player_.y() * opt.reverseValue()
+                                + r * std::sin( param.head_ * rcsc::AngleDeg::DEG2RAD ) );
+
+    painter.setPen( Qt::black );
+    painter.setBrush( dconf.transparentBrush() );
+    painter.drawLine( param.x_, param.y_, end_x, end_y );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+PlayerPainter::drawCatchableArea( QPainter & painter,
+                                  const PlayerPainter::Param & param ) const
+{
+    const Options & opt = Options::instance();
+    const DrawConfig & dconf = DrawConfig::instance();
+
+    rcsc::SideID side = param.player_.side();
+    if ( opt.reverseSide() )
     {
-        rcsc::SideID side = param.player_.side();
-        if ( M_main_data.viewConfig().reverseSide() )
-        {
-            side = static_cast< rcsc::SideID >( -1 * side );
-        }
-
-        int catchable = vconf.scale( rcsc::ServerParam::i().catchableArea() );
-        painter.setPen( ( side == rcsc::LEFT )
-                        ? dconf.leftGoaliePen()
-                        : dconf.rightGoaliePen() );
-        painter.setBrush( dconf.transparentBrush() );;
-
-        painter.drawEllipse( param.x_ - catchable,
-                             param.y_ - catchable,
-                             catchable * 2,
-                             catchable * 2 );
+        side = static_cast< rcsc::SideID >( -1 * side );
     }
 
+    int catchable = opt.scale( rcsc::ServerParam::i().catchableArea() );
+    painter.setPen( ( side == rcsc::LEFT )
+                    ? dconf.leftGoaliePen()
+                    : dconf.rightGoaliePen() );
+    painter.setBrush( dconf.transparentBrush() );;
+
+    painter.drawEllipse( param.x_ - catchable,
+                         param.y_ - catchable,
+                         catchable * 2,
+                         catchable * 2 );
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+PlayerPainter::drawTackleArea( QPainter & painter,
+                               const PlayerPainter::Param & param ) const
+{
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
+    const DrawConfig & dconf = DrawConfig::instance();
+
     rcsc::Vector2D ppos( param.player_.x(),
                          param.player_.y() );
     rcsc::Vector2D bpos( param.ball_.x(),
                          param.ball_.y() );
-    if ( vconf.reverseSide() )
+    if ( opt.reverseSide() )
     {
         ppos *= -1.0;
         bpos *= -1.0;
@@ -707,34 +798,34 @@ PlayerPainter::drawControlArea( QPainter & painter,
 
     // draw tackle area & probability
     double tackle_dist = ( player_to_ball.x > 0.0
-                           ? rcsc::ServerParam::i().tackleDist()
-                           : rcsc::ServerParam::i().tackleBackDist() );
+                           ? SP.tackleDist()
+                           : SP.tackleBackDist() );
     double tackle_prob = ( std::pow( player_to_ball.absX() / tackle_dist,
-                                     rcsc::ServerParam::i().tackleExponent() )
-                           + std::pow( player_to_ball.absY() / rcsc::ServerParam::i().tackleWidth(),
-                                       rcsc::ServerParam::i().tackleExponent() ) );
+                                     SP.tackleExponent() )
+                           + std::pow( player_to_ball.absY() / SP.tackleWidth(),
+                                       SP.tackleExponent() ) );
     if ( tackle_prob < 1.0 )
     {
         rcsc::AngleDeg body_angle = param.body_;
         rcsc::AngleDeg body_angle_side = body_angle + 90.0;
         double body_x = body_angle.cos();
         double body_y = body_angle.sin();
-        double forward_x = rcsc::ServerParam::i().tackleDist() * body_x;
-        double forward_y = rcsc::ServerParam::i().tackleDist() * body_y;
-        double back_x = rcsc::ServerParam::i().tackleBackDist() * -body_x;
-        double back_y = rcsc::ServerParam::i().tackleBackDist() * -body_y;
-        double right_x = rcsc::ServerParam::i().tackleWidth() * body_angle_side.cos();
-        double right_y = rcsc::ServerParam::i().tackleWidth() * body_angle_side.sin();
+        double forward_x = SP.tackleDist() * body_x;
+        double forward_y = SP.tackleDist() * body_y;
+        double back_x = SP.tackleBackDist() * -body_x;
+        double back_y = SP.tackleBackDist() * -body_y;
+        double right_x = SP.tackleWidth() * body_angle_side.cos();
+        double right_y = SP.tackleWidth() * body_angle_side.sin();
 
         QPoint pts[5];
-        pts[0].setX( vconf.absScreenX( ppos.x + forward_x + right_x ) );
-        pts[0].setY( vconf.absScreenY( ppos.y + forward_y + right_y ) );
-        pts[1].setX( vconf.absScreenX( ppos.x + forward_x - right_x ) );
-        pts[1].setY( vconf.absScreenY( ppos.y + forward_y - right_y ) );
-        pts[2].setX( vconf.absScreenX( ppos.x + back_x - right_x ) );
-        pts[2].setY( vconf.absScreenY( ppos.y + back_y - right_y ) );
-        pts[3].setX( vconf.absScreenX( ppos.x + back_x + right_x ) );
-        pts[3].setY( vconf.absScreenY( ppos.y + back_y + right_y ) );
+        pts[0].setX( opt.absScreenX( ppos.x + forward_x + right_x ) );
+        pts[0].setY( opt.absScreenY( ppos.y + forward_y + right_y ) );
+        pts[1].setX( opt.absScreenX( ppos.x + forward_x - right_x ) );
+        pts[1].setY( opt.absScreenY( ppos.y + forward_y - right_y ) );
+        pts[2].setX( opt.absScreenX( ppos.x + back_x - right_x ) );
+        pts[2].setY( opt.absScreenY( ppos.y + back_y - right_y ) );
+        pts[3].setX( opt.absScreenX( ppos.x + back_x + right_x ) );
+        pts[3].setY( opt.absScreenY( ppos.y + back_y + right_y ) );
         pts[4] = pts[0];
 
         painter.setPen( dconf.tackleAreaPen() );
@@ -752,20 +843,194 @@ PlayerPainter::drawControlArea( QPainter & painter,
         painter.setFont( dconf.playerFont() );
         painter.setPen( dconf.tacklePen() );
 
-        char msg[32];
-        std::snprintf( msg, 32, "TackleProb=%.3f", 1.0 - tackle_prob );
+        //snprintf( msg, 32, "TackleProb=%.3f", 1.0 - tackle_prob );
         painter.drawText( param.x_ + text_radius,
                           param.y_ - text_radius + painter.fontMetrics().height(),
-                          QString::fromAscii( msg ) );
+                          QString( "TackleProb=%1" ).arg( 1.0 - tackle_prob, 0, 'g', 3 ) );
     }
 }
 
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
 void
-PlayerPainter::drawPointTo( QPainter & painter,
+PlayerPainter::drawKickAccelArea( QPainter & painter,
+                                  const PlayerPainter::Param & param ) const
+{
+    if ( ! param.ball_.hasDelta() )
+    {
+        return;
+    }
+
+    const rcsc::ServerParam & SP = rcsc::ServerParam::i();
+    const Options & opt = Options::instance();
+    const DrawConfig & dconf = DrawConfig::instance();
+
+    rcsc::Vector2D ppos( param.player_.x(), param.player_.y() );
+    rcsc::Vector2D bpos( param.ball_.x(), param.ball_.y() );
+    rcsc::Vector2D player_to_ball = bpos - ppos;
+    player_to_ball.rotate( - param.player_.body() );
+
+    rcsc::Vector2D bnext( bpos.x + param.ball_.deltaX(),
+                          bpos.y + param.ball_.deltaY() );
+    double ball_dist = player_to_ball.r();
+
+    if ( ball_dist < param.player_type_.kickableArea() )
+    {
+        double max_kick_accel
+            = SP.maxPower()
+            * SP.kickPowerRate()
+            * ( 1.0 - 0.25 * player_to_ball.th().abs() / 180.0
+                - 0.25
+                * ( ball_dist - param.player_type_.playerSize() - SP.ballSize() )
+                / param.player_type_.kickableMargin() );
+
+        if ( max_kick_accel > SP.ballAccelMax() )
+        {
+            max_kick_accel = SP.ballAccelMax();
+        }
+
+        QPoint bpos_screen( opt.screenX( bpos.x ),
+                            opt.screenY( bpos.y ) );
+        QPoint bnext_screen( opt.screenX( bnext.x ),
+                             opt.screenY( bnext.y ) );
+        int max_speed_screen = opt.scale( SP.ballSpeedMax() );
+        int max_kick_accel_screen = opt.scale( max_kick_accel );
+
+        painter.setPen( dconf.kickAccelPen() );
+        painter.setBrush( dconf.transparentBrush() );
+
+        // draw no noise next ball move only by inertia
+        painter.drawLine( bpos_screen, bnext_screen );
+
+        rcsc::Circle2D max_speed_circle( bpos, SP.ballSpeedMax() );
+        rcsc::Circle2D max_accel_circle( bnext, max_kick_accel );
+        rcsc::Vector2D intersection_1, intersection_2;
+
+        if ( max_speed_circle.intersection( max_accel_circle,
+                                            &intersection_1,
+                                            &intersection_2 ) != 2 )
+
+        {
+            // no intersection points
+
+            // just draw a next ball reachable area by max accel
+            painter.drawEllipse( bnext_screen.x() - max_kick_accel_screen,
+                                 bnext_screen.y() - max_kick_accel_screen,
+                                 max_kick_accel_screen * 2,
+                                 max_kick_accel_screen * 2 );
+        }
+        else
+        {
+            // exists 2 intersection points
+
+            rcsc::AngleDeg bpos_to_intersection_1 = ( intersection_1 - bpos ).th();
+            rcsc::AngleDeg bpos_to_intersection_2 = ( intersection_2 - bpos ).th();
+
+            rcsc::AngleDeg bpos_to_bnext_angle = ( bnext - bpos ).th();
+
+            rcsc::AngleDeg * bpos_start_angle = 0;
+            double bpos_angle_span = 0.0;
+            if ( bpos_to_intersection_1.isLeftOf( bpos_to_bnext_angle ) )
+            {
+                bpos_start_angle = &bpos_to_intersection_1;
+                bpos_angle_span = ( bpos_to_intersection_2 - bpos_to_intersection_1 ).degree();
+                if ( bpos_angle_span < 0.0 )
+                {
+                    bpos_angle_span += 360.0;
+                }
+                bpos_angle_span *= -1.0;
+            }
+            else
+            {
+                bpos_start_angle = &bpos_to_intersection_2;
+                bpos_angle_span = ( bpos_to_intersection_1 - bpos_to_intersection_2 ).degree();
+                if ( bpos_angle_span < 0.0 )
+                {
+                    bpos_angle_span += 360.0;
+                }
+                bpos_angle_span *= -1.0;
+            }
+
+            if ( opt.reverseSide() )
+            {
+                *bpos_start_angle += 180.0;
+            }
+
+            int bpos_start_angle_int
+                = static_cast< int >( rint( - bpos_start_angle->degree() * 16 ) );
+            int bpos_angle_span_int
+                = static_cast< int >( rint( bpos_angle_span * 16 ) );
+            painter.drawArc( bpos_screen.x() - max_speed_screen,
+                             bpos_screen.y() - max_speed_screen,
+                             max_speed_screen * 2,
+                             max_speed_screen * 2,
+                             bpos_start_angle_int,
+                             bpos_angle_span_int  );
+
+            rcsc::AngleDeg bnext_to_intersection_1 = ( intersection_1 - bnext ).th();
+            rcsc::AngleDeg bnext_to_intersection_2 = ( intersection_2 - bnext ).th();
+
+            rcsc::AngleDeg bnext_to_bpos_angle = bpos_to_bnext_angle + 180.0;
+
+            rcsc::AngleDeg * bnext_start_angle = 0;
+            double bnext_angle_span = 0.0;
+            if ( bnext_to_intersection_1.isLeftOf( bnext_to_bpos_angle ) )
+            {
+                bnext_start_angle = &bnext_to_intersection_1;
+                bnext_angle_span = ( bnext_to_intersection_2 - bnext_to_intersection_1 ).degree();
+                if ( bnext_angle_span < 0.0 )
+                {
+                    bnext_angle_span += 360.0;
+                }
+                bnext_angle_span *= -1.0;
+            }
+            else
+            {
+                bnext_start_angle = &bnext_to_intersection_2;
+                bnext_angle_span = ( bnext_to_intersection_1 - bnext_to_intersection_2 ).degree();
+                if ( bnext_angle_span < 0.0 )
+                {
+                    bnext_angle_span += 360.0;
+                }
+                bnext_angle_span *= -1.0;
+            }
+
+            if ( opt.reverseSide() )
+            {
+                *bnext_start_angle += 180.0;
+            }
+
+            int bnext_start_angle_int
+                = static_cast< int >( rint( - bnext_start_angle->degree() * 16 ) );
+            int bnext_angle_span_int
+                = static_cast< int >( rint( bnext_angle_span * 16 ) );
+            painter.drawArc( bnext_screen.x() - max_kick_accel_screen,
+                             bnext_screen.y() - max_kick_accel_screen,
+                             max_kick_accel_screen * 2,
+                             max_kick_accel_screen * 2,
+                             bnext_start_angle_int,
+                             bnext_angle_span_int );
+        }
+
+        // draw kick info text
+        painter.setFont( dconf.playerFont() );
+        painter.setPen( dconf.kickAccelPen() );
+
+        //snprintf( buf, 32, "MaxAccel=%.3f", max_kick_accel );
+        painter.drawText( bnext_screen.x() + 10,
+                          bnext_screen.y() + painter.fontMetrics().ascent(),
+                          QString( "MaxAccel=%1" ).arg( max_kick_accel, 0, 'g', 4 ) );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+void
+PlayerPainter::drawPointto( QPainter & painter,
                             const PlayerPainter::Param & param ) const
 {
     if ( ! param.player_.isPointing() )
@@ -773,8 +1038,8 @@ PlayerPainter::drawPointTo( QPainter & painter,
         return;
     }
 
-    int px = M_main_data.viewConfig().screenX( param.player_.data().point_x_ );
-    int py = M_main_data.viewConfig().screenX( param.player_.data().point_y_ );
+    int px = Options::instance().screenX( param.player_.data().point_x_ );
+    int py = Options::instance().screenY( param.player_.data().point_y_ );
 
     painter.setPen( DrawConfig::instance().pointtoPen() );
     painter.setBrush( DrawConfig::instance().transparentBrush() );
@@ -788,31 +1053,103 @@ PlayerPainter::drawPointTo( QPainter & painter,
 /*-------------------------------------------------------------------*/
 /*!
 
-*/
+ */
+void
+PlayerPainter::drawAttentionto( QPainter & painter,
+                                const PlayerPainter::Param & param ) const
+{
+    if ( ! param.player_.isFocusing() )
+    {
+        return;
+    }
+
+    MonitorViewConstPtr view = M_main_data.getCurrentViewData();
+
+    if ( ! view )
+    {
+        return;
+    }
+
+    const Options & opt = Options::instance();
+
+    const rcsc::SideID self_side = ( Options::instance().reverseSide()
+                                     ? static_cast< rcsc::SideID >( - param.player_.side() )
+                                     : param.player_.side() );
+
+    const rcsc::SideID side = ( param.player_.data().focus_side_ == 'l'
+                                ? rcsc::LEFT
+                                : rcsc::RIGHT );
+    const int unum = param.player_.data().focus_unum_;
+
+    const std::vector< Player >::const_iterator end = view->players().end();
+    for ( std::vector< Player >::const_iterator p = view->players().begin();
+          p != end;
+          ++p )
+    {
+        if ( p->side() == side
+             && p->unum() == unum )
+        {
+            if ( self_side == rcsc::LEFT )
+            {
+                painter.setPen( DrawConfig::instance().leftGoaliePen() );
+            }
+            else
+            {
+                painter.setPen( DrawConfig::instance().rightGoaliePen() );
+            }
+            //painter.setPen( DrawConfig::instance().attentiontoPen() );
+            painter.setBrush( DrawConfig::instance().transparentBrush() );
+
+            int px = opt.screenX( p->x() );
+            int py = opt.screenY( p->y() );
+            //int r = param.draw_radius_ + 3;
+            int r = opt.scale( 2.0 );
+            int lx = px;
+            int ly = py;
+
+            rcsc::Vector2D rel( param.player_.x(),
+                                param.player_.y() );
+            rel.x -= p->x();
+            rel.y -= p->y();
+
+            double len = rel.r();
+            if ( len > std::pow( 2.0, 2 ) )
+            {
+                rel *= 2.0 / len;
+                lx = opt.screenX( p->x() + rel.x );
+                ly = opt.screenY( p->y() + rel.y );
+            }
+
+            painter.drawLine( param.x_, param.y_, lx, ly );
+            painter.drawEllipse( px - r, py - r, r * 2, r * 2 );
+            break;
+        }
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
 void
 PlayerPainter::drawText( QPainter & painter,
                          const PlayerPainter::Param & param ) const
 {
-    const ViewConfig & vconf = M_main_data.viewConfig();
+    const Options & opt = Options::instance();
     const DrawConfig & dconf = DrawConfig::instance();
 
     const int text_radius = std::min( 40, param.draw_radius_ );
 
-    if ( vconf.isShownPlayerNumber()
-         && vconf.isShownHeteroNumber() )
+    if ( opt.showPlayerNumber()
+         && opt.showPlayerType() )
     {
-        char buf[32];
-        std::snprintf( buf, 32,
-                       "%d,t%d",
-                       param.player_.unum(), param.player_.type() );
-
         painter.setPen( dconf.playerNumberFontPen() );
         painter.setFont( dconf.playerFont() );
         painter.drawText( param.x_ + text_radius,
                           param.y_ + 4,
-                          QString::fromAscii( buf ) );
+                          QString( "%1,t%2" ).arg( param.player_.unum() ).arg( param.player_.type() ) );
     }
-    else if ( vconf.isShownPlayerNumber() )
+    else if ( opt.showPlayerNumber() )
     {
         painter.setPen( dconf.playerNumberFontPen() );
         painter.setFont( dconf.playerFont() );
@@ -820,47 +1157,49 @@ PlayerPainter::drawText( QPainter & painter,
                           param.y_ + 4,
                           QString::number( param.player_.unum() ) );
     }
-    else if ( vconf.isShownHeteroNumber() )
+    else if ( opt.showPlayerType() )
     {
-        char buf[32];
-        std::snprintf( buf, 32,
-                       "t%d",
-                       param.player_.type() );
         painter.setPen( dconf.playerNumberFontPen() );
         painter.drawText( param.x_ + text_radius,
                           param.y_ + 4,
-                          QString::fromAscii( buf ) );
+                          QString( "t%1" ).arg( param.player_.type() ) );
     }
 
-    if ( param.player_.hasStamina()
-         && vconf.isShownStamina() )
+    if ( param.player_.hasStamina() )
     {
-        // this player is selected
-        if ( vconf.isSelectedPlayer( param.player_.side(),
-                                     param.player_.unum() ) )
+        QString str;
+        if ( opt.showStamina() )
         {
-            char buf[32];
-            std::snprintf( buf, 32,
-                           "%4.0f,e%.3f,r%.3f",
-                           param.player_.stamina(),
-                           param.player_.effort(),
-                           param.player_.recovery() );
-            painter.setPen( dconf.playerNumberFontPen() );
-            painter.drawText( param.x_ - text_radius,
-                              param.y_ - text_radius - 3,
-                              QString::fromAscii( buf ) );
+            str += QString::number( static_cast< int >( rint( param.player_.stamina() ) ) );
         }
-        else
-        {
-            char buf[16];
-            std::snprintf( buf, 16, "%4d",
-                           static_cast< int >( rint( param.player_.stamina() ) ) );
 
-            painter.setPen( DrawConfig::instance().playerStaminaFontPen() );
-            painter.setFont( DrawConfig::instance().playerFont() );
+        if ( opt.showStaminaCapacity()
+             && param.player_.hasStaminaCapacity() )
+        {
+            if ( ! str.isEmpty() ) str += '/';
+            str += QString::number( static_cast< int >( rint( param.player_.staminaCapacity() ) ) );
+        }
+
+        if ( ! str.isEmpty() )
+        {
+            // this player is selected
+            if ( opt.isSelectedPlayer( param.player_.side(),
+                                       param.player_.unum() ) )
+            {
+                str += QString( ",e%1,r%2" )
+                    .arg( param.player_.effort(), 0, 'g', 3 )
+                    .arg( param.player_.recovery(), 0, 'g', 3 );
+                painter.setPen( dconf.playerNumberFontPen() );
+            }
+            else
+            {
+                painter.setPen( dconf.playerStaminaFontPen() );
+            }
+
+            painter.setFont( dconf.playerFont() );
             painter.drawText( param.x_ - text_radius,
                               param.y_ - text_radius - 3,
-                              QString::fromAscii( buf ) );
+                              str );
         }
     }
 }
